/******************************************************************************* 
 * emulator_implementation.cpp - SNES9x PS3
 *
 *  Created on: Oct 15, 2010
 *      Author: Squarepusher2 
 *	Author: Themaister
********************************************************************************/
#include <sysutil/sysutil_msgdialog.h>

#include "snes9x.h"
#include "memmap.h"
#include "apu/apu.h"
#include "gfx.h"
#include "snapshot.h"
#include "controls.h"
#include "cheats.h"
#include "movie.h"
#include "logger.h"
#include "display.h"
#include "conffile.h"
#include "ctype.h"

#include "cellframework/logger/Logger.h"

#include "emulator_implementation.h"
#include "main.h"
#include "SNES9xGraphics.h"

ConfigFile	currentconfig;
			
typedef std::pair<int *, int> ControlSchemeLayout;

bool dialog_is_running;

// current save state id
unsigned int current_save_state = 0;
uint32_t current_cheat_position = 0;

unsigned int Emulator_Implementation_CurrentSaveStateSlot()
{
	return current_save_state;
}

void Emulator_Implementation_DecrementCurrentSaveStateSlot()
{
	if(Emulator_Implementation_CurrentSaveStateSlot() > 0)
	{
		current_save_state--;
	}
}

void Emulator_Implementation_IncrementCurrentSaveStateSlot()
{
	if(Emulator_Implementation_CurrentSaveStateSlot() < 9)
	{
		current_save_state++;
	}
}

uint32_t Emulator_Implementation_CurrentCheatPosition()
{
	return current_cheat_position;
}

static void cb_dialog_ok(int button_type, void *userdata)
{
	switch(button_type)
	{
		case CELL_MSGDIALOG_BUTTON_ESCAPE:
		dialog_is_running = false;
		break;
	}
}

void Emulator_Implementation_AddCheat(uint32_t address, uint8_t byte, const char *description)
{
	S9xAddCheat(false, true, address, byte);
	S9xEnableCheat(Cheat.num_cheats - 1);
	strncpy(Cheat.c[Cheat.num_cheats - 1].name, description, 22);
	
}

char * Emulator_Implementation_InputCheat()
{
	LOG_DBG("Emulator_Implementation_InputCheat()\n");
	// (1) Let the user enter his cheat code
	int ret;
	ret = Emulator_OSKStart(L"Enter cheat code (Game Genie/Action Replay/GoldFinger Pro format)",L"");

	while(Emulator_OSKIsRunning() && Emulator_ROMRunning())
	{
		//OSK Util gets updated
		S9xMainLoop();
		cellSysutilCheckCallback();	
	}

	// (2) Save the code to a char variable
	const char * newcode = new char[128];
	newcode = strdup(Emulator_OSKOutputString());

	// Close OSKUtil - hopefully to free up memory
	Emulator_OSKClose();

	// (3) Check the cheat format of the cheat code
	uint32 address;
	uint8 byte;
	uint8 bytes [3];
	bool8 sram;
	uint8 num_bytes;

	LOG_DBG("I was here - before !S9xGameGenieToRaw\n");
	const char * newlabel = new char[22];
	if (!S9xGameGenieToRaw (newcode, address, byte))
	{
		// (4) Let the user enter a label name for his Game Genie cheat code, make a function call
		LOG_DBG("Emulator_Implementation_InputLabel()\n");

		//NOTE - See OSKUtil.cpp for more information on this - this was a hack to compensate
		//for CELL_SYSUTIL_ERROR_BUSY - problem was that having two OSK utility tools succeed
		//each other in quick succession was causing problems because the other one had not
		//even been properly deinited yet - hence this delay here is necessary
	
		//add delay because of the above
		sys_timer_usleep(700000);
		Emulator_OSKStart(L"Enter cheat label",L"");
		while(Emulator_OSKIsRunning() && Emulator_ROMRunning())
		{
			//OSK Util gets updated
			S9xMainLoop();
			cellSysutilCheckCallback();	
		}
		newlabel = strdup(Emulator_OSKOutputString());

		// (5) add Game Genie cheat code
		Emulator_Implementation_AddCheat(address, byte, newlabel);

		// (6) Close OSKUtil - hopefully to free up memory
		Emulator_OSKClose();

		// (7) Return message text string
		return "Game Genie code added.";	
	}
	else if (!S9xProActionReplayToRaw(newcode, address, byte))
	{
		// (4) Let the user enter a label name for his Pro Action Replay cheat code, make a function call
		LOG_DBG("Emulator_Implementation_InputLabel()\n");

		//NOTE - See OSKUtil.cpp for more information on this - this was a hack to compensate
		//for CELL_SYSUTIL_ERROR_BUSY - problem was that having two OSK utility tools succeed
		//each other in quick succession was causing problems because the other one had not
		//even been properly deinited yet - hence this delay here is necessary
	
		//add delay because of the above
		sys_timer_usleep(700000);
		int ret = Emulator_OSKStart(L"Enter cheat label",L"");
		if(ret != 1)
		{
			Emulator_OSKClose();
			return "ERROR - Cheat added but couldn't input label due to bug. Restart SNES9x PS3.";
		}
		while(Emulator_OSKIsRunning() && Emulator_ROMRunning())
		{
			//OSK Util gets updated
			S9xMainLoop();
			cellSysutilCheckCallback();	
		}
		newlabel = strdup(Emulator_OSKOutputString());

		// (5) add Pro Action Replay cheat code
		Emulator_Implementation_AddCheat(address, byte, newlabel);

		// (6) Close OSKUtil - hopefully to free up memory
		Emulator_OSKClose();

		// (7) Return message text string
		return "Pro Action Replay code added.";	
	}
	else if (!S9xGoldFingerToRaw(newcode, address, sram, num_bytes, bytes))
	{
		// (4) Let the user enter a label name for his Gold Finger cheat code, make a function call
		LOG_DBG("Emulator_Implementation_InputLabel()\n");

		//NOTE - See OSKUtil.cpp for more information on this - this was a hack to compensate
		//for CELL_SYSUTIL_ERROR_BUSY - problem was that having two OSK utility tools succeed
		//each other in quick succession was causing problems because the other one had not
		//even been properly deinited yet - hence this delay here is necessary
	
		//add delay because of the above
		sys_timer_usleep(700000);
		Emulator_OSKStart(L"Enter cheat label",L"");
		while(Emulator_OSKIsRunning() && Emulator_ROMRunning())
		{
			//OSK Util gets updated
			S9xMainLoop();
			cellSysutilCheckCallback();	
		}
		newlabel = strdup(Emulator_OSKOutputString());

		// (5) add Gold Finger cheat code
		for (int c = 0; c < num_bytes; c++)
		{
			Emulator_Implementation_AddCheat(address + c, bytes[c], newlabel);
		}
		// (6) Close OSKUtil - hopefully to free up memory
		Emulator_OSKClose();

		// (7) Return message text string
		return "Gold Finger code added.";	
	}
	else
	{
		// (4) Close OSKUtil - hopefully to free up memory
		Emulator_OSKClose();

		// (5) Return message text string
		return "ERROR: Code entered was incorrect.";
	}

}

void Emulator_Implementation_DecrementCurrentCheatPosition()
{
	if(Emulator_Implementation_CurrentCheatPosition() > 0)
	{
		current_cheat_position--;
	}
}

void Emulator_Implementation_IncrementCurrentCheatPosition()
{
		current_cheat_position++;
}

void Emulator_Implementation_SetCurrentSaveStateSlot(int set_current_save_state_slot)
{
	if(set_current_save_state_slot <= 9)
	{
		current_save_state = set_current_save_state_slot;
	}
}

static std::auto_ptr<Threads::Mutex> audio_lock(new Threads::Mutex);
static bool audio_active = false;
ssize_t Emulator_Implementation_AudioCallback(int16_t *out, size_t frames, void *data)
{
   Threads::Mutex *lock = reinterpret_cast<Threads::Mutex*>(data);
   Threads::ScopedLock foo(*lock);
   if (audio_active)
   {
      S9xMixSamples((uint8_t*)out, frames * 2);
      return frames;
   }
   else
      return 0;
}

void Emulator_Implementation_S9xAudioCallback(void *data)
{
   Threads::Mutex *lock = reinterpret_cast<Threads::Mutex*>(data);
   Threads::ScopedLock foo(*lock);
   S9xFinalizeSamples();
}

bool Emulator_Implementation_IsSuperScopeCompatible()
{
	if	(
		Memory.match_na("BATTLE CLASH") ||			// Battle Clash (EU/US) (*)
									// Space Bazooka (JP)

		Memory.match_na("SPACE BAZOOKA") ||			// Space Bazooka (JP) (*)
									// Battle Clash (EU/US)

		//FIXME: TODO: Add The Hunt for Red October - used for bonus game
		//FIXME: TODO: Add Lamborghini American Challenge - if needed
		Memory.match_na("METAL COMBAT") ||			// Metal Combat: Falcon's Revenge
		Memory.match_na("OPERATION THUNDERBOLT1") ||		// Operation Thunderbolt
		Memory.match_na("SUPER SCOPE 6") ||			// Super Scope 6
		//Memory.match_na("TERMINATOR2 THE MOVIE0") ||		// Terminator 2: The Arcade Game

		Memory.match_na("TINSTAR") ||				// Tin Star
		Memory.match_id("9N") ||

		Memory.match_na("X ZONE") ||				// X-Zone

		//FIXME: TODO: Add Yoshi no Road Hunting - CRC32: 52948F3C
		Memory.match_na("YOSHI'S SAFARI")			// Yoshi's Safari
		)
		{
			return true;
		}
		else
		{
			return false;
		}
		
}

bool Emulator_Implementation_IsJustifierCompatible()
{
	if(Memory.match_nc("LETHAL ENFORCERS"))			// Lethal Enforcers
	{
		return true;
	}
	else
	{
		return false;
	}
}

bool Emulator_Implementation_IsMouseCompatible()
{
	if	(
		Memory.match_nc("ACME ANIMATION FACTOR") ||		// ACME Animation Factory
		Memory.match_id("ACM") ||
		Memory.match_nc("ALICE PAINT") ||			// Alice no Paint Adventure
		Memory.match_id("AALJ") ||
		Memory.match_nc("ARKANOID DOH IT AGAIN0") ||		// Arkanoid: Doh It Again
		Memory.match_id("A6") ||
		Memory.match_na("SFC SAILORMOON S") ||			// Bishojou Senshi Sailor Moon S: Kondowa Puzzle de
									// Oshiokiyo!

		Memory.match_na("BRANDISH2 EXPERT") ||			// Brandish 2: Expert
		Memory.match_id("AQIJ") ||				

		Memory.match_na("BREAKTRHU!") ||			// BreakThru!
		Memory.match_id("ABXE") ||

		Memory.match_na("CIVILIZATION") ||			// Civilization
		Memory.match_id("EQ") ||

		Memory.match_na("CAMELTRY") ||				// Cameltry (JP) (*)
									// On The Ball (EU/US)


		Memory.match_na("CANNON FODDER") ||			// Cannon Fodder
		Memory.match_id("ACNP") ||				

		Memory.match_id("M Ve") ||				// Dai-3-ji Super Robot Taisen

		Memory.match_id("AR4J") ||				// Dai-4-ji Super Robot Taisen
		Memory.match_na("SUPER ROBOT WARS 4") ||

		Memory.match_na("DOOM") ||				// Doom
		Memory.match_id("AD8") ||				

		Memory.match_id("A88J") ||				// Doukyuusei 2 (JP) (NP)

		Memory.match_na("DRAGON KNIGHT 4") ||			// Dragon Knight 4
		Memory.match_id("A87J") ||				

		Memory.match_na("Eye of the Beholder") ||		// Eye of the Beholder
	
		Memory.match_na("FARLANDSTORY2") ||			// Farland Story 2
		Memory.match_id("A2FJ") ||

		Memory.match_na("Fun 'N Games") ||			// Fun 'N Games (EU/US)
		Memory.match_na("GALAXY ROBO") ||			// Galaxy Robo
		Memory.match_na("HIOUDEN") ||				// Hiouden: Mamono-tachi tono Chikai

		Memory.match_na("JIGSAW PARTY") ||			// Jigsaw Party (US) (*)
									// Pieces (JP)

		//Memory.match_na("JURASSIC PARK") ||			// Jurassic Park

		Memory.match_na("King Arthurs World") ||		// King Arthur's World (EU/US) (*)
									// Royal Conquest (JP)
		Memory.match_na("KOUTETSU-NO KISHI") ||			// Koutetsu No Kishi
		Memory.match_na("KOUTETSU-NO KISHI 2") ||		// Koutetsu No Kishi 2

		Memory.match_na("KOUTETSU-NO KISHI 3") ||		// Koutetsu No Kishi 3
		Memory.match_id("APZJ") ||

		Memory.match_na("LAMBORGHINI AMERICAN") ||		// Lamborghini American Challenge (EU/US)

		Memory.match_na("LAS VEGAS DREAM") ||			// Las Vegas Dream in Golden Paradise (JP) (*)
									// Vegas Stakes (EU/US)

		Memory.match_na("Lemmings 2,The Tribes") ||		// Lemmings 2: The Tribes
		Memory.match_id("L2") ||

		Memory.match_na("LORD MONARCH") ||			// Lord Monarch
		Memory.match_na("Mario&Wario") ||			// Mario & Wario

		Memory.match_na("SUPER PICROSS") ||			// Mario no Super Picross
		Memory.match_id("ACXJ") ||

		Memory.match_nc("MARIOPAINT") ||			// Mario Paint

		Memory.match_na("MEY  Preschool Fun") ||		// Mario's Early Years: Preschool Fun
		Memory.match_id("AMEE") ||

		Memory.match_na("MEGA LO MANIA") ||			// Mega-Lo Mania
		Memory.match_na("MIGHT  AND MAGIC III") ||		// Might And Magic III - Isles of Terra

		Memory.match_na("WONDER KITCHEN") ||			// Motoko-chan no Wonder Kitchen

		Memory.match_na("Nobunaga's Ambition") ||		// Nobunaga's Ambition (US) (*)
									// Super Nobunaga no Yabou: Zenkokuban (JP)

		Memory.match_na("ON THE BALL") ||			// On The Ball (EU/US) (*)
									// Cameltry (JP)

		Memory.match_na("OPERATION THUNDERBOLT1") ||		// Operation Thunderbolt

		Memory.match_na("PIECES") ||
		Memory.match_id("Z5") ||				// Pieces (JP) (*)
									// Jigsaw Party (US)

		Memory.match_nc("POPULOUS II") ||			// Populous II
		Memory.match_na("POWERMONGER") ||			// Power Monger

		Memory.match_na("Revolution X") ||			// Revolution X
		Memory.match_id("AXR") ||

		Memory.match_na("Royal Conquest") ||			// Royal Conquest (JP) (*)
									// King Arthur's World (EU/US)

		Memory.match_na("TENBU SPIRIT") ||			// San Goku Shi Seishi: Tenbu Spirits

		Memory.match_na("SHIEN THE BLADE CHASE0") ||		// Shien The Blade Chaser (JP) (*)
		Memory.match_na("SHIEN'S REVENGE") ||			// Shien's Revenge (*)

		Memory.match_nc("SimAnt") ||				// SimAnt
		Memory.match_na("SNOOPY CONCERT") ||			// Snoopy Concert
		Memory.match_na("SUPER CAESARS PALACE") ||		// Super Caesar's Palace

		Memory.match_na("SUPER CASTLES") ||			// Super Castles
		Memory.match_id("AVIJ") ||

		//FIXME: TODO: Super Noah's Ark 3D: - bad checksum - CRC32: A2315A14

		Memory.match_na("SuperPachiSlotMahjong") ||		// Super Pachi Slot Mahjong
		Memory.match_na("SUPER SOLITAIRE") ||			// Super Solitaire

		Memory.match_na("SUPER ZENKOKUBAN") ||			// Super Nobunaga no Yabou: Zenkokuban (JP) (*)
									// Nobunaga's Ambition (US)
		Memory.match_na("TERMINATOR2 THE MOVIE0") ||		// Terminator 2: The Arcade Game
	
		Memory.match_na("TINSTAR") ||				// Tin Star
		Memory.match_id("9N") ||

		Memory.match_na("TOKIMEKI MEMORIAL") ||			// Tokimeki Memorial
		Memory.match_id("AM8J") ||

		Memory.match_na("Troddlers") ||				// Troddlers
		Memory.match_na("UTOPIA") ||				// Utopia

		Memory.match_na("VEGAS STAKES") ||			// Vegas Stakes (EU/US) (*)
									// Las Vegas Dream in Golden Paradise (JP)
		Memory.match_nc("WOLFENSTEIN 3D") ||			// Wolfenstein 3D
		
		Memory.match_na("WONDERPROJECT J") ||			// Wonder Project J
		Memory.match_id("APJJ") ||				

		Memory.match_na("ZAN2 SPIRITS") ||			// Zan 2: Spirits
		Memory.match_na("ZAN3 SFC")				// Zan 3: Spirits
		)
		{
			return true;
		}
		else
		{
			return false;
		}
}

bool Emulator_Implementation_IsMultitapCompatible()
{
	if 	(
		Memory.match_nn("BARKLEY") ||				// Barkley Shut Up and Jam!
		Memory.match_id("ABCJ") ||				// Battle Cross
		Memory.match_na("Bill Walsh College FB1") ||		// Bill Walsh College Football
		Memory.match_id("ANYE") ||				// College Slam
		Memory.match_id("AC3J") ||				// Crystal Beans from Dungeon Explorer
		Memory.match_nc("Bruce Lee") ||				// Dragon - Bruce Lee Story

		Memory.match_id("AVS") ||				// Fever Pitch Soccer (EU) (*)
									// Head-On Soccer (US) (*)

		Memory.match_na("FURI-FURI GIRLS") ||			// Furi-Furi Girls

		Memory.match_na("HAT TRICK HERO 2") ||			// Hat Trick Hero 2
		Memory.match_na("HEBEREKE NO PUZZLE") ||		// Hebereke no Oishii Puzzle wa Irimasenka

		Memory.match_id("AWJ") ||				// International Superstar Soccer Deluxe (EU/US) (*)
									// Jikkyou World Soccer 2 - Fighting Eleven (JP) (*)

		Memory.match_na("J-LEAGUE SUPER SOCCER0") ||		// J-League Super Soccer (JP)
									// Virtual Soccer (EU)

		Memory.match_na("JIGSAW PARTY") ||			// Jigsaw Party (US) (*)
									// Pieces (JP)

		Memory.match_na("JIMMY CONNORS TENNIS") ||		// Jimmy Connors Pro Tennis Tour

		Memory.match_na("LAS VEGAS DREAM") ||			// Las Vegas Dream in Golden Paradise (JP) (*)
									// Vegas Stakes (EU/US)

		Memory.match_id("ALT") ||				// Looney Tunes B-Ball (US) (*)
									// Looney Tunes Basketball (EU) (*)
		
		Memory.match_id("AYHJ") ||				// Mizuki Shigeru no Youkai Hyakkiyakou
		Memory.match_id("A3VJ") ||				// Multi Play Volleyball

		Memory.match_na("MUSCLE BOMBER") ||			// Muscle Bomber: The Body Explosion (JP) (*)
									// Saturday Night Slam Masters (US)

		Memory.match_id("CHAMP WRESTLING") ||			// Natsume Championship Wrestling
		Memory.match_id("ANJ") || 				// NBA Give 'n Go
		Memory.match_id("AXG") || 				// NBA Hangtime
		Memory.match_id("AJT") || 				// NBA Jam Tournament Edition (EU/US/JP)
		Memory.match_na("NBA JAM")   || 			// NBA Jam
		Memory.match_id("AFIE") ||				// NCAA Final Four Basketball
		Memory.match_id("AFBE") ||				// NCAA Football
		Memory.match_id("Q9") ||				// NFL Quarterback Club / NFL Quarterback Club '95
		Memory.match_id("AQB") ||				// NFL Quarterback Club '96
		
		Memory.match_na("PIECES") ||
		Memory.match_id("Z5") ||				// Pieces (JP) (*)
									// Jigsaw Party (US)

		Memory.match_id("ARVE") ||				// Rap Jam - Volume One

		Memory.match_na("RUSHING BEAT SYURA") ||		// Rushing Beat Shura (JP) (*)
									// The Peace Keepers (US)

		Memory.match_na("S.Night SLAM MASTERS") ||		// Saturday Night Slam Masters (US) (*)
									// Muscle Bomber: The Body Explosion (JP)

		Memory.match_na("Secret of MANA") ||			// Secret of Mana (EU/US) (*)
									// Seiken Densetsu 2 (JP)
		
		Memory.match_na("SeikenDensetsu 2") ||			// Seiken Densetsu 2 (JP) (*) / 
									// Secret of Mana (EU/US)

		Memory.match_nn("SeikenDensetsu3") ||			// Seiken Densetsu 3 (in case you use
									// 3-player IPS patch)

		Memory.match_na("SMASH TENNIS") ||			// Smash Tennis (EU) (*) / 
									// Super Family Tennis (JP)

		//FIXME: TODO - Add CRC32 for Super Family Tennis - CRC32: 2BCBFF26

		Memory.match_id("ASR") ||				// Street Racer
		Memory.match_na("SUGOI HEBEREKE") ||			// Sugoi Hebereke
		Memory.match_na("Sugoro Quest++") ||			// Sugoro Quest++: Dicenics
		Memory.match_id("AO9") ||				// Summer Olympics
		Memory.match_id("AS6") ||				// Super Bomberman 3
		Memory.match_id("A4BJ") || 				// Super Bomberman 4
		Memory.match_id("APBJ") ||				// Super Bomberman Panic Bomber W
		Memory.match_id("AF5J") ||				// Super Fire Pro Wrestling X
		Memory.match_id("AP4J") ||				// Super Fire Pro Wrestling Special
		Memory.match_id("AQQJ") ||				// Super Fire Pro Wrestling - Queen's Special
		Memory.match_id("A7PJ") ||				// Super Puyo Puyo Tsuu Remix
		Memory.match_id("AT3J") ||				// Super Tetris 3
		Memory.match_id("AFY") ||				// Syndicate

		Memory.match_na("THE PEACE KEEPERS") ||			// The Peace Keepers (US) (*)
									// Rushing Beat Shura (JP)

		Memory.match_na("Tiny Toon Sports") ||			// Tiny Toon Adventures: Dotabata Daiundoukai (JP) (*)
									// Tiny Toon Adventures: Wacky Sports Challenge (US) (*)
									// Tiny Toon Adventures: Wild & Wacky Sports (EU) (*)

		Memory.match_id("A3T") ||				// Top Gear 3000 (EU/US) (*)
									// Planet's Champ TG 3000 (JP) (*)

		Memory.match_na("VEGAS STAKES") ||			// Vegas Stakes (EU/US) (*)
									// Las Vegas Dream in Golden Paradise (JP)

		Memory.match_na("Virtual Soccer") ||			// Virtual Soccer (EU) (*)
									// J-League Super Soccer (JP)

		Memory.match_id("AWF") ||				// WWF RAW

		//Catch-alls
		Memory.match_nc("SUPER BOMBERMAN") ||			// All Bomberman games (FIXME: remove
									// individual two Bomberman entries if this works)

		Memory.match_nc("FIFA") ||				// All FIFA games
		Memory.match_nn("NHL") ||				// All NHL games
		Memory.match_nc("MADDEN") ||				// All Madden games
		Memory.match_nc("MICRO MACHINES") ||			// All Micro Machines games

		Memory.match_id("Q4")					// ?
	)
	{
		return true;
		//Settings.CurrentROMisMultitapCompatible = TRUE;
	}
	else
	{
		return false;
		//Settings.CurrentROMisMultitapCompatible = FALSE;
	}
}

void Emulator_Implementation_OddScreenHeightJumpFix()
{
	if 	(
		Memory.match_nc("CHRONO TRIGGER") ||			// Chrono Trigger
		Memory.match_id("ACT") ||
		Memory.match_id("AC9J")					// Chrono Trigger (Sample)
		)
		{
			Settings.OddScreenHeightJumpFix = 0;
		}
		else
		{
			Settings.OddScreenHeightJumpFix = 1;
		}
}


void Emulator_Implementation_ToggleSound()
{
	if(CellAudio)
	{
		delete CellAudio;
	}
	if((Settings.RSoundEnabled) && (strlen(Settings.RSoundServerIPAddress) > 0))
	{
		CellAudio = new Audio::RSound<int16_t>(Settings.RSoundServerIPAddress, 2, AUDIO_INPUT_RATE);
      		CellAudio->set_audio_callback(Emulator_Implementation_AudioCallback, (void*)audio_lock.get());
		// If we couldn't connect, fall back to normal audio...
		if (!CellAudio->alive())
		{
			delete CellAudio;
			CellAudio = new Audio::AudioPort<int16_t>(2, AUDIO_INPUT_RATE);
         		CellAudio->set_audio_callback(Emulator_Implementation_AudioCallback, (void*)audio_lock.get());
			Settings.RSoundEnabled = false;
			dialog_is_running = true;
			cellMsgDialogOpen2(CELL_MSGDIALOG_TYPE_SE_TYPE_ERROR|CELL_MSGDIALOG_TYPE_BG_VISIBLE|\
			CELL_MSGDIALOG_TYPE_BUTTON_TYPE_NONE|CELL_MSGDIALOG_TYPE_DISABLE_CANCEL_OFF|\
			CELL_MSGDIALOG_TYPE_DEFAULT_CURSOR_OK,\
			"Couldn't connect to RSound server at specified IP address. Falling back to\
			 regular audio.",cb_dialog_ok,NULL,NULL);
			while(dialog_is_running && Emulator_ROMRunning())
			{
				Graphics->Clear();
				Graphics->Swap();
				cellSysutilCheckCallback();	
			}
		}
	}
	else
	{
		CellAudio = new Audio::AudioPort<int16_t>(2, AUDIO_INPUT_RATE);
      		CellAudio->set_audio_callback(Emulator_Implementation_AudioCallback, (void*)audio_lock.get());
	}
}

void Emulator_Implementation_SaveSRAM()
{
	if(!Settings.SRAMWriteProtect)
	{
		Memory.SaveSRAM(S9xGetFilename(".srm", SRAM_DIR));
	}
	S9xSaveCheatFile(S9xGetFilename(".cht", CHEAT_DIR));
}

void Emulator_Implementation_SwitchPAL60Hz(bool pal60Hz)
{
	Settings.PS3PALTemporalMode60Hz = pal60Hz;
	Graphics->SetPAL60Hz(Settings.PS3PALTemporalMode60Hz);
	Graphics->SwitchResolution(Graphics->GetCurrentResolution(), Settings.PS3PALTemporalMode60Hz);
}

static void cb_multitap_compatible(int button_type, void *userdata)
{
	switch(button_type)
	{
		case CELL_MSGDIALOG_BUTTON_YES:
		Settings.CurrentROMisMultitapCompatible = true;
		Emulator_Implementation_MuteSound(false);
		dialog_is_running = false;
		cellMsgDialogClose(0.0f);
		break;
		case CELL_MSGDIALOG_BUTTON_ESCAPE:
		case CELL_MSGDIALOG_BUTTON_NO:
		Settings.CurrentROMisMultitapCompatible = false;
		Emulator_Implementation_MuteSound(false);
		dialog_is_running = false;
		cellMsgDialogClose(0.0f);
		break;
	}
}

static void cb_mouse_compatible(int button_type, void *userdata)
{
	switch(button_type)
	{
		case CELL_MSGDIALOG_BUTTON_YES:
		Settings.CurrentROMisMouseCompatible = true;
		Emulator_Implementation_MuteSound(false);
		dialog_is_running = false;
		cellMsgDialogClose(0.0f);
		break;
		case CELL_MSGDIALOG_BUTTON_ESCAPE:
		case CELL_MSGDIALOG_BUTTON_NO:
		Settings.CurrentROMisMouseCompatible = false;
		Emulator_Implementation_MuteSound(false);
		dialog_is_running = false;
		cellMsgDialogClose(0.0f);
		break;
	}
}

static void cb_superscope_compatible(int button_type, void *userdata)
{
	switch(button_type)
	{
		case CELL_MSGDIALOG_BUTTON_YES:
		Settings.CurrentROMisSuperScopeCompatible = true;
		Emulator_Implementation_MuteSound(false);
		dialog_is_running = false;
		cellMsgDialogClose(0.0f);
		break;
		case CELL_MSGDIALOG_BUTTON_ESCAPE:
		case CELL_MSGDIALOG_BUTTON_NO:
		Settings.CurrentROMisSuperScopeCompatible = false;
		Emulator_Implementation_MuteSound(false);
		dialog_is_running = false;
		cellMsgDialogClose(0.0f);
		break;
	}
}

static void cb_justifier_compatible(int button_type, void *userdata)
{
	switch(button_type)
	{
		case CELL_MSGDIALOG_BUTTON_YES:
		Settings.CurrentROMisJustifierCompatible = true;
		Emulator_Implementation_MuteSound(false);
		dialog_is_running = false;
		cellMsgDialogClose(0.0f);
		break;
		case CELL_MSGDIALOG_BUTTON_ESCAPE:
		case CELL_MSGDIALOG_BUTTON_NO:
		Settings.CurrentROMisJustifierCompatible = false;
		Emulator_Implementation_MuteSound(false);
		dialog_is_running = false;
		cellMsgDialogClose(0.0f);
		break;
	}
}

bool Emulator_Implementation_InitSystem()
{
   	audio_lock->lock();
	if(Emulator_IsInitialized())
	{
		Memory.Deinit();
	}
	Memory.Init();

	if(Emulator_IsInitialized)
	{
		S9xDeinitAPU();
	}
	S9xInitAPU();

	S9xInitSound(32, 0);
   	S9xSetSamplesAvailableCallback(Emulator_Implementation_S9xAudioCallback, (void*)audio_lock.get());
   	audio_active = true;
   	audio_lock->unlock();

	Emulator_Implementation_MuteSound(true);
	S9xSetRenderPixelFormat(RGB555);

	if (Emulator_GetROM() == NULL)
	{
		Emulator_SetDebugMessage("No ROM to load");
		return false;
	}

	if (!Memory.LoadROM(Emulator_GetROM())) {
		Emulator_SetDebugMessage("Load ROM failed");
		return false;
	}

	if(Emulator_Implementation_IsMultitapCompatible())
	{
		if(Settings.AccessoryAutoDetection == ACCESSORY_AUTODETECTION_CONFIRM)
		{
			dialog_is_running = true;
			cellMsgDialogOpen2(CELL_MSGDIALOG_DIALOG_TYPE_NORMAL|CELL_MSGDIALOG_TYPE_BG_VISIBLE|\
			CELL_MSGDIALOG_TYPE_BUTTON_TYPE_YESNO|CELL_MSGDIALOG_TYPE_DISABLE_CANCEL_OFF|\
			CELL_MSGDIALOG_TYPE_DEFAULT_CURSOR_YES,\
			"This game supports the [Multitap] accessory. Do you want to enable [Multitap] support?",\
			cb_multitap_compatible,NULL,NULL);
			while(dialog_is_running && Emulator_ROMRunning())
			{
				Emulator_Implementation_MuteSound(true);
				Graphics->Clear();
				Graphics->Swap();
				cellSysutilCheckCallback();	
			}
			LOG_DBG("CurrentROMisMultitapCompatible: %d\n", Emulator_Implementation_IsMultitapCompatible());
		}
		else if(Settings.AccessoryAutoDetection == ACCESSORY_AUTODETECTION_ENABLED)
		{
			Settings.CurrentROMisMultitapCompatible = true;
		}
		else if(Settings.AccessoryAutoDetection == ACCESSORY_AUTODETECTION_NONE)
		{
			Settings.CurrentROMisMultitapCompatible = false;
		}
		if(Settings.CurrentROMisMultitapCompatible)
		{
			Emulator_SetControllerMode(MULTITAP);
		}
	}
	else if(Emulator_Implementation_IsMouseCompatible())
	{
		if(Settings.AccessoryAutoDetection == ACCESSORY_AUTODETECTION_CONFIRM)
		{
			dialog_is_running = true;
			cellMsgDialogOpen2(CELL_MSGDIALOG_DIALOG_TYPE_NORMAL|CELL_MSGDIALOG_TYPE_BG_VISIBLE\
			|CELL_MSGDIALOG_TYPE_BUTTON_TYPE_YESNO|CELL_MSGDIALOG_TYPE_DISABLE_CANCEL_OFF|\
			CELL_MSGDIALOG_TYPE_DEFAULT_CURSOR_YES,\
			"This game supports the [SNES Mouse] accessory. Do you want to enable [SNES Mouse] support? (NOTE: [SNES Mouse] will be mapped to the left analog stick)",cb_mouse_compatible,NULL,NULL);
			while(dialog_is_running && Emulator_ROMRunning())
			{
				Emulator_Implementation_MuteSound(true);
				Graphics->Clear();
				Graphics->Swap();
				cellSysutilCheckCallback();	
			}
			LOG_DBG("CurrentROMisMouseCompatible: %d\n", Emulator_Implementation_IsMouseCompatible());
		}
		else if(Settings.AccessoryAutoDetection == ACCESSORY_AUTODETECTION_ENABLED)
		{
			Settings.CurrentROMisMouseCompatible = true;
		}
		else if(Settings.AccessoryAutoDetection == ACCESSORY_AUTODETECTION_NONE)
		{
			Settings.CurrentROMisMouseCompatible = false;
		}
		if(Settings.CurrentROMisMouseCompatible)
		{
			Emulator_SetControllerMode(MOUSE);
		}
	}
	else if(Emulator_Implementation_IsSuperScopeCompatible())
	{
		if(Settings.AccessoryAutoDetection == ACCESSORY_AUTODETECTION_CONFIRM)
		{
			dialog_is_running = true;
			LOG_DBG("CurrentROMisSuperScopeCompatible: %d\n", Emulator_Implementation_IsSuperScopeCompatible());
			cellMsgDialogOpen2(CELL_MSGDIALOG_DIALOG_TYPE_NORMAL|CELL_MSGDIALOG_TYPE_BG_VISIBLE|\
			CELL_MSGDIALOG_TYPE_BUTTON_TYPE_YESNO|CELL_MSGDIALOG_TYPE_DISABLE_CANCEL_OFF|\
			CELL_MSGDIALOG_TYPE_DEFAULT_CURSOR_YES,\
			"This game supports the [Super Scope] accessory. Do you want to enable [Super Scope] support? (NOTE: [Super Scope] will be mapped to the left analog stick)",\
			cb_superscope_compatible,NULL,NULL);
			while(dialog_is_running && Emulator_ROMRunning())
			{
				Emulator_Implementation_MuteSound(true);
				Graphics->Clear();
				Graphics->Swap();
				cellSysutilCheckCallback();	
			}
		}
		else if(Settings.AccessoryAutoDetection == ACCESSORY_AUTODETECTION_ENABLED)
		{
			Settings.CurrentROMisSuperScopeCompatible = true;
		}
		else if(Settings.AccessoryAutoDetection == ACCESSORY_AUTODETECTION_NONE)
		{
			Settings.CurrentROMisSuperScopeCompatible = false;
		}
		if(Settings.CurrentROMisSuperScopeCompatible)
		{
			Emulator_SetControllerMode(SUPERSCOPE);
		}
	}
	else if(Emulator_Implementation_IsJustifierCompatible())
	{
		if(Settings.AccessoryAutoDetection == ACCESSORY_AUTODETECTION_CONFIRM)
		{
			dialog_is_running = true;
			LOG_DBG("CurrentROMisJustifierCompatible: %d\n", Emulator_Implementation_IsJustifierCompatible());
			cellMsgDialogOpen2(CELL_MSGDIALOG_DIALOG_TYPE_NORMAL|CELL_MSGDIALOG_TYPE_BG_VISIBLE|\
			CELL_MSGDIALOG_TYPE_BUTTON_TYPE_YESNO|CELL_MSGDIALOG_TYPE_DISABLE_CANCEL_OFF|\
			CELL_MSGDIALOG_TYPE_DEFAULT_CURSOR_YES,\
			"This game supports the [Justifier] accessory. Do you want to enable [Justifier] support? (NOTE: [Justifier] will be mapped to the left analog stick)",\
			cb_justifier_compatible,NULL,NULL);
			while(dialog_is_running && Emulator_ROMRunning())
			{
				Emulator_Implementation_MuteSound(true);
				Graphics->Clear();
				Graphics->Swap();
				cellSysutilCheckCallback();	
			}
		}
		else if(Settings.AccessoryAutoDetection == ACCESSORY_AUTODETECTION_ENABLED)
		{
			Settings.CurrentROMisJustifierCompatible = true;
		}
		else if(Settings.AccessoryAutoDetection == ACCESSORY_AUTODETECTION_NONE)
		{
			Settings.CurrentROMisJustifierCompatible = false;
		}
		if(Settings.CurrentROMisJustifierCompatible)
		{
			Emulator_SetControllerMode(TWO_JUSTIFIERS);
		}
	}
	//this last rule is in case Multitap has been set manually by the user by pressing TRIANGLE on a ROM
	else if (Emulator_GetControllerMode() != MULTITAP)
	{
		LOG_DBG("Default to TWO_JOYSTICKS\n");
		Emulator_SetControllerMode(TWO_JOYSTICKS);
	}

	// Check if Chrono Trigger is loaded, if so, we need to set a variable to true
	// to get rid of an annoying mid-frame resolution switch to 256x239 which can cause
	// an undesirable flicker/breakup of the screen for a split second
	Emulator_Implementation_OddScreenHeightJumpFix();

	Memory.LoadSRAM(S9xGetFilename(".srm", SRAM_DIR));

	GFX.Pitch = 1024;
	GFX.Screen = (uint16_t *)memalign(1024, 1024 * 478);

	if(Emulator_IsInitialized())
	{
		S9xGraphicsDeinit();
		S9xControlsSoftReset();
	}
	S9xGraphicsInit();


	// Controller initialization
	switch(Emulator_GetControllerMode())
	{
		case TWO_JOYSTICKS:
			S9xSetController(0, CTL_JOYPAD,		0,	0,	0,	0);
			S9xSetController(1, CTL_JOYPAD,		1,	0,	0,	0);
			break;
		case MULTITAP:
			S9xSetController(0, CTL_JOYPAD,		0,	0,	0,	0);
			S9xSetController(1, CTL_MP5,		1,	2,	3,	4);
			break;
		case MOUSE:
			S9xSetController(0, CTL_MOUSE,		0,	0,	0,	0);
			S9xSetController(1, CTL_JOYPAD,		1,	0,	0,	0);
			break;
		case SUPERSCOPE:
			S9xSetController(0, CTL_JOYPAD,		0,	0,	0,	0);
			S9xSetController(1, CTL_SUPERSCOPE,	0,	0,	0,	0);
			break;
		case JUSTIFIER:
			S9xSetController(0, CTL_JOYPAD,		0,	0,	0,	0);
			S9xSetController(1, CTL_JUSTIFIER,	0,	0,	0,	0);
			break;
		case TWO_JUSTIFIERS:
			S9xSetController(0, CTL_JOYPAD,		0,	0,	0,	0);
			S9xSetController(1, CTL_JUSTIFIER,	1,	0,	0,	0);	
			break;
		default:
			break;
	}

	S9xUnmapAllControls();

#define MAP_BUTTON(id, name) S9xMapButton((id), S9xGetCommandT((name)), false)

	MAP_BUTTON(MAKE_BUTTON(PAD_1, BTN_A), "{Joypad1 A,Superscope ToggleTurbo}");
	MAP_BUTTON(MAKE_BUTTON(PAD_1, BTN_B), "{Joypad1 B,Superscope Cursor}");
	MAP_BUTTON(MAKE_BUTTON(PAD_1, BTN_X), "{Joypad1 X,Mouse1 R,Superscope AimOffscreen,Justifier1 AimOffscreen}");
	MAP_BUTTON(MAKE_BUTTON(PAD_1, BTN_Y), "{Joypad1 Y,Mouse1 L,Superscope Fire,Justifier1 Trigger}");
	MAP_BUTTON(MAKE_BUTTON(PAD_1, BTN_SELECT), "Joypad1 Select");
	MAP_BUTTON(MAKE_BUTTON(PAD_1, BTN_START), "{Joypad1 Start,Superscope Pause,Justifier1 Start}");
	MAP_BUTTON(MAKE_BUTTON(PAD_1, BTN_L), "Joypad1 L");
	MAP_BUTTON(MAKE_BUTTON(PAD_1, BTN_R), "Joypad1 R");
	MAP_BUTTON(MAKE_BUTTON(PAD_1, BTN_LEFT), "Joypad1 Left");
	MAP_BUTTON(MAKE_BUTTON(PAD_1, BTN_RIGHT), "Joypad1 Right");
	MAP_BUTTON(MAKE_BUTTON(PAD_1, BTN_UP), "Joypad1 Up");
	MAP_BUTTON(MAKE_BUTTON(PAD_1, BTN_DOWN), "Joypad1 Down");
	MAP_BUTTON(MAKE_BUTTON(PAD_1, BTN_QUICKSAVE), "SaveFreezeFile");
	MAP_BUTTON(MAKE_BUTTON(PAD_1, BTN_QUICKLOAD), "LoadFreezeFile");
	MAP_BUTTON(MAKE_BUTTON(PAD_1, BTN_INCREMENTSAVE), "IncSave");
	MAP_BUTTON(MAKE_BUTTON(PAD_1, BTN_DECREMENTSAVE), "DecSave");
	MAP_BUTTON(MAKE_BUTTON(PAD_1, BTN_INCREMENTCHEAT), "IncCheat");
	MAP_BUTTON(MAKE_BUTTON(PAD_1, BTN_DECREMENTCHEAT), "DecCheat");
	MAP_BUTTON(MAKE_BUTTON(PAD_1, BTN_CHEATENABLE), "CheatEnable");
	MAP_BUTTON(MAKE_BUTTON(PAD_1, BTN_CHEATTOGGLE), "CheatToggle");
	MAP_BUTTON(MAKE_BUTTON(PAD_1, BTN_CHEATDISABLE), "CheatDisable");
	MAP_BUTTON(MAKE_BUTTON(PAD_1, BTN_CHEATINPUT), "CheatInput");
	MAP_BUTTON(MAKE_BUTTON(PAD_1, BTN_EXITTOMENU), "ExitToMenu");
	MAP_BUTTON(MAKE_BUTTON(PAD_1, BTN_RESET), "Reset");
	MAP_BUTTON(MAKE_BUTTON(PAD_1, BTN_SOFTRESET), "SoftReset");
	MAP_BUTTON(MAKE_BUTTON(PAD_1, BTN_PAUSE), "Pause");
	MAP_BUTTON(MAKE_BUTTON(PAD_1, BTN_BEGIN_RECORDING_MOVIE), "BeginRecordingMovie");
	MAP_BUTTON(MAKE_BUTTON(PAD_1, BTN_END_RECORDING_MOVIE), "EndRecordingMovie");
	MAP_BUTTON(MAKE_BUTTON(PAD_1, BTN_TOGGLE_RECORDING_MOVIE), "ToggleRecordingMovie");
	MAP_BUTTON(MAKE_BUTTON(PAD_1, BTN_LOAD_MOVIE), "LoadMovie");
	MAP_BUTTON(MAKE_BUTTON(PAD_1, BTN_FASTFORWARD), "ToggleEmuTurbo");
	MAP_BUTTON(MAKE_BUTTON(PAD_1, BTN_INCREMENTTURBO), "IncEmuTurbo");
	MAP_BUTTON(MAKE_BUTTON(PAD_1, BTN_DECREMENTTURBO), "DecEmuTurbo");
	MAP_BUTTON(MAKE_BUTTON(PAD_1, BTN_SWAPJOYPADS), "SwapJoypads");
	MAP_BUTTON(MAKE_BUTTON(PAD_1, BTN_SRAM_WRITEPROTECT), "SramWriteProtect");
	S9xMapPointer((BTN_POINTER1), S9xGetCommandT("Pointer Mouse1"), false);
	S9xMapPointer((BTN_SCOPE_POINTER), S9xGetCommandT("Pointer Superscope+Justifier1"), false);

	MAP_BUTTON(MAKE_BUTTON(PAD_2, BTN_A), "Joypad2 A");
	MAP_BUTTON(MAKE_BUTTON(PAD_2, BTN_B), "Joypad2 B");
	MAP_BUTTON(MAKE_BUTTON(PAD_2, BTN_X), "{Joypad2 X,Mouse2 R,Justifier2 AimOffscreen}");
	MAP_BUTTON(MAKE_BUTTON(PAD_2, BTN_Y), "{Joypad2 Y,Mouse2 R,Justifier2 Trigger}");
	MAP_BUTTON(MAKE_BUTTON(PAD_2, BTN_SELECT), "Joypad2 Select");
	MAP_BUTTON(MAKE_BUTTON(PAD_2, BTN_START), "{Joypad2 Start,Justifier2 Start}");
	MAP_BUTTON(MAKE_BUTTON(PAD_2, BTN_L), "Joypad2 L");
	MAP_BUTTON(MAKE_BUTTON(PAD_2, BTN_R), "Joypad2 R");
	MAP_BUTTON(MAKE_BUTTON(PAD_2, BTN_LEFT), "Joypad2 Left");
	MAP_BUTTON(MAKE_BUTTON(PAD_2, BTN_RIGHT), "Joypad2 Right");
	MAP_BUTTON(MAKE_BUTTON(PAD_2, BTN_UP), "Joypad2 Up");
	MAP_BUTTON(MAKE_BUTTON(PAD_2, BTN_DOWN), "Joypad2 Down");
	MAP_BUTTON(MAKE_BUTTON(PAD_2, BTN_QUICKSAVE), "SaveFreezeFile");
	MAP_BUTTON(MAKE_BUTTON(PAD_2, BTN_QUICKLOAD), "LoadFreezeFile");
	MAP_BUTTON(MAKE_BUTTON(PAD_2, BTN_INCREMENTSAVE), "IncSave");
	MAP_BUTTON(MAKE_BUTTON(PAD_2, BTN_DECREMENTSAVE), "DecSave");
	MAP_BUTTON(MAKE_BUTTON(PAD_2, BTN_CHEATENABLE), "CheatEnable");
	MAP_BUTTON(MAKE_BUTTON(PAD_2, BTN_CHEATTOGGLE), "CheatToggle");
	MAP_BUTTON(MAKE_BUTTON(PAD_2, BTN_CHEATDISABLE), "CheatDisable");
	MAP_BUTTON(MAKE_BUTTON(PAD_2, BTN_CHEATINPUT), "CheatInput");
	MAP_BUTTON(MAKE_BUTTON(PAD_2, BTN_INCREMENTCHEAT), "IncCheat");
	MAP_BUTTON(MAKE_BUTTON(PAD_2, BTN_DECREMENTCHEAT), "DecCheat");
	MAP_BUTTON(MAKE_BUTTON(PAD_2, BTN_EXITTOMENU), "ExitToMenu");
	MAP_BUTTON(MAKE_BUTTON(PAD_2, BTN_PAUSE), "Pause");
	MAP_BUTTON(MAKE_BUTTON(PAD_2, BTN_BEGIN_RECORDING_MOVIE), "BeginRecordingMovie");
	MAP_BUTTON(MAKE_BUTTON(PAD_2, BTN_END_RECORDING_MOVIE), "EndRecordingMovie");
	MAP_BUTTON(MAKE_BUTTON(PAD_2, BTN_TOGGLE_RECORDING_MOVIE), "ToggleRecordingMovie");
	MAP_BUTTON(MAKE_BUTTON(PAD_2, BTN_LOAD_MOVIE), "LoadMovie");
	MAP_BUTTON(MAKE_BUTTON(PAD_2, BTN_FASTFORWARD), "ToggleEmuTurbo");
	MAP_BUTTON(MAKE_BUTTON(PAD_2, BTN_RESET), "Reset");
	MAP_BUTTON(MAKE_BUTTON(PAD_2, BTN_SOFTRESET), "SoftReset");
	MAP_BUTTON(MAKE_BUTTON(PAD_2, BTN_INCREMENTTURBO), "IncEmuTurbo");
	MAP_BUTTON(MAKE_BUTTON(PAD_2, BTN_DECREMENTTURBO), "DecEmuTurbo");
	MAP_BUTTON(MAKE_BUTTON(PAD_2, BTN_SWAPJOYPADS), "SwapJoypads");
	MAP_BUTTON(MAKE_BUTTON(PAD_2, BTN_SRAM_WRITEPROTECT), "SramWriteProtect");
	S9xMapPointer((BTN_POINTER2), S9xGetCommandT("Pointer Mouse2"), false);
	S9xMapPointer((BTN_SCOPE_POINTER), S9xGetCommandT("Pointer Superscope+Justifier2"), false);
	switch(Emulator_GetControllerMode())
	{
		case MULTITAP:
			//player 3 - for multitap
			MAP_BUTTON(MAKE_BUTTON(PAD_3, BTN_A), "Joypad3 A");
			MAP_BUTTON(MAKE_BUTTON(PAD_3, BTN_B), "Joypad3 B");
			MAP_BUTTON(MAKE_BUTTON(PAD_3, BTN_X), "Joypad3 X");
			MAP_BUTTON(MAKE_BUTTON(PAD_3, BTN_Y), "Joypad3 Y");
			MAP_BUTTON(MAKE_BUTTON(PAD_3, BTN_SELECT), "Joypad3 Select");
			MAP_BUTTON(MAKE_BUTTON(PAD_3, BTN_START), "Joypad3 Start");
			MAP_BUTTON(MAKE_BUTTON(PAD_3, BTN_L), "Joypad3 L");
			MAP_BUTTON(MAKE_BUTTON(PAD_3, BTN_R), "Joypad3 R");
			MAP_BUTTON(MAKE_BUTTON(PAD_3, BTN_LEFT), "Joypad3 Left");
			MAP_BUTTON(MAKE_BUTTON(PAD_3, BTN_RIGHT), "Joypad3 Right");
			MAP_BUTTON(MAKE_BUTTON(PAD_3, BTN_UP), "Joypad3 Up");
			MAP_BUTTON(MAKE_BUTTON(PAD_3, BTN_DOWN), "Joypad3 Down");
			MAP_BUTTON(MAKE_BUTTON(PAD_3, BTN_QUICKSAVE), "SaveFreezeFile");
			MAP_BUTTON(MAKE_BUTTON(PAD_3, BTN_QUICKLOAD), "LoadFreezeFile");
			MAP_BUTTON(MAKE_BUTTON(PAD_3, BTN_INCREMENTSAVE), "IncSave");
			MAP_BUTTON(MAKE_BUTTON(PAD_3, BTN_DECREMENTSAVE), "DecSave");
			MAP_BUTTON(MAKE_BUTTON(PAD_3, BTN_INCREMENTCHEAT), "IncCheat");
			MAP_BUTTON(MAKE_BUTTON(PAD_3, BTN_DECREMENTCHEAT), "DecCheat");
			MAP_BUTTON(MAKE_BUTTON(PAD_3, BTN_CHEATENABLE), "CheatEnable");
			MAP_BUTTON(MAKE_BUTTON(PAD_3, BTN_CHEATTOGGLE), "CheatToggle");
			MAP_BUTTON(MAKE_BUTTON(PAD_3, BTN_CHEATDISABLE), "CheatDisable");
			MAP_BUTTON(MAKE_BUTTON(PAD_3, BTN_CHEATINPUT), "CheatInput");
			MAP_BUTTON(MAKE_BUTTON(PAD_3, BTN_EXITTOMENU), "ExitToMenu");
			MAP_BUTTON(MAKE_BUTTON(PAD_3, BTN_PAUSE), "Pause");
			MAP_BUTTON(MAKE_BUTTON(PAD_3, BTN_BEGIN_RECORDING_MOVIE), "BeginRecordingMovie");
			MAP_BUTTON(MAKE_BUTTON(PAD_3, BTN_END_RECORDING_MOVIE), "EndRecordingMovie");
			MAP_BUTTON(MAKE_BUTTON(PAD_3, BTN_TOGGLE_RECORDING_MOVIE), "ToggleRecordingMovie");
			MAP_BUTTON(MAKE_BUTTON(PAD_3, BTN_LOAD_MOVIE), "LoadMovie");
			MAP_BUTTON(MAKE_BUTTON(PAD_3, BTN_FASTFORWARD), "ToggleEmuTurbo");
			MAP_BUTTON(MAKE_BUTTON(PAD_3, BTN_RESET), "Reset");
			MAP_BUTTON(MAKE_BUTTON(PAD_3, BTN_SOFTRESET), "SoftReset");
			MAP_BUTTON(MAKE_BUTTON(PAD_3, BTN_INCREMENTTURBO), "IncEmuTurbo");
			MAP_BUTTON(MAKE_BUTTON(PAD_3, BTN_DECREMENTTURBO), "DecEmuTurbo");
			MAP_BUTTON(MAKE_BUTTON(PAD_3, BTN_SWAPJOYPADS), "SwapJoypads");
			MAP_BUTTON(MAKE_BUTTON(PAD_3, BTN_SRAM_WRITEPROTECT), "SramWriteProtect");
			//player 4 - for multitap
			MAP_BUTTON(MAKE_BUTTON(PAD_4, BTN_A), "Joypad4 A");
			MAP_BUTTON(MAKE_BUTTON(PAD_4, BTN_B), "Joypad4 B");
			MAP_BUTTON(MAKE_BUTTON(PAD_4, BTN_X), "Joypad4 X");
			MAP_BUTTON(MAKE_BUTTON(PAD_4, BTN_Y), "Joypad4 Y");
			MAP_BUTTON(MAKE_BUTTON(PAD_4, BTN_SELECT), "Joypad4 Select");
			MAP_BUTTON(MAKE_BUTTON(PAD_4, BTN_START), "Joypad4 Start");
			MAP_BUTTON(MAKE_BUTTON(PAD_4, BTN_L), "Joypad4 L");
			MAP_BUTTON(MAKE_BUTTON(PAD_4, BTN_R), "Joypad4 R");
			MAP_BUTTON(MAKE_BUTTON(PAD_4, BTN_LEFT), "Joypad4 Left");
			MAP_BUTTON(MAKE_BUTTON(PAD_4, BTN_RIGHT), "Joypad4 Right");
			MAP_BUTTON(MAKE_BUTTON(PAD_4, BTN_UP), "Joypad4 Up");
			MAP_BUTTON(MAKE_BUTTON(PAD_4, BTN_DOWN), "Joypad4 Down");
			MAP_BUTTON(MAKE_BUTTON(PAD_4, BTN_QUICKSAVE), "SaveFreezeFile");
			MAP_BUTTON(MAKE_BUTTON(PAD_4, BTN_QUICKLOAD), "LoadFreezeFile");
			MAP_BUTTON(MAKE_BUTTON(PAD_4, BTN_INCREMENTSAVE), "IncSave");
			MAP_BUTTON(MAKE_BUTTON(PAD_4, BTN_DECREMENTSAVE), "DecSave");
			MAP_BUTTON(MAKE_BUTTON(PAD_4, BTN_CHEATENABLE), "CheatEnable");
			MAP_BUTTON(MAKE_BUTTON(PAD_4, BTN_CHEATTOGGLE), "CheatToggle");
			MAP_BUTTON(MAKE_BUTTON(PAD_4, BTN_CHEATDISABLE), "CheatDisable");
			MAP_BUTTON(MAKE_BUTTON(PAD_4, BTN_CHEATINPUT), "CheatInput");
			MAP_BUTTON(MAKE_BUTTON(PAD_4, BTN_EXITTOMENU), "ExitToMenu");
			MAP_BUTTON(MAKE_BUTTON(PAD_4, BTN_INCREMENTCHEAT), "IncCheat");
			MAP_BUTTON(MAKE_BUTTON(PAD_4, BTN_DECREMENTCHEAT), "DecCheat");
			MAP_BUTTON(MAKE_BUTTON(PAD_4, BTN_PAUSE), "Pause");
			MAP_BUTTON(MAKE_BUTTON(PAD_4, BTN_BEGIN_RECORDING_MOVIE), "BeginRecordingMovie");
			MAP_BUTTON(MAKE_BUTTON(PAD_4, BTN_END_RECORDING_MOVIE), "EndRecordingMovie");
			MAP_BUTTON(MAKE_BUTTON(PAD_4, BTN_TOGGLE_RECORDING_MOVIE), "ToggleRecordingMovie");
			MAP_BUTTON(MAKE_BUTTON(PAD_4, BTN_LOAD_MOVIE), "LoadMovie");
			MAP_BUTTON(MAKE_BUTTON(PAD_4, BTN_FASTFORWARD), "ToggleEmuTurbo");
			MAP_BUTTON(MAKE_BUTTON(PAD_4, BTN_RESET), "Reset");
			MAP_BUTTON(MAKE_BUTTON(PAD_4, BTN_SOFTRESET), "SoftReset");
			MAP_BUTTON(MAKE_BUTTON(PAD_4, BTN_INCREMENTTURBO), "IncEmuTurbo");
			MAP_BUTTON(MAKE_BUTTON(PAD_4, BTN_DECREMENTTURBO), "DecEmuTurbo");
			MAP_BUTTON(MAKE_BUTTON(PAD_4, BTN_SWAPJOYPADS), "SwapJoypads");
			MAP_BUTTON(MAKE_BUTTON(PAD_4, BTN_SRAM_WRITEPROTECT), "SramWriteProtect");
			//player 5 - for multitap
			MAP_BUTTON(MAKE_BUTTON(PAD_5, BTN_A), "Joypad5 A");
			MAP_BUTTON(MAKE_BUTTON(PAD_5, BTN_B), "Joypad5 B");
			MAP_BUTTON(MAKE_BUTTON(PAD_5, BTN_X), "Joypad5 X");
			MAP_BUTTON(MAKE_BUTTON(PAD_5, BTN_Y), "Joypad5 Y");
			MAP_BUTTON(MAKE_BUTTON(PAD_5, BTN_SELECT), "Joypad5 Select");
			MAP_BUTTON(MAKE_BUTTON(PAD_5, BTN_START), "Joypad5 Start");
			MAP_BUTTON(MAKE_BUTTON(PAD_5, BTN_L), "Joypad5 L");
			MAP_BUTTON(MAKE_BUTTON(PAD_5, BTN_R), "Joypad5 R");
			MAP_BUTTON(MAKE_BUTTON(PAD_5, BTN_LEFT), "Joypad5 Left");
			MAP_BUTTON(MAKE_BUTTON(PAD_5, BTN_RIGHT), "Joypad5 Right");
			MAP_BUTTON(MAKE_BUTTON(PAD_5, BTN_UP), "Joypad5 Up");
			MAP_BUTTON(MAKE_BUTTON(PAD_5, BTN_DOWN), "Joypad5 Down");
			MAP_BUTTON(MAKE_BUTTON(PAD_5, BTN_QUICKSAVE), "SaveFreezeFile");
			MAP_BUTTON(MAKE_BUTTON(PAD_5, BTN_QUICKLOAD), "LoadFreezeFile");
			MAP_BUTTON(MAKE_BUTTON(PAD_5, BTN_INCREMENTSAVE), "IncSave");
			MAP_BUTTON(MAKE_BUTTON(PAD_5, BTN_DECREMENTSAVE), "DecSave");
			MAP_BUTTON(MAKE_BUTTON(PAD_5, BTN_CHEATENABLE), "CheatEnable");
			MAP_BUTTON(MAKE_BUTTON(PAD_5, BTN_CHEATTOGGLE), "CheatToggle");
			MAP_BUTTON(MAKE_BUTTON(PAD_5, BTN_CHEATDISABLE), "CheatDisable");
			MAP_BUTTON(MAKE_BUTTON(PAD_5, BTN_CHEATINPUT), "CheatInput");
			MAP_BUTTON(MAKE_BUTTON(PAD_5, BTN_EXITTOMENU), "ExitToMenu");
			MAP_BUTTON(MAKE_BUTTON(PAD_5, BTN_INCREMENTCHEAT), "IncCheat");
			MAP_BUTTON(MAKE_BUTTON(PAD_5, BTN_DECREMENTCHEAT), "DecCheat");
			MAP_BUTTON(MAKE_BUTTON(PAD_5, BTN_PAUSE), "Pause");
			MAP_BUTTON(MAKE_BUTTON(PAD_5, BTN_BEGIN_RECORDING_MOVIE), "BeginRecordingMovie");
			MAP_BUTTON(MAKE_BUTTON(PAD_5, BTN_END_RECORDING_MOVIE), "EndRecordingMovie");
			MAP_BUTTON(MAKE_BUTTON(PAD_5, BTN_TOGGLE_RECORDING_MOVIE), "ToggleRecordingMovie");
			MAP_BUTTON(MAKE_BUTTON(PAD_5, BTN_LOAD_MOVIE), "LoadMovie");
			MAP_BUTTON(MAKE_BUTTON(PAD_5, BTN_FASTFORWARD), "ToggleEmuTurbo");
			MAP_BUTTON(MAKE_BUTTON(PAD_5, BTN_RESET), "Reset");
			MAP_BUTTON(MAKE_BUTTON(PAD_5, BTN_SOFTRESET), "SoftReset");
			MAP_BUTTON(MAKE_BUTTON(PAD_5, BTN_INCREMENTTURBO), "IncEmuTurbo");
			MAP_BUTTON(MAKE_BUTTON(PAD_5, BTN_DECREMENTTURBO), "DecEmuTurbo");
			MAP_BUTTON(MAKE_BUTTON(PAD_5, BTN_SWAPJOYPADS), "SwapJoypads");
			MAP_BUTTON(MAKE_BUTTON(PAD_5, BTN_SRAM_WRITEPROTECT), "SramWriteProtect");
			break;

		default:
			break;
	}

	return true;	
}

void Emulator_Implementation_ReportMouse(int x, int y,uint32 pointer)
{
	static bool first = true;
	static int _x = 128;
	static int _y = 128;

	if (first)
	{
		S9xReportPointer(pointer, 128, 128);
		first = false;
	}
	else
	{
		_x += (x - 128) / 15;
		_y += (y - 128) / 15;
		S9xReportPointer(pointer, _x, _y);
	}
}

void Emulator_Implementation_ReportScope(int x, int y)
{
	static bool first = true;
	static int _x = 128;
	static int _y = 128;

	if (first)
	{
		S9xReportPointer(BTN_SCOPE_POINTER, 128, 128);
		first = false;
	}
	else
	{
		_x += (x - 128) / 15;
		_y += (y - 128) / 15;
		if(_x > 277)
		{
			_x = 255;
		}
		if(_x < -16)
		{
			_x = 0;
		}
		if(_y > 256)
		{
			_y = 255;
		}
		if(_y < -16)
		{
			_y = 0;
		}
		S9xReportPointer(BTN_SCOPE_POINTER, _x, _y);
	}
}

void Emulator_Implementation_Inputloop()
{
	uint8_t pads_connected = CellInput->NumberPadsConnected();
	for (uint8_t i = 0; i < pads_connected; ++i)
	{
		if(CellInput->UpdateDevice(i) != CELL_PAD_OK)
		{
			continue;
		}
		unsigned int pad_id = i + 1;
		S9xReportButton(MAKE_BUTTON(pad_id, Settings.ButtonCircle), (CellInput->IsButtonPressed(i,CTRL_CIRCLE)));
		S9xReportButton(MAKE_BUTTON(pad_id, Settings.ButtonCross), (CellInput->IsButtonPressed(i,CTRL_CROSS)));
		S9xReportButton(MAKE_BUTTON(pad_id, Settings.ButtonTriangle), (CellInput->IsButtonPressed(i,CTRL_TRIANGLE)));
		S9xReportButton(MAKE_BUTTON(pad_id, Settings.ButtonSquare), (CellInput->IsButtonPressed(i,CTRL_SQUARE)));
		S9xReportButton(MAKE_BUTTON(pad_id, Settings.ButtonSelect), (CellInput->IsButtonPressed(i,CTRL_SELECT)));
		S9xReportButton(MAKE_BUTTON(pad_id, Settings.ButtonStart), (CellInput->IsButtonPressed(i,CTRL_START)));
		S9xReportButton(MAKE_BUTTON(pad_id, Settings.ButtonL1), (CellInput->IsButtonPressed(i,CTRL_L1)));
		S9xReportButton(MAKE_BUTTON(pad_id, Settings.ButtonL2), (CellInput->IsButtonPressed(i,CTRL_L2) & !(CellInput->IsButtonPressed(i,CTRL_R2))));
		S9xReportButton(MAKE_BUTTON(pad_id, Settings.ButtonR1), (CellInput->IsButtonPressed(i,CTRL_R1)));
		S9xReportButton(MAKE_BUTTON(pad_id, Settings.ButtonR2), (CellInput->IsButtonPressed(i,CTRL_R2) & !(CellInput->IsButtonPressed(i,CTRL_L2))));
 		S9xReportButton(MAKE_BUTTON(pad_id, Settings.DPad_Up), (CellInput->IsButtonPressed(i,CTRL_UP) | CellInput->IsAnalogPressedUp(i,CTRL_LSTICK)));
                S9xReportButton(MAKE_BUTTON(pad_id, Settings.DPad_Down), (CellInput->IsButtonPressed(i,CTRL_DOWN) | CellInput->IsAnalogPressedDown(i,CTRL_LSTICK)));
                S9xReportButton(MAKE_BUTTON(pad_id, Settings.DPad_Left), (CellInput->IsButtonPressed(i,CTRL_LEFT) | CellInput->IsAnalogPressedLeft(i,CTRL_LSTICK)));
                S9xReportButton(MAKE_BUTTON(pad_id, Settings.DPad_Right), (CellInput->IsButtonPressed(i,CTRL_RIGHT) | CellInput->IsAnalogPressedRight(i,CTRL_LSTICK)));
		S9xReportButton(MAKE_BUTTON(pad_id, Settings.ButtonR2_ButtonR3), (CellInput->IsButtonPressed(i,CTRL_R2) & CellInput->IsButtonPressed(i,CTRL_R3)));
		S9xReportButton(MAKE_BUTTON(pad_id, Settings.ButtonL2_ButtonR3), (CellInput->IsButtonPressed(i,CTRL_L2) & CellInput->IsButtonPressed(i,CTRL_R3)));
		S9xReportButton(MAKE_BUTTON(pad_id, Settings.ButtonL2_ButtonL3), (CellInput->IsButtonPressed(i,CTRL_L2) & CellInput->IsButtonPressed(i,CTRL_L3)));
		S9xReportButton(MAKE_BUTTON(pad_id, Settings.ButtonL2_ButtonR2), (CellInput->WasButtonHeld(i,CTRL_L2) & CellInput->WasButtonHeld(i,CTRL_R2)));
		S9xReportButton(MAKE_BUTTON(pad_id, Settings.ButtonL3), (CellInput->IsButtonPressed(i,CTRL_L3)));
		S9xReportButton(MAKE_BUTTON(pad_id, Settings.ButtonR3), (CellInput->IsButtonPressed(i,CTRL_R3)));
		S9xReportButton(MAKE_BUTTON(pad_id, Settings.AnalogR_Right), Settings.AnalogR_Right_Type ? CellInput->IsAnalogPressedRight(i,CTRL_RSTICK) & !(CellInput->WasButtonHeld(i,CTRL_L2)) & !(CellInput->WasButtonHeld(i,CTRL_R2)) : CellInput->WasAnalogPressedRight(i,CTRL_RSTICK) & !(CellInput->WasButtonHeld(i,CTRL_L2)) & !(CellInput->WasButtonHeld(i,CTRL_R2)));
		S9xReportButton(MAKE_BUTTON(pad_id, Settings.AnalogR_Left), Settings.AnalogR_Left_Type ? CellInput->IsAnalogPressedLeft(i,CTRL_RSTICK) & !(CellInput->WasButtonHeld(i,CTRL_L2)) & !(CellInput->WasButtonHeld(i,CTRL_R2)) : CellInput->WasAnalogPressedLeft(i,CTRL_RSTICK) & !(CellInput->WasButtonHeld(i,CTRL_L2)) & !(CellInput->WasButtonHeld(i,CTRL_R2)));
		S9xReportButton(MAKE_BUTTON(pad_id, Settings.AnalogR_Up), Settings.AnalogR_Up_Type ? CellInput->IsAnalogPressedUp(i,CTRL_RSTICK) & !(CellInput->WasButtonHeld(i,CTRL_L2)) & !(CellInput->WasButtonHeld(i,CTRL_R2)) : CellInput->WasAnalogPressedUp(i,CTRL_RSTICK) & !(CellInput->WasButtonHeld(i,CTRL_L2)) & !(CellInput->WasButtonHeld(i,CTRL_R2)));
		S9xReportButton(MAKE_BUTTON(pad_id, Settings.AnalogR_Down), Settings.AnalogR_Down_Type ? CellInput->IsAnalogPressedDown(i,CTRL_RSTICK) & !(CellInput->WasButtonHeld(i,CTRL_L2)) & !(CellInput->WasButtonHeld(i,CTRL_R2)) : CellInput->WasAnalogPressedDown(i,CTRL_RSTICK) & !(CellInput->WasButtonHeld(i,CTRL_L2)) & !(CellInput->WasButtonHeld(i,CTRL_R2)));
		S9xReportButton(MAKE_BUTTON(pad_id, Settings.ButtonL2_AnalogR_Right), (CellInput->WasButtonHeld(i,CTRL_L2) & CellInput->WasAnalogPressedRight(i,CTRL_RSTICK)));
		S9xReportButton(MAKE_BUTTON(pad_id, Settings.ButtonL2_AnalogR_Left), (CellInput->WasButtonHeld(i,CTRL_L2) & CellInput->WasAnalogPressedLeft(i,CTRL_RSTICK)));
		S9xReportButton(MAKE_BUTTON(pad_id, Settings.ButtonL2_AnalogR_Up), (CellInput->IsButtonPressed(i,CTRL_L2) & CellInput->WasAnalogPressedUp(i,CTRL_RSTICK)));
		S9xReportButton(MAKE_BUTTON(pad_id, Settings.ButtonL2_AnalogR_Down), (CellInput->IsButtonPressed(i,CTRL_L2) & CellInput->WasAnalogPressedDown(i,CTRL_RSTICK)));
		S9xReportButton(MAKE_BUTTON(pad_id, Settings.ButtonR2_AnalogR_Right), (CellInput->IsButtonPressed(i,CTRL_R2) & CellInput->WasAnalogPressedRight(i,CTRL_RSTICK)));
		S9xReportButton(MAKE_BUTTON(pad_id, Settings.ButtonR2_AnalogR_Left), (CellInput->IsButtonPressed(i,CTRL_R2) & CellInput->WasAnalogPressedLeft(i,CTRL_RSTICK)));
		S9xReportButton(MAKE_BUTTON(pad_id, Settings.ButtonR2_AnalogR_Up), (CellInput->IsButtonPressed(i,CTRL_R2) & CellInput->WasAnalogPressedUp(i,CTRL_RSTICK)));
		S9xReportButton(MAKE_BUTTON(pad_id, Settings.ButtonR2_AnalogR_Down), (CellInput->IsButtonPressed(i,CTRL_R2) & CellInput->WasAnalogPressedDown(i,CTRL_RSTICK)));
		S9xReportButton(MAKE_BUTTON(pad_id, Settings.ButtonR3_ButtonL3), (CellInput->IsButtonPressed(i,CTRL_R3) & CellInput->IsButtonPressed(i,CTRL_L3)));
		if (Emulator_GetControllerMode() & (MOUSE))
		{
			Emulator_Implementation_ReportMouse(CellInput->GetAnalogXValue(i,CTRL_LSTICK),CellInput->GetAnalogYValue(i,CTRL_LSTICK), pad_id == 1 ? BTN_POINTER1 : BTN_POINTER2);
		}
		else if(Emulator_GetControllerMode() &(SUPERSCOPE | JUSTIFIER | TWO_JUSTIFIERS))
		{
			Emulator_Implementation_ReportScope(CellInput->GetAnalogXValue(i,CTRL_LSTICK),CellInput->GetAnalogYValue(i,CTRL_LSTICK));
		}
	}}

void Emulator_Implementation_Mainloop()
{
	Emulator_Implementation_Inputloop();

	if(!Settings.Paused)
	{
		S9xMainLoop();
	}
	cellSysutilCheckCallback();
}

bool Emulator_Implementation_InitSettings()
{
	ZeroMemory(&Settings, sizeof(Settings));

	/*
	Settings.DumpStreamsMaxFrames = -1;
	Settings.CartAName[0] = 0;
	Settings.CartBName[0] = 0;
	*/

	S9xLoadConfigFiles();

	return true;
}

void Emulator_Implementation_ButtonMappingSettings(int map_button_option_enum)
{
	switch(map_button_option_enum)
	{
		case MAP_BUTTONS_OPTION_SETTER:
			currentconfig.SetInt("PS3ButtonMappings::DPad_Up",Settings.DPad_Up);
			currentconfig.SetInt("PS3ButtonMappings::DPad_Down",Settings.DPad_Down);
			currentconfig.SetInt("PS3ButtonMappings::DPad_Left",Settings.DPad_Left);
			currentconfig.SetInt("PS3ButtonMappings::DPad_Right",Settings.DPad_Right);
			currentconfig.SetInt("PS3ButtonMappings::ButtonCircle",Settings.ButtonCircle);
			currentconfig.SetInt("PS3ButtonMappings::ButtonCross",Settings.ButtonCross);
			currentconfig.SetInt("PS3ButtonMappings::ButtonTriangle",Settings.ButtonTriangle);
			currentconfig.SetInt("PS3ButtonMappings::ButtonSquare",Settings.ButtonSquare);
			currentconfig.SetInt("PS3ButtonMappings::ButtonSelect",Settings.ButtonSelect);
			currentconfig.SetInt("PS3ButtonMappings::ButtonStart",Settings.ButtonStart);
			currentconfig.SetInt("PS3ButtonMappings::ButtonL1",Settings.ButtonL1);
			currentconfig.SetInt("PS3ButtonMappings::ButtonR1",Settings.ButtonR1);
			currentconfig.SetInt("PS3ButtonMappings::ButtonL2",Settings.ButtonL2);
			currentconfig.SetInt("PS3ButtonMappings::ButtonR2",Settings.ButtonR2);
			currentconfig.SetInt("PS3ButtonMappings::ButtonL2_ButtonL3",Settings.ButtonL2_ButtonL3);
			currentconfig.SetInt("PS3ButtonMappings::ButtonL2_ButtonR3",Settings.ButtonL2_ButtonR3);
			currentconfig.SetInt("PS3ButtonMappings::ButtonR3",Settings.ButtonR3);
			currentconfig.SetInt("PS3ButtonMappings::ButtonL3",Settings.ButtonL3);
			currentconfig.SetInt("PS3ButtonMappings::ButtonL2_ButtonR2",Settings.ButtonL2_ButtonR2);
			currentconfig.SetInt("PS3ButtonMappings::ButtonL2_AnalogR_Right",Settings.ButtonL2_AnalogR_Right);
			currentconfig.SetInt("PS3ButtonMappings::ButtonL2_AnalogR_Left",Settings.ButtonL2_AnalogR_Left);
			currentconfig.SetInt("PS3ButtonMappings::ButtonL2_AnalogR_Up",Settings.ButtonL2_AnalogR_Up);
			currentconfig.SetInt("PS3ButtonMappings::ButtonL2_AnalogR_Down",Settings.ButtonL2_AnalogR_Down);
			currentconfig.SetInt("PS3ButtonMappings::ButtonR2_AnalogR_Right",Settings.ButtonR2_AnalogR_Right);
			currentconfig.SetInt("PS3ButtonMappings::ButtonR2_AnalogR_Left",Settings.ButtonR2_AnalogR_Left);
			currentconfig.SetInt("PS3ButtonMappings::ButtonR2_AnalogR_Up",Settings.ButtonR2_AnalogR_Up);
			currentconfig.SetInt("PS3ButtonMappings::ButtonR2_AnalogR_Down",Settings.ButtonR2_AnalogR_Down);
			currentconfig.SetInt("PS3ButtonMappings::ButtonR2_ButtonR3",Settings.ButtonR2_ButtonR3);
			currentconfig.SetInt("PS3ButtonMappings::ButtonR3_ButtonL3",Settings.ButtonR3_ButtonL3);
			currentconfig.SetInt("PS3ButtonMappings::AnalogR_Up",Settings.AnalogR_Up);
			currentconfig.SetInt("PS3ButtonMappings::AnalogR_Down",Settings.AnalogR_Down);
			currentconfig.SetInt("PS3ButtonMappings::AnalogR_Left",Settings.AnalogR_Left);
			currentconfig.SetInt("PS3ButtonMappings::AnalogR_Right",Settings.AnalogR_Right);

			currentconfig.SetInt("PS3ButtonMappings::AnalogR_Up_Type",Settings.AnalogR_Up_Type);
			currentconfig.SetInt("PS3ButtonMappings::AnalogR_Down_Type",Settings.AnalogR_Down_Type);
			currentconfig.SetInt("PS3ButtonMappings::AnalogR_Left_Type",Settings.AnalogR_Left_Type);
			currentconfig.SetInt("PS3ButtonMappings::AnalogR_Right_Type",Settings.AnalogR_Right_Type);
			break;
		case MAP_BUTTONS_OPTION_GETTER:
			Settings.DPad_Up		= currentconfig.GetInt("PS3ButtonMappings::DPad_Up",BTN_UP);
			Settings.DPad_Down		= currentconfig.GetInt("PS3ButtonMappings::DPad_Down",BTN_DOWN);
			Settings.DPad_Left		= currentconfig.GetInt("PS3ButtonMappings::DPad_Left",BTN_LEFT);
			Settings.DPad_Right		= currentconfig.GetInt("PS3ButtonMappings::DPad_Right",BTN_RIGHT);
			Settings.ButtonCircle		= currentconfig.GetInt("PS3ButtonMappings::ButtonCircle",BTN_A);
			Settings.ButtonCross		= currentconfig.GetInt("PS3ButtonMappings::ButtonCross",BTN_B);
			Settings.ButtonTriangle		= currentconfig.GetInt("PS3ButtonMappings::ButtonTriangle",BTN_X);
			Settings.ButtonSquare		= currentconfig.GetInt("PS3ButtonMappings::ButtonSquare",BTN_Y);
			Settings.ButtonSelect		= currentconfig.GetInt("PS3ButtonMappings::ButtonSelect",BTN_SELECT);
			Settings.ButtonStart		= currentconfig.GetInt("PS3ButtonMappings::ButtonStart",BTN_START);
			Settings.ButtonL1		= currentconfig.GetInt("PS3ButtonMappings::ButtonL1",BTN_L);
			Settings.ButtonR1		= currentconfig.GetInt("PS3ButtonMappings::ButtonR1",BTN_R);
			Settings.ButtonL2		= currentconfig.GetInt("PS3ButtonMappings::ButtonL2",BTN_NONE);
			Settings.ButtonR2		= currentconfig.GetInt("PS3ButtonMappings::ButtonR2",BTN_NONE);
			Settings.ButtonL2_ButtonL3	= currentconfig.GetInt("PS3ButtonMappings::ButtonL2_ButtonL3",BTN_NONE);
			Settings.ButtonL2_ButtonR3	= currentconfig.GetInt("PS3ButtonMappings::ButtonL2_ButtonR3",BTN_QUICKLOAD);
			Settings.ButtonR3		= currentconfig.GetInt("PS3ButtonMappings::ButtonR3",BTN_NONE);
			Settings.ButtonL3		= currentconfig.GetInt("PS3ButtonMappings::ButtonL3",BTN_NONE);
			Settings.ButtonL2_ButtonR2	= currentconfig.GetInt("PS3ButtonMappings::ButtonL2_ButtonR2",BTN_NONE);
			Settings.ButtonL2_AnalogR_Right = currentconfig.GetInt("PS3ButtonMappings::ButtonL2_AnalogR_Right",BTN_INCREMENTCHEAT);
			Settings.ButtonL2_AnalogR_Left	= currentconfig.GetInt("PS3ButtonMappings::ButtonL2_AnalogR_Left",BTN_DECREMENTCHEAT);
			Settings.ButtonL2_AnalogR_Up	= currentconfig.GetInt("PS3ButtonMappings::ButtonL2_AnalogR_Up",BTN_NONE);
			Settings.ButtonL2_AnalogR_Down	= currentconfig.GetInt("PS3ButtonMappings::ButtonL2_AnalogR_Down",BTN_NONE);
			Settings.ButtonR2_AnalogR_Right	= currentconfig.GetInt("PS3ButtonMappings::ButtonR2_AnalogR_Right",BTN_NONE);
			Settings.ButtonR2_AnalogR_Left	= currentconfig.GetInt("PS3ButtonMappings::ButtonR2_AnalogR_Left",BTN_NONE);
			Settings.ButtonR2_AnalogR_Up	= currentconfig.GetInt("PS3ButtonMappings::ButtonR2_AnalogR_Up",BTN_NONE);
			Settings.ButtonR2_AnalogR_Down	= currentconfig.GetInt("PS3ButtonMappings::ButtonR2_AnalogR_Down",BTN_NONE);
			Settings.ButtonR2_ButtonR3	= currentconfig.GetInt("PS3ButtonMappings::ButtonR2_ButtonR3",BTN_QUICKSAVE);
			Settings.ButtonR3_ButtonL3	= currentconfig.GetInt("PS3ButtonMappings::ButtonR3_ButtonL3",BTN_EXITTOMENU);
			Settings.AnalogR_Up		= currentconfig.GetInt("PS3ButtonMappings::AnalogR_Up",BTN_CHEATENABLE);
			Settings.AnalogR_Down		= currentconfig.GetInt("PS3ButtonMappings::AnalogR_Down",BTN_CHEATDISABLE);
			Settings.AnalogR_Left		= currentconfig.GetInt("PS3ButtonMappings::AnalogR_Left",BTN_DECREMENTSAVE);
			Settings.AnalogR_Right		= currentconfig.GetInt("PS3ButtonMappings::AnalogR_Right",BTN_INCREMENTSAVE);

			Settings.AnalogR_Up_Type	= currentconfig.GetBool("PS3ButtonMappings::AnalogR_Up_Type",0);
			Settings.AnalogR_Down_Type	= currentconfig.GetBool("PS3ButtonMappings::AnalogR_Down_Type",0);
			Settings.AnalogR_Left_Type	= currentconfig.GetBool("PS3ButtonMappings::AnalogR_Left_Type",0);
			Settings.AnalogR_Right_Type	= currentconfig.GetBool("PS3ButtonMappings::AnalogR_Right_Type",0);
			break;
		case MAP_BUTTONS_OPTION_DEFAULT:
			Input_MapButton(&Settings.DPad_Up,false,BTN_UP);
			Input_MapButton(&Settings.DPad_Down,false,BTN_DOWN);
			Input_MapButton(&Settings.DPad_Left,false,BTN_LEFT);
			Input_MapButton(&Settings.DPad_Right,false,BTN_RIGHT);
			Input_MapButton(&Settings.ButtonCircle,false,BTN_A);
			Input_MapButton(&Settings.ButtonCross,false,BTN_B);
			Input_MapButton(&Settings.ButtonTriangle,false,BTN_X);
			Input_MapButton(&Settings.ButtonSquare,false,BTN_Y);
			Input_MapButton(&Settings.ButtonSelect,false,BTN_SELECT);
			Input_MapButton(&Settings.ButtonStart,false,BTN_START);
			Input_MapButton(&Settings.ButtonL1,false,BTN_L);
			Input_MapButton(&Settings.ButtonL2,false,BTN_NONE);
			Input_MapButton(&Settings.ButtonR2,false,BTN_NONE);
			Input_MapButton(&Settings.ButtonL3,false,BTN_NONE);
			Input_MapButton(&Settings.ButtonR3,false,BTN_NONE);
			Input_MapButton(&Settings.ButtonR1,false,BTN_R);
			Input_MapButton(&Settings.ButtonL2_ButtonL3,false,BTN_NONE);
			Input_MapButton(&Settings.ButtonL2_ButtonR2,false,BTN_NONE);
			Input_MapButton(&Settings.ButtonL2_ButtonR3,false,BTN_QUICKLOAD);
			Input_MapButton(&Settings.ButtonR2_ButtonR3,false,BTN_QUICKSAVE);
			Input_MapButton(&Settings.AnalogR_Up,false,BTN_CHEATENABLE);
			Input_MapButton(&Settings.AnalogR_Down,false,BTN_CHEATDISABLE);
			Input_MapButton(&Settings.AnalogR_Left,false,BTN_DECREMENTSAVE);
			Input_MapButton(&Settings.AnalogR_Right,false,BTN_INCREMENTSAVE);
			Input_MapButton(&Settings.ButtonL2_AnalogR_Right,false,BTN_INCREMENTCHEAT);
			Input_MapButton(&Settings.ButtonL2_AnalogR_Left,false,BTN_DECREMENTCHEAT);
			Input_MapButton(&Settings.ButtonL2_AnalogR_Up,false,BTN_NONE);
			Input_MapButton(&Settings.ButtonL2_AnalogR_Down,false,BTN_NONE);
			Input_MapButton(&Settings.ButtonR2_AnalogR_Right,false,BTN_NONE);
			Input_MapButton(&Settings.ButtonR2_AnalogR_Left,false,BTN_NONE);
			Input_MapButton(&Settings.ButtonR2_AnalogR_Up,false,BTN_NONE);
			Input_MapButton(&Settings.ButtonR2_AnalogR_Down,false,BTN_NONE);
			Input_MapButton(&Settings.ButtonR3_ButtonL3,false,BTN_EXITTOMENU);
			Settings.AnalogR_Up_Type = 0;
			Settings.AnalogR_Down_Type = 0;
			Settings.AnalogR_Left_Type = 0;
			Settings.AnalogR_Right_Type = 0;
			break;
		case MAP_BUTTONS_OPTION_NEW:
			Input_MapButton(&Settings.DPad_Up,false,BTN_UP);
			Input_MapButton(&Settings.DPad_Down,false,BTN_DOWN);
			Input_MapButton(&Settings.DPad_Left,false,BTN_LEFT);
			Input_MapButton(&Settings.DPad_Right,false,BTN_RIGHT);
			Input_MapButton(&Settings.ButtonCircle,false,BTN_A);
			Input_MapButton(&Settings.ButtonCross,false,BTN_B);
			Input_MapButton(&Settings.ButtonTriangle,false,BTN_X);
			Input_MapButton(&Settings.ButtonSquare,false,BTN_Y);
			Input_MapButton(&Settings.ButtonSelect,false,BTN_SELECT);
			Input_MapButton(&Settings.ButtonStart,false,BTN_START);
			Input_MapButton(&Settings.ButtonL1,false,BTN_L);
			Input_MapButton(&Settings.ButtonL2,false,BTN_NONE);
			Input_MapButton(&Settings.ButtonR2,false,BTN_NONE);
			Input_MapButton(&Settings.ButtonL3,false,BTN_NONE);
			Input_MapButton(&Settings.ButtonR3,false,BTN_NONE);
			Input_MapButton(&Settings.ButtonR1,false,BTN_R);
			Input_MapButton(&Settings.ButtonL2_ButtonL3,false,BTN_LOAD_MOVIE);
			Input_MapButton(&Settings.ButtonL2_ButtonR2,false,BTN_NONE);
			Input_MapButton(&Settings.ButtonL2_ButtonR3,false,BTN_TOGGLE_RECORDING_MOVIE);
			Input_MapButton(&Settings.ButtonR2_ButtonR3,false,BTN_SRAM_WRITEPROTECT);
			Input_MapButton(&Settings.AnalogR_Up,false,BTN_PAUSE);
			Input_MapButton(&Settings.AnalogR_Down,false,BTN_FASTFORWARD);
			Input_MapButton(&Settings.AnalogR_Left,false,BTN_DECREMENTTURBO);
			Input_MapButton(&Settings.AnalogR_Right,false,BTN_INCREMENTTURBO);
			Input_MapButton(&Settings.ButtonL2_AnalogR_Right,false,BTN_INCREMENTCHEAT);
			Input_MapButton(&Settings.ButtonL2_AnalogR_Left,false,BTN_DECREMENTCHEAT);
			Input_MapButton(&Settings.ButtonL2_AnalogR_Up,false,BTN_CHEATTOGGLE);
			Input_MapButton(&Settings.ButtonL2_AnalogR_Down,false,BTN_CHEATINPUT);
			Input_MapButton(&Settings.ButtonR2_AnalogR_Right,false,BTN_INCREMENTSAVE);
			Input_MapButton(&Settings.ButtonR2_AnalogR_Left,false,BTN_DECREMENTSAVE);
			Input_MapButton(&Settings.ButtonR2_AnalogR_Up,false,BTN_QUICKLOAD);
			Input_MapButton(&Settings.ButtonR2_AnalogR_Down,false,BTN_QUICKSAVE);
			Input_MapButton(&Settings.ButtonR3_ButtonL3,false,BTN_EXITTOMENU);
			Settings.AnalogR_Up_Type = 0;
			Settings.AnalogR_Down_Type = 0;
			Settings.AnalogR_Left_Type = 0;
			Settings.AnalogR_Right_Type = 0;
			break;
	}
}

static void cb_save_custom_controls(int button_type, void *userdata)
{
	switch(button_type)
	{
		case CELL_MSGDIALOG_BUTTON_YES:
		Settings.SaveCustomControlScheme = true;
		dialog_is_running = false;
		cellMsgDialogClose(0.0f);
		break;
		case CELL_MSGDIALOG_BUTTON_ESCAPE:
		case CELL_MSGDIALOG_BUTTON_NO:
		Settings.SaveCustomControlScheme = false;
		dialog_is_running = false;
		cellMsgDialogClose(0.0f);
		break;
	}
}

void Emulator_Implementation_SaveCustomControls(bool showdialog)
{
	if(Settings.ControlScheme == CONTROL_SCHEME_CUSTOM)
	{
		if(showdialog)
		{
			dialog_is_running = true;
			cellMsgDialogOpen2(CELL_MSGDIALOG_DIALOG_TYPE_NORMAL|\
			CELL_MSGDIALOG_TYPE_BG_VISIBLE|\
			CELL_MSGDIALOG_TYPE_BUTTON_TYPE_YESNO|\
			CELL_MSGDIALOG_TYPE_DISABLE_CANCEL_OFF|\
			CELL_MSGDIALOG_TYPE_DEFAULT_CURSOR_YES,\
			"Do you want to save the custom controller settings?",\
			cb_save_custom_controls,NULL,NULL);
			while(dialog_is_running && Emulator_ROMRunning())
			{
				Graphics->Clear();
				Graphics->Swap();
				cellSysutilCheckCallback();	
			}
		}
		if(!showdialog || Settings.SaveCustomControlScheme)
		{
			Emulator_Implementation_ButtonMappingSettings(MAP_BUTTONS_OPTION_SETTER);
		}
	}
}

bool Emulator_Implementation_SaveSettings()
{
	currentconfig.SetInt	("PS3General::ControlScheme",Settings.ControlScheme);
	currentconfig.SetBool	("PS3General::KeepAspect",Settings.PS3KeepAspect);
	currentconfig.SetBool	("PS3General::Smooth", Settings.PS3Smooth);
	currentconfig.SetBool	("PS3General::OverscanEnabled", Settings.PS3OverscanEnabled);
	currentconfig.SetInt	("PS3General::OverscanAmount",Settings.PS3OverscanAmount);
	currentconfig.SetInt	("PS3General::PS3FontSize",Settings.PS3FontSize);
	currentconfig.SetBool	("PS3General::Throttled",Settings.Throttled);
	currentconfig.SetBool	("PS3General::PS3PALTemporalMode60Hz",Settings.PS3PALTemporalMode60Hz);
	currentconfig.SetString	("PS3General::PS3CurrentShader",Graphics->GetFragmentShaderPath());
	currentconfig.SetUInt	("PS3General::PS3CurrentResolution",Graphics->GetCurrentResolution());
	currentconfig.SetString	("PS3Paths::PathSaveStates",Settings.PS3PathSaveStates);
	currentconfig.SetString	("PS3Paths::PathSRAM",Settings.PS3PathSRAM);
	currentconfig.SetString	("PS3Paths::PathROMDirectory",Settings.PS3PathROMDirectory);
	currentconfig.SetString	("PS3Paths::PathCheats",Settings.PS3PathCheats);
	currentconfig.SetString	("RSound::RSoundServerIPAddress",Settings.RSoundServerIPAddress);
	currentconfig.SetBool	("RSound::RSoundEnabled",Settings.RSoundEnabled);
	currentconfig.SetUInt	("Sound::InputRate",Settings.SoundInputRate);
	currentconfig.SetBool	("PS3SNES9x::ResetBeforeRecordingMovie",Settings.ResetBeforeRecordingMovie);
	currentconfig.SetInt	("PS3SNES9x::AccessoryAutoDetection",Settings.AccessoryAutoDetection);
	currentconfig.SetBool	("PS3SNES9x::SRAMWriteProtect",Settings.SRAMWriteProtect);
	currentconfig.SetBool	("Sound::Sync",Settings.SoundSync);
	currentconfig.SetBool	("ROM::PAL",Settings.ForcePAL);
	currentconfig.SetBool	("ROM::NTSC",Settings.ForceNTSC);
	currentconfig.SetBool	("ROM::Cheat",Settings.ApplyCheats);
	currentconfig.SetBool	("ROM::Patch",!Settings.NoPatch);
	currentconfig.SetBool	("Display::DisplayFrameRate",Settings.DisplayFrameRate);
	currentconfig.SetBool	("Display::GraphicWindows",!Settings.DisableGraphicWindows);
	currentconfig.SetBool	("Display::Transparency",Settings.Transparency);
	currentconfig.SetBool	("Display::DisplayInput",Settings.DisplayPressedKeys);
	currentconfig.SetBool	("Display::MessagesInImage",Settings.AutoDisplayMessages);
	currentconfig.SetBool	("Settings::BSXBootup",Settings.BSXBootup);
	currentconfig.SetInt	("Settings::TurboFrameSkip",Settings.TurboSkipFrames);
	Emulator_Implementation_SaveCustomControls(false);
	return currentconfig.SaveTo(SYS_CONFIG_FILE);
}


void Emulator_Implementation_MuteSound(bool paused)
{
	S9xSetSoundMute(paused);
}

void Emulator_Implementation_SwitchControlScheme()
{
	switch(Settings.ControlScheme)
	{
		case CONTROL_SCHEME_DEFAULT:
			Emulator_Implementation_ButtonMappingSettings(MAP_BUTTONS_OPTION_DEFAULT);
			break;
		case CONTROL_SCHEME_NEW:
			Emulator_Implementation_ButtonMappingSettings(MAP_BUTTONS_OPTION_NEW);
			break;
		case CONTROL_SCHEME_CUSTOM:
			Emulator_Implementation_ButtonMappingSettings(MAP_BUTTONS_OPTION_GETTER);
			break;
	}
}

void S9xParsePortConfig (ConfigFile &conf, int pass)
{
	currentconfig = conf;
	if (pass == 1)
	{
		//Some additional settings that are not properly set by SNES9x's LoadConfigFiles
		if (currentconfig.Exists("Settings::FrameTimePAL"))
		{
			Settings.FrameTimePAL = conf.GetUInt("Settings::FrameTimePAL", 20000);
		}

		//PS3 - General Settings
		if (currentconfig.Exists("PS3General::KeepAspect"))
		{
			Settings.PS3KeepAspect		= currentconfig.GetBool("PS3General::KeepAspect");
		}
		else
		{
			Settings.PS3KeepAspect		= TRUE;
		}

		if (currentconfig.Exists("PS3General::Smooth"))
		{
			Settings.PS3Smooth		= currentconfig.GetBool("PS3General::Smooth");
		}
		else
		{
			Settings.PS3Smooth		= TRUE;
		}

		if (currentconfig.Exists("PS3General::PS3CurrentShader"))
		{
			Settings.PS3CurrentShader.assign(currentconfig.GetString("PS3General::PS3CurrentShader"));
		}
		else
		{
			Settings.PS3CurrentShader.assign(DEFAULT_SHADER_FILE);
		}

		if (currentconfig.Exists("PS3General::PS3CurrentResolution"))
		{
			Settings.PS3CurrentResolution = currentconfig.GetInt("PS3General::PS3CurrentResolution");
		}
		else
		{
			Settings.PS3CurrentResolution = NULL;
		}

		if (currentconfig.Exists("PS3General::OverscanEnabled"))
		{
			Settings.PS3OverscanEnabled	= currentconfig.GetBool("PS3General::OverscanEnabled");
		}
		else
		{
			Settings.PS3OverscanEnabled	= FALSE;
		}

		if (currentconfig.Exists("PS3General::OverscanAmount"))
		{
			Settings.PS3OverscanAmount	= currentconfig.GetInt("PS3General::OverscanAmount");
		}
		else
		{
			Settings.PS3OverscanAmount	= 0;
		}

		if (currentconfig.Exists("PS3General::PS3PALTemporalMode60Hz"))
		{
			Settings.PS3PALTemporalMode60Hz = currentconfig.GetBool("PS3General::PS3PALTemporalMode60Hz");
		}
		else
		{
			Settings.PS3PALTemporalMode60Hz = false;
		}

		//SNES9x PS3 Settings
		if(currentconfig.Exists("PS3SNES9x::ResetBeforeRecordingMovie"))
		{
			Settings.ResetBeforeRecordingMovie	= currentconfig.GetBool("PS3SNES9x::ResetBeforeRecordingMovie");
		}
		else
		{
			Settings.ResetBeforeRecordingMovie	= false;
		}

		if(currentconfig.Exists("PS3SNES9x::AccessoryAutoDetection"))
		{
			Settings.AccessoryAutoDetection	= currentconfig.GetInt("PS3SNES9x::AccessoryAutoDetection");
		}
		else
		{
			Settings.AccessoryAutoDetection	= ACCESSORY_AUTODETECTION_CONFIRM;
		}

		if(currentconfig.Exists("PS3SNES9x::SRAMWriteProtect"))
		{
			Settings.SRAMWriteProtect	= currentconfig.GetBool("PS3SNES9x::SRAMWriteProtect");
		}
		else
		{
			Settings.SRAMWriteProtect	= false;
		}
	
		//RSound Settings
		if(currentconfig.Exists("RSound::RSoundEnabled"))
		{
			Settings.RSoundEnabled		= currentconfig.GetBool("RSound::RSoundEnabled");
		}
		else
		{
			Settings.RSoundEnabled		= FALSE;
		}

		if(currentconfig.Exists("RSound::RSoundServerIPAddress"))
		{
			Settings.RSoundServerIPAddress	= currentconfig.GetString("RSound::RSoundServerIPAddress");
		}
		else
		{
			Settings.RSoundServerIPAddress = "0.0.0.0";
		}

		if(currentconfig.Exists("PS3General::Throttled"))
		{
			Settings.Throttled		= currentconfig.GetBool("PS3General::Throttled");
		}
		else
		{
			Settings.Throttled		= true;
		}

		if(currentconfig.Exists("PS3General::PS3FontSize"))
		{
			Settings.PS3FontSize		= currentconfig.GetInt("PS3General::PS3FontSize");
		}
		else
		{
			Settings.PS3FontSize		= 100;
		}
		
		// PS3 Path Settings
		if ((currentconfig.Exists("PS3Paths::PathSaveStates")) && strlen(currentconfig.GetString("PS3Paths::PathSaveStates")) > 0)
		{
			Settings.PS3PathSaveStates		= currentconfig.GetString("PS3Paths::PathSaveStates");
		}
		else
		{
			Settings.PS3PathSaveStates		= USRDIR;
		}

		// PS3 Path Cheat Settings
		if ((currentconfig.Exists("PS3Paths::PathCheats")) && strlen(currentconfig.GetString("PS3Paths::PathCheats")) > 0)
		{
			Settings.PS3PathCheats		= currentconfig.GetString("PS3Paths::PathCheats");
		}
		else
		{
			Settings.PS3PathCheats		= USRDIR;
		}
		if (currentconfig.Exists("PS3Paths::PathSRAM"))
		{
			Settings.PS3PathSRAM		= currentconfig.GetString("PS3Paths::PathSRAM");
		}
		
		 /*
		 if (currentconfig.Exists("PS3Paths::PathScreenshots"))
		 {
		 Settings.PS3PathScreenshots		= currentconfig.GetString("PS3Paths::PathScreenshots");
		 }
		 */
		if (currentconfig.Exists("PS3Paths::PathROMDirectory"))
		{
			Settings.PS3PathROMDirectory		= currentconfig.GetString("PS3Paths::PathROMDirectory");
		}
		else
		{
			Settings.PS3PathROMDirectory	= "/";
		}
		if (currentconfig.Exists("Settings::TurboFrameSkip"))
		{
			Settings.TurboSkipFrames = currentconfig.GetInt("Settings::TurboFrameSkip");
		}
		else
		{
			Settings.TurboSkipFrames = 15;
		}
		if (currentconfig.Exists("PS3General::ControlScheme"))
		{
			Settings.ControlScheme = currentconfig.GetInt("PS3General::ControlScheme");
		}
		else
		{
			Settings.ControlScheme = CONTROL_SCHEME_DEFAULT;
		}
		Emulator_Implementation_SwitchControlScheme();
	}
}

void S9xExtraUsage (void)
{
}

void S9xParseArg (char **argv, int &i, int argc)
{
}

bool S9xPollAxis (uint32_t id, int16_t *value)
{
	return false;
}

bool S9xPollButton (uint32_t id, bool *pressed)
{
	return false;
}

bool S9xPollPointer (uint32_t id, int16_t *x, int16_t *y)
{
	return false;
}

void S9xHandlePortCommand (s9xcommand_t cmd, int16_t data1, int16_t data2)
{
}


const char *S9xStringInput (const char *message)
{
	static char	buffer[256];
	
	printf("%s: ", message);
	fflush(stdout);

	if (fgets(buffer, sizeof(buffer) - 2, stdin))
	{
		return (buffer);
	}

	return (NULL);
}

bool8 S9xOpenSnapshotFile (const char *filepath, bool8 read_only, STREAM *file)
{
	if(read_only)
	{
		if((*file = OPEN_STREAM(filepath, "rb")) != 0)
		{
			return (TRUE);
		}
	}
	else
	{
		if((*file = OPEN_STREAM(filepath, "wb")) != 0)
		{
			return (TRUE);
		}
	}
	return (FALSE);
}

void S9xCloseSnapshotFile (STREAM file)
{
	CLOSE_STREAM (file);
}

void S9xExit (void)
{
}

bool8 S9xInitUpdate (void)
{
	return true;
}

static inline int64_t isel(int64_t a, int64_t b, int64_t c)
{
   int64_t mask = c >> 63;
   return (a & ~mask) | (b & mask);
}

static inline int64_t isel_if(int64_t a, int64_t b, int64_t c)
{
   int64_t abs = isel(c, -c, c) - 1;
   return isel(a, b, abs);
}

bool8 S9xDeinitUpdate (int width, int height)
{
	Graphics->Draw(width, isel_if(height,224,239 - height + Settings.OddScreenHeightJumpFix), GFX.Screen);
	return true;
}

bool8 S9xContinueUpdate (int width, int height)
{
	return true;
}

void S9xMessage (int type, int number, char const *message)
{
	S9xSetInfoString(message);
}

bool8 S9xOpenSoundDevice (void) {
	return true;
}

const char * S9xGetFilename (const char *ex, enum s9x_getdirtype dirtype)
{
	static char	s[PATH_MAX + 1];
	char		drive[_MAX_DRIVE + 1];
	char		dir[_MAX_DIR + 1];
	char		fname[_MAX_FNAME + 1];
	char		ext[_MAX_EXT + 1];

	_splitpath(Memory.ROMFilename, drive, dir, fname, ext);
	snprintf(s, PATH_MAX + 1, "%s%s%s%s", S9xGetDirectory(dirtype), SLASH_STR, fname, ex);

	return (s);
}

const char * S9xGetFilenameInc (const char *ex, enum s9x_getdirtype dirtype)
{
	static char	s[PATH_MAX + 1];
	char		drive[_MAX_DRIVE + 1];
	char		dir[_MAX_DIR + 1];
	char		fname[_MAX_FNAME + 1];
	char		ext[_MAX_EXT + 1];

	unsigned int	i = 0;
	const char	*d;

	_splitpath(Memory.ROMFilename, drive, dir, fname, ext);
	d = S9xGetDirectory(dirtype);

	snprintf(s, PATH_MAX + 1, "%s%s%s.%03d%s", d, SLASH_STR, fname, i++, ex);

	return (s);
}

const char * S9xBasename (const char *f)
{
	const char	*p;

	if ((p = strrchr(f, '/')) != NULL || (p = strrchr(f, '\\')) != NULL)
	{
		return (p + 1);
	}

	return (f);
}

const char *S9xGetDirectory (enum s9x_getdirtype dirtype) {
	static char s[PATH_MAX + 1];

	switch (dirtype)
	{
		case SNAPSHOT_DIR:
			return Settings.PS3PathSaveStates.c_str();
			break;
		case CHEAT_DIR:
			return Settings.PS3PathCheats.c_str();
			break;
		case SRAM_DIR:
			if(!Settings.PS3PathSRAM.empty())
			{
				return Settings.PS3PathSRAM.c_str();
				break;
			}
		case ROMFILENAME_DIR:
			strncpy(s, Memory.ROMFilename, PATH_MAX + 1);
			s[PATH_MAX] = 0;

			for (unsigned int i = strlen(s); i >= 0; --i)
			{
				if (s[i] == SLASH_CHAR)
				{
					s[i] = 0;
					break;
				}
			}

			return s;
			break;
		default:
			return USRDIR;
	}
}

const char *S9xChooseFilename (bool8 read_only)
{
	static char	filename[PATH_MAX + 1];
	static char	drive[_MAX_DRIVE + 1];
	static char	dir[_MAX_DIR + 1];
	static char	def[_MAX_FNAME + 1];
	static char	ext[_MAX_EXT + 1];

	_splitpath(Memory.ROMFilename, drive, dir, def, ext);
	snprintf(filename, PATH_MAX + 1, "%s%s%s.%03d", S9xGetDirectory(SNAPSHOT_DIR), SLASH_STR, def, Emulator_Implementation_CurrentSaveStateSlot());

	return filename;
}

const char * S9xChooseMovieFilename (bool8 read_only)
{
	char	s[PATH_MAX + 1];
	char	drive[_MAX_DRIVE + 1], dir[_MAX_DIR + 1], fname[_MAX_FNAME + 1], ext[_MAX_EXT + 1];

	char	title[64];

	_splitpath(Memory.ROMFilename, drive, dir, fname, ext);
	sprintf(title, "Choose movie %s filename", read_only ? "playback" : "record");
	const char	*filename;

	S9xSetSoundMute(TRUE);
	filename = S9xGetFilename(".smv", SNAPSHOT_DIR);
	S9xSetSoundMute(FALSE);

	return (filename);
}

void S9xAutoSaveSRAM (void)
{
	if(!Settings.SRAMWriteProtect)
	{
		Memory.SaveSRAM(S9xGetFilename(".srm", SRAM_DIR));
	}
}

void S9xToggleSoundChannel (int c)
{
	static uint8	sound_switch = 255;
	
	if (c == 8)
	{
		sound_switch = 255;
	}
	else
	{
		sound_switch ^= 1 << c;
	}

	S9xSetSoundControl(sound_switch);
}

void S9xSetPalette (void)
{
}

void S9xSyncSpeed (void)
{
   if (Settings.Throttled)
   {
      while (!S9xSyncSound())
      {
         sys_timer_usleep(500);
         // Ye, everyone and their mothers hate busywaiting, but s9x doesn't give us a choice :(
      }
   }
   else
   {
      // Drain the sound buffer, but do not use it.
      uint8 buf[1024];
      while (S9xGetSampleCount() >= 512)
         S9xMixSamples(buf, 512);
   }

	if(Settings.HighSpeedSeek > 0)
	{
		Settings.HighSpeedSeek--;
	}

	if(Settings.TurboMode)
	{
		if (++IPPU.FrameSkip >= Settings.TurboSkipFrames)
		{
			IPPU.FrameSkip = 0;
			IPPU.SkippedFrames = 0;
			IPPU.RenderThisFrame = true;
		}
		else
		{
			IPPU.SkippedFrames++;
			IPPU.RenderThisFrame = false;
		}
		
		return;
	}
	else
	{
		IPPU.RenderThisFrame = true;
	}
}


void _splitpath (const char *path, char *drive, char *dir, char *fname, char *ext)
{
	*drive = 0;

	const char	*slash = strrchr(path, SLASH_CHAR);
	const char	*dot   = strrchr(path, '.');

	if (dot && slash && dot < slash)
	{
		dot = NULL;
	}

	if (!slash)
	{
		*dir = 0;

		strcpy(fname, path);

		if (dot)
		{
			fname[dot - path] = 0;
			strcpy(ext, dot + 1);
		}
		else
		{
			*ext = 0;
		}
	}
	else
	{
		strcpy(dir, path);
		dir[slash - path] = 0;

		strcpy(fname, slash + 1);

		if (dot)
		{
			fname[dot - slash - 1] = 0;
			strcpy(ext, dot + 1);
		}
		else
		{
			*ext = 0;
		}
	}
}

void _makepath (char *path, const char *, const char *dir, const char *fname, const char *ext)
{
	if (dir && *dir)
	{
		strcpy(path, dir);
		strcat(path, SLASH_STR);
	}
	else
	{
		*path = 0;
	}

	strcat(path, fname);

	if (ext && *ext)
	{
		strcat(path, ".");
		strcat(path, ext);
	}
}
