/******************************************************************************* 
 * menu.cpp - SNES9x PS3
 *
 *  Created on: Oct 10, 2010
 *      Author: halsafar
 *	Author: Squarepusher2
********************************************************************************/

#include <string.h>
#include <stack>
#include <vector>

#include "snes9x.h"

#include "main.h"
#include "emulator_implementation.h"
#include "SNES9xGraphics.h"
#include "menu.h"
#include "cellframework/input/cellInput.h"
#include "cellframework/fileio/FileBrowser.h"
#include "cellframework/logger/Logger.h"

#include <cell/pad.h>

#include <cell/audio.h>
#include <cell/sysmodule.h>
#include <cell/cell_fs.h>
#include <cell/dbgfont.h>
#include <sysutil/sysutil_sysparam.h>

#define MIN(x,y) ((x) < (y) ? (x) : (y))
#define MAX(x,y) ((x) > (y) ? (x) : (y))

#define NUM_ENTRY_PER_PAGE 22
#define NUM_ENTRY_PER_SETTINGS_PAGE 18

// is the menu running
bool menuRunning = false;

// menu to render
typedef void (*curMenuPtr)();

//
std::stack<curMenuPtr> menuStack;

// main file browser->for rom browser
FileBrowser* browser = NULL;

// tmp file browser->for everything else
FileBrowser* tmpBrowser = NULL;

int16_t currently_selected_setting =		FIRST_GENERAL_SETTING;
int16_t currently_selected_snes9x_setting =	FIRST_SNES9X_SETTING;
int16_t currently_selected_path_setting = 	FIRST_PATH_SETTING;
int16_t currently_selected_controller_setting = FIRST_CONTROLS_SETTING;

#define FILEBROWSER_DELAY	100000
#define SETTINGS_DELAY		150000	

void MenuStop()
{
	menuRunning = false;
}


bool MenuIsRunning()
{
	return menuRunning;
}

void UpdateBrowser(FileBrowser* b)
{
	if (CellInput->WasButtonHeld(0,CTRL_DOWN) | CellInput->IsAnalogPressedDown(0,CTRL_LSTICK))
	{
		if(b->GetCurrentEntryIndex() < b->GetCurrentDirectoryInfo().numEntries-1)
		{
			b->IncrementEntry();
			if(CellInput->IsButtonPressed(0,CTRL_DOWN))
			{
				sys_timer_usleep(FILEBROWSER_DELAY);
			}
		}
	}
	if (CellInput->WasButtonHeld(0,CTRL_UP) | CellInput->IsAnalogPressedUp(0,CTRL_LSTICK))
	{
		if(b->GetCurrentEntryIndex() > 0)
		{
			b->DecrementEntry();
			if(CellInput->IsButtonPressed(0,CTRL_UP))
			{
				sys_timer_usleep(FILEBROWSER_DELAY);
			}
		}
	}
	if (CellInput->WasButtonHeld(0,CTRL_RIGHT) | CellInput->IsAnalogPressedRight(0,CTRL_LSTICK))
	{
		b->GotoEntry(MIN(b->GetCurrentEntryIndex()+5, b->GetCurrentDirectoryInfo().numEntries-1));
		if(CellInput->IsButtonPressed(0,CTRL_RIGHT))
		{
			sys_timer_usleep(FILEBROWSER_DELAY);
		}
	}
	if (CellInput->WasButtonHeld(0, CTRL_LEFT) | CellInput->IsAnalogPressedLeft(0,CTRL_LSTICK))
	{
		if (b->GetCurrentEntryIndex() <= 5)
		{
			b->GotoEntry(0);
			if(CellInput->IsButtonPressed(0,CTRL_LEFT))
			{
				sys_timer_usleep(FILEBROWSER_DELAY);
			}
		}
		else
		{
			b->GotoEntry(b->GetCurrentEntryIndex()-5);
			if(CellInput->IsButtonPressed(0,CTRL_LEFT))
			{
				sys_timer_usleep(FILEBROWSER_DELAY);
			}
		}
	}
	if (CellInput->WasButtonHeld(0,CTRL_R1))
	{
		b->GotoEntry(MIN(b->GetCurrentEntryIndex()+NUM_ENTRY_PER_PAGE, b->GetCurrentDirectoryInfo().numEntries-1));
		if(CellInput->IsButtonPressed(0,CTRL_R1))
		{
			sys_timer_usleep(FILEBROWSER_DELAY);
		}
	}
	if (CellInput->WasButtonHeld(0,CTRL_L1))
	{
		if (b->GetCurrentEntryIndex() <= NUM_ENTRY_PER_PAGE)
		{
			b->GotoEntry(0);
		}
		else
		{
			b->GotoEntry(b->GetCurrentEntryIndex()-NUM_ENTRY_PER_PAGE);
		}
		if(CellInput->IsButtonPressed(0,CTRL_L1))
		{
			sys_timer_usleep(FILEBROWSER_DELAY);
		}
	}

	if (CellInput->WasButtonPressed(0, CTRL_CIRCLE))
	{
		// don't let people back out past root
		if (b->DirectoryStackCount() > 1)
		{
			b->PopDirectory();
		}
	}
}

void RenderBrowser(FileBrowser* b)
{
	uint32_t file_count = b->GetCurrentDirectoryInfo().numEntries;
	int current_index = b->GetCurrentEntryIndex();

	if (file_count <= 0)
	{
		LOG_DBG("1: filecount <= 0");
	}
	else if (current_index > file_count)
	{
		LOG_DBG("2: current_index >= file_count");
	}
	else
	{
		int page_number = current_index / NUM_ENTRY_PER_PAGE;
		int page_base = page_number * NUM_ENTRY_PER_PAGE;
		float currentX = 0.09f;
		float currentY = 0.09f;
		float ySpacing = 0.035f;
		for (int i = page_base; i < file_count && i < page_base + NUM_ENTRY_PER_PAGE; ++i)
		{
			currentY = currentY + ySpacing;
			cellDbgFontPuts(currentX, currentY, Emulator_GetFontSize(),
					i == current_index ? RED : (*b)[i]->d_type == CELL_FS_TYPE_DIRECTORY ? GREEN : WHITE,
					(*b)[i]->d_name);
			Graphics->FlushDbgFont();
		}
	}
	Graphics->FlushDbgFont();
}

void do_shaderChoice()
{
	if (tmpBrowser == NULL)
	{
		tmpBrowser = new FileBrowser("/dev_hdd0/game/SNES90000/USRDIR/shaders/\0");
	}

	string path;

	if (CellInput->UpdateDevice(0) == CELL_PAD_OK)
	{
		UpdateBrowser(tmpBrowser);

		if (CellInput->WasButtonPressed(0,CTRL_CROSS))
		{
			if(tmpBrowser->IsCurrentADirectory())
			{
				tmpBrowser->PushDirectory(	tmpBrowser->GetCurrentDirectoryInfo().dir + "/" + tmpBrowser->GetCurrentEntry()->d_name,
						CELL_FS_TYPE_REGULAR | CELL_FS_TYPE_DIRECTORY, "cg");
			}
			else if (tmpBrowser->IsCurrentAFile())
			{
				path = tmpBrowser->GetCurrentDirectoryInfo().dir + "/" + tmpBrowser->GetCurrentEntry()->d_name;

				//load shader
				Graphics->LoadFragmentShader(path);
				menuStack.pop();
			}
		}

		if (CellInput->WasButtonHeld(0, CTRL_TRIANGLE))
		{
			menuStack.pop();
		}
	}

	cellDbgFontPrintf(0.09f, 0.09f, Emulator_GetFontSize(), YELLOW, "PATH: %s", tmpBrowser->GetCurrentDirectoryInfo().dir.c_str());
	cellDbgFontPuts	(0.09f,	0.05f,	Emulator_GetFontSize(),	RED,	"SHADER SELECTION");
	cellDbgFontPuts(0.09f, 0.92f, Emulator_GetFontSize(), YELLOW,
	"X - Select shader               /\\ - return to settings");
	cellDbgFontPrintf(0.09f, 0.89f, 0.86f, LIGHTBLUE, "%s",
	"INFO - Select a shader from the menu by pressing the X button. ");
	Graphics->FlushDbgFont();

	RenderBrowser(tmpBrowser);
}

void DisplayHelpMessage(int currentsetting)
{
	switch(currentsetting)
	{
		case SETTING_CURRENT_SAVE_STATE_SLOT:
			cellDbgFontPrintf(0.09f, 0.83f, 0.86f, LIGHTBLUE, "%s", "INFO - Set the current savestate slot (can also be configured ingame).\n");
			break;
		case SETTING_PATH_SAVESTATES_DIRECTORY:
			cellDbgFontPrintf(0.09f, 0.83f, 0.86f, LIGHTBLUE, "%s", "INFO - Set the default path where all the savestate files will be saved.\n");
			break;
		case SETTING_PATH_DEFAULT_ROM_DIRECTORY:
			cellDbgFontPrintf(0.09f, 0.83f, 0.86f, LIGHTBLUE, "%s", "INFO - Set the default ROM startup directory. NOTE: You will have to\nrestart the emulator for this change to have any effect.\n");
			break;
		case SETTING_PATH_SRAM_DIRECTORY:
			cellDbgFontPrintf(0.09f, 0.83f, 0.86f, LIGHTBLUE, "%s", "INFO - Set the default SRAM (SaveRAM) directory path.\n");
			break;
		case SETTING_PATH_CHEATS:
			cellDbgFontPrintf(0.09f, 0.83f, 0.86f, LIGHTBLUE, "%s", "INFO - Set the default cheatfile directory path. All CHT files\nwill be stored here.\n");
			break;
		case SETTING_SHADER:
			cellDbgFontPrintf(0.09f, 0.83f, 0.86f, LIGHTBLUE, "%s", "INFO - Select a pixel shader. NOTE: Some shaders might be too slow at 1080p.\nIf you experience any slowdown, try another shader.");
			break;
		case SETTING_SNES9X_FPS:
			cellDbgFontPrintf(0.09f, 0.83f, 0.86f, LIGHTBLUE, "%s", Settings.DisplayFrameRate ? "INFO - Display Framerate is set to 'Enabled' - an FPS counter is displayed onscreen." : "INFO - Display Framerate is set to 'Disabled'.");
			break;
		case SETTING_SNES9X_SOUND_INPUT_RATE:
			cellDbgFontPrintf(0.09f, 0.83f, 0.86f, LIGHTBLUE, "%s", "INFO - Set the Sound Inputrate. The default value (31942) is ideal for A/V\nsynchronization.");
			break;
		case SETTING_SNES9X_TRANSPARENCY:
			cellDbgFontPrintf(0.09f, 0.83f, 0.86f, LIGHTBLUE, "%s", Settings.Transparency ? "INFO - Transparency is set to 'On'. This is the default recommended setting." : "INFO - Transparency is set to 'Off'. Certain transparency features used\nby the SNES in games will not be displayed.");
			break;
		case SETTING_SNES9X_SKIP_FRAMES:
			cellDbgFontPrintf(0.09f, 0.83f, 0.86f, LIGHTBLUE, "%s", "INFO - Set frameskipping. NOTE: This option doesn't work at the moment.");
			break;
		case SETTING_SNES9X_DISABLE_GRAPHIC_WINDOWS:
			cellDbgFontPrintf(0.09f, 0.83f, 0.86f, LIGHTBLUE, "%s", Settings.DisableGraphicWindows ? "INFO - Graphic windows is set to 'Off'. Certain graphical features used\nby the SNES in games will be disabled." : "INFO - Graphic windows is set to 'On'. This is the default recommended setting.");
			break;
		case SETTING_SNES9X_DISPLAY_PRESSED_KEYS:
			cellDbgFontPrintf(0.09f, 0.83f, 0.86f, LIGHTBLUE, "%s", Settings.DisplayPressedKeys ? "INFO - Display Pressed Keys is set to 'On'. Button input will be displayed\non-screen." : "INFO - Display Pressed Keys is set to 'Off'.");
			break;
		case SETTING_SNES9X_FORCE_PAL:
			cellDbgFontPrintf(0.09f, 0.83f, 0.86f, LIGHTBLUE, "%s", Settings.ForcePAL ? "INFO - Force PAL is set to 'On' - this will force NTSC games to run at PAL\nfrequencies (50Hz). NOTE: This needs to be set per-game." : "INFO - Force PAL is set to 'Off'.");
			break;
		case SETTING_SNES9X_FORCE_NTSC:
			if(Settings.ForceNTSC)
			{
				cellDbgFontPrintf(0.09f, 0.83f, 0.86f, LIGHTBLUE, "%s", "INFO - Force NTSC is set to 'On' - this will force PAL games to run at\nNTSC frequencies (60Hz). NOTE: This needs to be set per-game.");
			}
			else
			{
				cellDbgFontPrintf(0.09f, 0.83f, 0.86f, LIGHTBLUE, "%s", "INFO - Force NTSC is set to 'Off'.");
			}
			break;
		case SETTING_SNES9X_PAL_TIMING:
			cellDbgFontPrintf(0.09f, 0.83f, 0.86f, LIGHTBLUE, "%s", "INFO - Change PAL timing frequency. NOTE: Doesn't need changing from the\ndefault value.");
			break;
		case SETTING_SNES9X_AUTO_APPLY_CHEATS:
			cellDbgFontPrintf(0.09f, 0.83f, 0.86f, LIGHTBLUE, "%s", Settings.ApplyCheats ? "INFO - Auto-apply cheats is set to 'On'. Automatically apply cheat files if it\nhas the same name as the ROM." : "INFO - Auto-apply cheats is set to 'Off'. Cheat files will not be applied.");
			break;
		case SETTING_SNES9X_AUTO_APPLY_PATCH:
			if(Settings.NoPatch == false)
			{
			cellDbgFontPrintf(0.09f, 0.83f, 0.86f, LIGHTBLUE, "%s","INFO - Auto-apply IPS/UPS patch is set to 'On'. Automatically applies similarly\nnamed patch files if they are in the same directory as the ROM.");
			}
			else
			{
			cellDbgFontPrintf(0.09f, 0.83f, 0.86f, LIGHTBLUE, "%s", "INFO - Auto-apply IPS/UPS patch is set to 'Off'. Translation/hack IPS/UPS\npatches will not be applied.");
			}
			break;
		case SETTING_SNES9X_RESET_BEFORE_RECORDING_MOVIE:
			if(Settings.ResetBeforeRecordingMovie)
			{
			cellDbgFontPrintf(0.09f, 0.83f, 0.86f, LIGHTBLUE, "%s","INFO - Reset Before Recording Movie is set to 'On'. The game will reset before\nbeginning to record a movie.");
			}
			else
			{
			cellDbgFontPrintf(0.09f, 0.83f, 0.86f, LIGHTBLUE, "%s", "INFO - Reset Before Recording Movie is set to 'Off'. The game will start\nfrom the current position in the game while recording a movie.");
			}
			break;
		case SETTING_SNES9X_TURBO_SKIP_FRAMES:
			cellDbgFontPrintf(0.09f, 0.83f, 0.86f, LIGHTBLUE, "%s", "INFO - Set the amount of frames to be skipped during 'Fast Forward'.");
			break;
		case SETTING_SNES9X_SRAM_WRITEPROTECT:
			cellDbgFontPrintf(0.09f, 0.83f, 0.86f, LIGHTBLUE, "%s", Settings.SRAMWriteProtect ? "INFO - [SRAM Overwrite Protect] is set to 'ON'. If you load a savestate\n,it will not overwrite the SRAM. (NOTE: SRAM is not saved)" : "INFO - [SRAM Overwrite Protect] is set to 'OFF'. Savestates that you\n load will immediately overwrite SRAM.");
			break;
		case SETTING_SNES9X_ACCESSORY_AUTODETECTION:
			if(Settings.AccessoryAutoDetection == ACCESSORY_AUTODETECTION_ENABLED)
			{
				cellDbgFontPrintf(0.09f, 0.83f, 0.86f, LIGHTBLUE, "%s", "INFO - Accessory Autodetection is set to 'ON'. Games which support\nMouse/Scope/Multitap will be automatically detected and enabled.");
			}
			else if(Settings.AccessoryAutoDetection == ACCESSORY_AUTODETECTION_CONFIRM)
			{
				cellDbgFontPrintf(0.09f, 0.83f, 0.86f, LIGHTBLUE, "%s", "INFO - Accessory Autodetection is set to 'Confirm'. When detecting a Mouse/\nScope/Multitap-compatible game, you are asked if you want to enable it.");
			}
			else if(Settings.AccessoryAutoDetection == ACCESSORY_AUTODETECTION_NONE)
			{
				cellDbgFontPrintf(0.09f, 0.83f, 0.86f, LIGHTBLUE, "%s", "INFO - Accessory Autodetection is set to 'OFF'. Accessories will not be\ndetected or enabled - standard two joypad controls.");
			}
		break;
		case SETTING_CHANGE_RESOLUTION:
			cellDbgFontPrintf(0.09f, 0.83f, 0.86f, LIGHTBLUE, "%s", "INFO - Change the display resolution - press X to confirm.");
			#ifndef PS3_SDK_3_41
				cellDbgFontPrintf(0.09f, 0.86f, 0.86f, RED, "%s", "WARNING - This setting might not work correctly on 1.92 FW.");
			#endif
			break;
/*
		case SETTING_PAL60_MODE:
			cellDbgFontPrintf(0.09f, 0.83f, 0.86f, LIGHTBLUE, "%s", Settings.PS3PALTemporalMode60Hz ? "INFO - PAL 60Hz mode is enabled - 60Hz NTSC games will run correctly at 576p PAL\nresolution. NOTE: This is configured on-the-fly." : "INFO - PAL 60Hz mode disabled - 50Hz PAL games will run correctly at 576p PAL\nresolution. NOTE: This is configured on-the-fly.");
			break;
*/
		case SETTING_HW_TEXTURE_FILTER:
			cellDbgFontPrintf(0.09f, 0.83f, 0.86f, LIGHTBLUE, "%s", Settings.PS3Smooth ? "INFO - Hardware filtering is set to 'Bilinear filtering'." : "INFO - Hardware filtering is set to 'Point filtering' - most shaders\nlook much better on this setting.");
			break;
		case SETTING_KEEP_ASPECT_RATIO:
			cellDbgFontPrintf(0.09f, 0.83f, 0.86f, LIGHTBLUE, "%s", Settings.PS3KeepAspect ? "INFO - Aspect ratio is set to 'Scaled' - screen will have black borders\nleft and right on widescreen TVs/monitors." : "INFO - Aspect ratio is set to 'Stretched' - widescreen TVs/monitors will have\na full stretched picture.");
			break;
		case SETTING_RSOUND_ENABLED:
			cellDbgFontPrintf(0.09f, 0.83f, 0.86f, LIGHTBLUE, "%s", Settings.RSoundEnabled ? "INFO - Sound is set to 'RSound' - the sound will be streamed over the network\nto the RSound audio server." : "INFO - Sound is set to 'Normal' - normal audio output will be used.");
			break;
		case SETTING_RSOUND_SERVER_IP_ADDRESS:
			cellDbgFontPuts(0.09f, 0.83f, 0.86f, LIGHTBLUE, "INFO - Enter the IP address of the RSound audio server. IP address must be\nan IPv4 32-bits address, eg: '192.168.0.7'.");
			break;
		case SETTING_THROTTLE_MODE:
			cellDbgFontPrintf(0.09f, 0.83f, 0.86f, LIGHTBLUE, "%s", Settings.Throttled ? "INFO - [Throttle mode] is set to 'ON' - VSync is enabled and sound is turned on." : "INFO - [Throttle Mode] is set to 'OFF' - VSync is disabled and sound is turned off.\nNOTE: Useful for benchmarking (measure raw FPS).");
			break;
		case SETTING_FONT_SIZE:
			cellDbgFontPuts(0.09f, 0.83f, 0.86f, LIGHTBLUE, "INFO - Increase or decrease the font size in the menu.");
			break;
		case SETTING_HW_OVERSCAN_AMOUNT:
			cellDbgFontPuts(0.09f, 0.83f, 0.86f, LIGHTBLUE, "INFO - Adjust or decrease overscan. Set this to higher than 0.000\nif the screen doesn't fit on your TV/monitor.");
			break;
		case SETTING_DEFAULT_ALL:
			cellDbgFontPuts(0.09f, 0.83f, 0.86f, LIGHTBLUE, "INFO - Set all 'General' settings back to their default values.");
			break;
		case SETTING_SNES9X_DEFAULT_ALL:
			cellDbgFontPuts(0.09f, 0.83f, 0.86f, LIGHTBLUE, "INFO - Set all 'SNES9x' settings back to their default values.");
			break;
		case SETTING_PATH_DEFAULT_ALL:
			cellDbgFontPuts(0.09f, 0.83f, 0.86f, LIGHTBLUE, "INFO - Set all 'Path' settings back to their default values.");
			break;
		case SETTING_CONTROLS_SCHEME:
			cellDbgFontPrintf(0.09f, 0.83f, 0.86f, LIGHTBLUE,
				Settings.ControlScheme == CONTROL_SCHEME_DEFAULT ? "INFO - Control scheme [Default] is selected.\nNOTE: You can't customize the controls with this scheme." : Settings.ControlScheme == CONTROL_SCHEME_NEW ? "INFO - Control scheme [New] is selected.\nNOTE: You can't customize the controls with this scheme." : "INFO - Control scheme [Custom] is selected.\nNOTE: You can customize the controls with this scheme.");
			break;
		case SETTING_CONTROLS_DPAD_UP:
			cellDbgFontPrintf(0.09f, 0.83f, 0.86f, LIGHTBLUE, "INFO - [D-Pad Up] on the PS3 controller is mapped to action:\n[%s].", Input_PrintMappedButton(Settings.DPad_Up));
			break;
		case SETTING_CONTROLS_DPAD_DOWN:
			cellDbgFontPrintf(0.09f, 0.83f, 0.86f, LIGHTBLUE, "INFO - [D-Pad Down] on the PS3 controller is mapped to action:\n[%s].", Input_PrintMappedButton(Settings.DPad_Down));
			break;
		case SETTING_CONTROLS_DPAD_LEFT:
			cellDbgFontPrintf(0.09f, 0.83f, 0.86f, LIGHTBLUE, "INFO - [D-Pad Left] on the PS3 controller is mapped to action:\n[%s].", Input_PrintMappedButton(Settings.DPad_Left));
			break;
		case SETTING_CONTROLS_DPAD_RIGHT:
			cellDbgFontPrintf(0.09f, 0.83f, 0.86f, LIGHTBLUE, "INFO - [D-Pad Right] on the PS3 controller is mapped to action:\n[%s].", Input_PrintMappedButton(Settings.DPad_Right));
			break;
		case SETTING_CONTROLS_BUTTON_CIRCLE:
			cellDbgFontPrintf(0.09f, 0.83f, 0.86f, LIGHTBLUE, "INFO - [Circle button] on the PS3 controller is mapped to action:\n[%s].", Input_PrintMappedButton(Settings.ButtonCircle));
			break;
		case SETTING_CONTROLS_BUTTON_CROSS:
			cellDbgFontPrintf(0.09f, 0.83f, 0.86f, LIGHTBLUE, "INFO - [Cross button] on the PS3 controller is mapped to action:\n[%s].", Input_PrintMappedButton(Settings.ButtonCross));
			break;
		case SETTING_CONTROLS_BUTTON_TRIANGLE:
			cellDbgFontPrintf(0.09f, 0.83f, 0.86f, LIGHTBLUE, "INFO - [Triangle button] on the PS3 controller is mapped to action:\n[%s].", Input_PrintMappedButton(Settings.ButtonTriangle));
			break;
		case SETTING_CONTROLS_BUTTON_SQUARE:
			cellDbgFontPrintf(0.09f, 0.83f, 0.86f, LIGHTBLUE, "INFO - [Square button] on the PS3 controller is mapped to action:\n[%s].", Input_PrintMappedButton(Settings.ButtonSquare));
			break;
		case SETTING_CONTROLS_BUTTON_SELECT:
			cellDbgFontPrintf(0.09f, 0.83f, 0.86f, LIGHTBLUE, "INFO - [Select button] on the PS3 controller is mapped to action:\n[%s].", Input_PrintMappedButton(Settings.ButtonSelect));
			break;
		case SETTING_CONTROLS_BUTTON_START:
			cellDbgFontPrintf(0.09f, 0.83f, 0.86f, LIGHTBLUE, "INFO - [Start button] on the PS3 controller is mapped to action:\n[%s].", Input_PrintMappedButton(Settings.ButtonStart));
			break;
		case SETTING_CONTROLS_BUTTON_L1:
			cellDbgFontPrintf(0.09f, 0.83f, 0.86f, LIGHTBLUE, "INFO - [L1 button] on the PS3 controller is mapped to action:\n[%s].", Input_PrintMappedButton(Settings.ButtonL1));
			break;
		case SETTING_CONTROLS_BUTTON_R1:
			cellDbgFontPrintf(0.09f, 0.83f, 0.86f, LIGHTBLUE, "INFO - [R1 button] on the PS3 controller is mapped to action:\n[%s].", Input_PrintMappedButton(Settings.ButtonR1));
			break;
		case SETTING_CONTROLS_BUTTON_L2:
			cellDbgFontPrintf(0.09f, 0.83f, 0.86f, LIGHTBLUE, "INFO - [L2 button] on the PS3 controller is mapped to action:\n[%s].", Input_PrintMappedButton(Settings.ButtonL2));
			break;
		case SETTING_CONTROLS_BUTTON_R2:
			cellDbgFontPrintf(0.09f, 0.83f, 0.86f, LIGHTBLUE, "INFO - [R2 button] on the PS3 controller is mapped to action:\n[%s].", Input_PrintMappedButton(Settings.ButtonR2));
			break;
		case SETTING_CONTROLS_BUTTON_L3:
			cellDbgFontPrintf(0.09f, 0.83f, 0.86f, LIGHTBLUE, "INFO - [L3 button] on the PS3 controller is mapped to action:\n[%s].", Input_PrintMappedButton(Settings.ButtonL3));
			break;
		case SETTING_CONTROLS_BUTTON_R3:
			cellDbgFontPrintf(0.09f, 0.83f, 0.86f, LIGHTBLUE, "INFO - [R3 button] on the PS3 controller is mapped to action:\n[%s].", Input_PrintMappedButton(Settings.ButtonR3));
			break;
		case SETTING_CONTROLS_BUTTON_L2_BUTTON_L3:
			cellDbgFontPrintf(0.09f, 0.83f, 0.86f, LIGHTBLUE, "INFO - Button combo [L2 + L3] on the PS3 controller is mapped to action:\n[%s].", Input_PrintMappedButton(Settings.ButtonL2_ButtonL3));
			break;
		case SETTING_CONTROLS_BUTTON_L2_BUTTON_R2:
			cellDbgFontPrintf(0.09f, 0.83f, 0.86f, LIGHTBLUE, "INFO - Button combo [L2 + R2] on the PS3 controller is mapped to action:\n[%s].", Input_PrintMappedButton(Settings.ButtonL2_ButtonR2));
			break;
		case SETTING_CONTROLS_BUTTON_L2_BUTTON_R3:
			cellDbgFontPrintf(0.09f, 0.83f, 0.86f, LIGHTBLUE, "INFO - Button combo [L2 + R3] on the PS3 controller is mapped to action:\n[%s].", Input_PrintMappedButton(Settings.ButtonL2_ButtonR3));
			break;
		case SETTING_CONTROLS_BUTTON_L2_ANALOG_R_RIGHT:
			cellDbgFontPrintf(0.09f, 0.83f, 0.86f, LIGHTBLUE, "INFO - Button combo [L2 + Right Analog Stick - Right] on the PS3 controller\nis mapped to action: [%s].", Input_PrintMappedButton(Settings.ButtonL2_AnalogR_Right));
			break;
		case SETTING_CONTROLS_BUTTON_L2_ANALOG_R_LEFT:
			cellDbgFontPrintf(0.09f, 0.83f, 0.86f, LIGHTBLUE, "INFO - Button combo [L2 + Right Analog Stick - Left] on the PS3 controller\nis mapped to action: [%s].", Input_PrintMappedButton(Settings.ButtonL2_AnalogR_Left));
			break;
		case SETTING_CONTROLS_BUTTON_L2_ANALOG_R_UP:
			cellDbgFontPrintf(0.09f, 0.83f, 0.86f, LIGHTBLUE, "INFO - Button combo [L2 + Right Analog Stick - Up] on the PS3 controller\nis mapped to action: [%s].", Input_PrintMappedButton(Settings.ButtonL2_AnalogR_Up));
			break;
		case SETTING_CONTROLS_BUTTON_L2_ANALOG_R_DOWN:
			cellDbgFontPrintf(0.09f, 0.83f, 0.86f, LIGHTBLUE, "INFO - Button combo [L2 + Right Analog Stick - Down] on the PS3 controller\nis mapped to action: [%s].", Input_PrintMappedButton(Settings.ButtonL2_AnalogR_Down));
			break;
		case SETTING_CONTROLS_BUTTON_R2_ANALOG_R_RIGHT:
			cellDbgFontPrintf(0.09f, 0.83f, 0.86f, LIGHTBLUE, "INFO - Button combo [R2 + Right Analog Stick - Right] on the PS3 controller\nis mapped to action: [%s].", Input_PrintMappedButton(Settings.ButtonR2_AnalogR_Right));
			break;
		case SETTING_CONTROLS_BUTTON_R2_ANALOG_R_LEFT:
			cellDbgFontPrintf(0.09f, 0.83f, 0.86f, LIGHTBLUE, "INFO - Button combo [R2 + Right Analog Stick - Left] on the PS3 controller\nis mapped to action: [%s].", Input_PrintMappedButton(Settings.ButtonR2_AnalogR_Left));
			break;
		case SETTING_CONTROLS_BUTTON_R2_ANALOG_R_UP:
			cellDbgFontPrintf(0.09f, 0.83f, 0.86f, LIGHTBLUE, "INFO - Button combo [R2 + Right Analog Stick - Up] on the PS3 controller\nis mapped to action: [%s].", Input_PrintMappedButton(Settings.ButtonR2_AnalogR_Up));
			break;
		case SETTING_CONTROLS_BUTTON_R2_ANALOG_R_DOWN:
			cellDbgFontPrintf(0.09f, 0.83f, 0.86f, LIGHTBLUE, "INFO - Button combo [R2 + Right Analog Stick - Down] on the PS3 controller\nis mapped to action: [%s].", Input_PrintMappedButton(Settings.ButtonR2_AnalogR_Down));
			break;
		case SETTING_CONTROLS_BUTTON_R2_BUTTON_R3:
			cellDbgFontPrintf(0.09f, 0.83f, 0.86f, LIGHTBLUE, "INFO - Button combo [R2 + R3] on the PS3 controller is mapped to action:\n[%s].", Input_PrintMappedButton(Settings.ButtonR2_ButtonR3));
			break;
		case SETTING_CONTROLS_BUTTON_R3_BUTTON_L3:
			cellDbgFontPrintf(0.09f, 0.83f, 0.86f, LIGHTBLUE, "INFO - Button combo [R3 + L3] on the PS3 controller is mapped to action:\n[%s].", Input_PrintMappedButton(Settings.ButtonR3_ButtonL3));
			break;
		case SETTING_CONTROLS_ANALOG_R_UP:
			cellDbgFontPrintf(0.09f, 0.83f, 0.86f, LIGHTBLUE, "INFO - [Analog Stick Right - Up] on the PS3 controller is mapped to action:\n[%s] (NOTE: Press SELECT to change type).", Input_PrintMappedButton(Settings.AnalogR_Up));
			break;
		case SETTING_CONTROLS_ANALOG_R_DOWN:
			cellDbgFontPrintf(0.09f, 0.83f, 0.86f, LIGHTBLUE, "INFO - [Analog Stick Right - Down] on the PS3 controller is mapped to action:\n[%s] (NOTE: Press SELECT to change type).", Input_PrintMappedButton(Settings.AnalogR_Down));
			break;
		case SETTING_CONTROLS_ANALOG_R_LEFT:
			cellDbgFontPrintf(0.09f, 0.83f, 0.86f, LIGHTBLUE, "INFO - [Analog Stick Right - Left] on the PS3 controller is mapped to action:\n[%s] (NOTE: Press SELECT to change type).", Input_PrintMappedButton(Settings.AnalogR_Left));
			break;
		case SETTING_CONTROLS_ANALOG_R_RIGHT:
			cellDbgFontPrintf(0.09f, 0.83f, 0.86f, LIGHTBLUE, "INFO - [Analog Stick Right - Right] on the PS3 controller is mapped to action:\n[%s] (NOTE: Press SELECT to change type).", Input_PrintMappedButton(Settings.AnalogR_Right));
			break;
		case SETTING_CONTROLS_SAVE_CUSTOM_CONTROLS:
			cellDbgFontPuts(0.09f, 0.83f, 0.86f, LIGHTBLUE, "INFO - Save the custom control settings.\nNOTE: This option will not do anything with Control Scheme [New] or [Default].");
			break;
		case SETTING_CONTROLS_DEFAULT_ALL:
			cellDbgFontPuts(0.09f, 0.83f, 0.86f, LIGHTBLUE, "INFO - Set all 'Controls' settings back to their default values.");
			break;
			
	}
}

void do_pathChoice()
{
	if (tmpBrowser == NULL)
	{
		tmpBrowser = new FileBrowser("/\0");
	}
	string path;

	if (CellInput->UpdateDevice(0) == CELL_PAD_OK)
	{
		UpdateBrowser(tmpBrowser);
		if (CellInput->WasButtonPressed(0,CTRL_SQUARE))
		{
			if(tmpBrowser->IsCurrentADirectory())
			{
				path = tmpBrowser->GetCurrentDirectoryInfo().dir + "/" + tmpBrowser->GetCurrentEntry()->d_name + "/";
				switch(currently_selected_path_setting)
				{
					case SETTING_PATH_SAVESTATES_DIRECTORY:
						Settings.PS3PathSaveStates = path;
						break;
					case SETTING_PATH_SRAM_DIRECTORY:
						Settings.PS3PathSRAM = path;
						break;
					case SETTING_PATH_CHEATS:
						Settings.PS3PathCheats = path;
						break;
					case SETTING_PATH_DEFAULT_ROM_DIRECTORY:
						Settings.PS3PathROMDirectory = path;
						break;
				}
				menuStack.pop();
			}
		}
		if (CellInput->WasButtonPressed(0, CTRL_TRIANGLE))
		{
			path = USRDIR;
			switch(currently_selected_path_setting)
			{
				case SETTING_PATH_SAVESTATES_DIRECTORY:
					Settings.PS3PathSaveStates = path;
					break;
				case SETTING_PATH_SRAM_DIRECTORY:
					Settings.PS3PathSRAM = path;
					break;
				case SETTING_PATH_CHEATS:
					Settings.PS3PathCheats = path;
					break;
				case SETTING_PATH_DEFAULT_ROM_DIRECTORY:
					Settings.PS3PathROMDirectory = path;
					break;
			}
			menuStack.pop();
		}
		if (CellInput->WasButtonPressed(0,CTRL_CROSS))
		{
			if(tmpBrowser->IsCurrentADirectory())
			{
				tmpBrowser->PushDirectory(tmpBrowser->GetCurrentDirectoryInfo().dir + "/" + tmpBrowser->GetCurrentEntry()->d_name, CELL_FS_TYPE_REGULAR | CELL_FS_TYPE_DIRECTORY, "empty");
			}
		}
	}
			
	cellDbgFontPrintf(0.09f, 0.09f, Emulator_GetFontSize(), YELLOW, "PATH: %s", tmpBrowser->GetCurrentDirectoryInfo().dir.c_str());
	cellDbgFontPuts	(0.09f,	0.05f,	Emulator_GetFontSize(),	RED,	"DIRECTORY SELECTION");
	cellDbgFontPuts(0.09f, 0.93f, Emulator_GetFontSize(), YELLOW,"X - Enter directory             /\\ - return to settings");
	//cellDbgFontPuts(0.09f, 0.93f, Emulator_GetFontSize(), YELLOW,"SQUARE - Select directory as path");
	cellDbgFontPrintf(0.09f, 0.89f, 0.86f, LIGHTBLUE, "%s",
	"INFO - Browse to a directory and assign it as the path by pressing SQUARE button.");
	Graphics->FlushDbgFont();
			
	RenderBrowser(tmpBrowser);
}

void do_controls_settings_entry(int *settingsptr, int defaultvalue)
{
	if(CellInput->WasButtonHeld(0, CTRL_LEFT) | CellInput->IsAnalogPressedLeft(0, CTRL_LSTICK))
	{
		if(Settings.ControlScheme == CONTROL_SCHEME_CUSTOM)
		{
			Input_MapButton(settingsptr,false,NULL);
			if(CellInput->IsButtonPressed(0,CTRL_LEFT) | CellInput->IsAnalogPressedLeft(0,CTRL_LSTICK))
			{
				sys_timer_usleep(SETTINGS_DELAY);
			}
		}
	}
	if(CellInput->WasButtonHeld(0, CTRL_RIGHT) | CellInput->IsAnalogPressedRight(0, CTRL_LSTICK) | CellInput->WasButtonPressed(0, CTRL_CROSS))
	{
		if(Settings.ControlScheme == CONTROL_SCHEME_CUSTOM)
		{
			Input_MapButton(settingsptr,true,NULL);
			if(CellInput->IsButtonPressed(0,CTRL_RIGHT) | CellInput->IsAnalogPressedRight(0,CTRL_LSTICK))
			{
				sys_timer_usleep(SETTINGS_DELAY);
			}
		}
	}
	if(CellInput->IsButtonPressed(0, CTRL_START))
	{
		if(Settings.ControlScheme == CONTROL_SCHEME_CUSTOM)
		{
			Input_MapButton(settingsptr,true,defaultvalue);
		}
	}
}

void do_controls_settings()
{
	if(CellInput->UpdateDevice(0) == CELL_OK)
	{
			// back to ROM menu if CIRCLE is pressed
			if (CellInput->WasButtonPressed(0, CTRL_L1) | CellInput->WasButtonPressed(0, CTRL_CIRCLE))
			{
				menuStack.pop();
				return;
			}

			if (CellInput->WasButtonHeld(0, CTRL_DOWN) | CellInput->IsAnalogPressedDown(0, CTRL_LSTICK))	// down to next setting
			{
				currently_selected_controller_setting++;
				if (currently_selected_controller_setting >= MAX_NO_OF_CONTROLS_SETTINGS)
				{
					currently_selected_controller_setting = FIRST_CONTROLS_SETTING;
				}
				if(CellInput->IsButtonPressed(0,CTRL_DOWN) | CellInput->IsAnalogPressedDown(0, CTRL_LSTICK))
				{
					sys_timer_usleep(SETTINGS_DELAY);
				}
			}

			if (CellInput->IsButtonPressed(0,CTRL_L2) && CellInput->IsButtonPressed(0,CTRL_R2))
			{
				// if a rom is loaded then resume it
				if (Emulator_IsROMLoaded())
				{
					MenuStop();
					Emulator_StartROMRunning();
				}
				return;
			}

			if (CellInput->WasButtonHeld(0, CTRL_UP) | CellInput->IsAnalogPressedUp(0, CTRL_LSTICK))	// up to previous setting
			{
					currently_selected_controller_setting--;
					if (currently_selected_controller_setting < FIRST_CONTROLS_SETTING)
					{
						currently_selected_controller_setting = MAX_NO_OF_CONTROLS_SETTINGS-1;
					}
					if(CellInput->IsButtonPressed(0,CTRL_UP) | CellInput->IsAnalogPressedUp(0, CTRL_LSTICK))
					{
						sys_timer_usleep(SETTINGS_DELAY);
					}
			}
					switch(currently_selected_controller_setting)
					{
						case SETTING_CONTROLS_SCHEME:
						   if(CellInput->WasButtonPressed(0, CTRL_RIGHT) | CellInput->IsAnalogPressedRight(0,CTRL_LSTICK))
						   {
							switch(Settings.ControlScheme)
							{
								case CONTROL_SCHEME_DEFAULT:
									Settings.ControlScheme = CONTROL_SCHEME_NEW;
									break;
								case CONTROL_SCHEME_NEW:
									Settings.ControlScheme = CONTROL_SCHEME_CUSTOM;
									break;
								case CONTROL_SCHEME_CUSTOM:
									break;
							}
							Emulator_Implementation_SwitchControlScheme();
						   }
						   if(CellInput->WasButtonPressed(0, CTRL_LEFT) | CellInput->IsAnalogPressedLeft(0,CTRL_LSTICK))
						   {
							switch(Settings.ControlScheme)
							{
								case CONTROL_SCHEME_CUSTOM:
									Settings.ControlScheme = CONTROL_SCHEME_NEW;
									break;
								case CONTROL_SCHEME_NEW:
									Settings.ControlScheme = CONTROL_SCHEME_DEFAULT;
									break;
								case CONTROL_SCHEME_DEFAULT:
									break;
							}
							Emulator_Implementation_SwitchControlScheme();
						   }
						   if(CellInput->IsButtonPressed(0, CTRL_START))
						   {
							Settings.ControlScheme = CONTROL_SCHEME_DEFAULT;
						   }
							break;
						case SETTING_CONTROLS_DPAD_UP:
							do_controls_settings_entry(&Settings.DPad_Up,BTN_UP);
							break;
						case SETTING_CONTROLS_DPAD_DOWN:
							do_controls_settings_entry(&Settings.DPad_Down,BTN_DOWN);
							break;
						case SETTING_CONTROLS_DPAD_LEFT:
							do_controls_settings_entry(&Settings.DPad_Left,BTN_LEFT);
							break;
						case SETTING_CONTROLS_DPAD_RIGHT:
							do_controls_settings_entry(&Settings.DPad_Right,BTN_RIGHT);
							break;
						case SETTING_CONTROLS_BUTTON_CIRCLE:
							do_controls_settings_entry(&Settings.ButtonCircle,BTN_A);
							break;
						case SETTING_CONTROLS_BUTTON_CROSS:
							do_controls_settings_entry(&Settings.ButtonCross,BTN_B);
							break;
						case SETTING_CONTROLS_BUTTON_TRIANGLE:
							do_controls_settings_entry(&Settings.ButtonTriangle,BTN_X);
							break;
						case SETTING_CONTROLS_BUTTON_SQUARE:
							do_controls_settings_entry(&Settings.ButtonSquare,BTN_Y);
							break;
						case SETTING_CONTROLS_BUTTON_SELECT:
							do_controls_settings_entry(&Settings.ButtonSelect,BTN_SELECT);
							break;
						case SETTING_CONTROLS_BUTTON_START:
							do_controls_settings_entry(&Settings.ButtonStart,BTN_START);
							break;
						case SETTING_CONTROLS_BUTTON_L1:
							do_controls_settings_entry(&Settings.ButtonL1,BTN_L);
							break;
						case SETTING_CONTROLS_BUTTON_L2:
							do_controls_settings_entry(&Settings.ButtonL2,BTN_NONE);
							break;
						case SETTING_CONTROLS_BUTTON_R2:
							do_controls_settings_entry(&Settings.ButtonR2,BTN_NONE);
							break;
						case SETTING_CONTROLS_BUTTON_L3:
							do_controls_settings_entry(&Settings.ButtonL3,BTN_NONE);
							break;
						case SETTING_CONTROLS_BUTTON_R3:
							do_controls_settings_entry(&Settings.ButtonR3,BTN_NONE);
							break;
						case SETTING_CONTROLS_BUTTON_R1:
							do_controls_settings_entry(&Settings.ButtonR1,BTN_R);
							break;
						case SETTING_CONTROLS_BUTTON_L2_BUTTON_L3:
							do_controls_settings_entry(&Settings.ButtonL2_ButtonL3,BTN_NONE);
							break;
						case SETTING_CONTROLS_BUTTON_L2_BUTTON_R2:
							do_controls_settings_entry(&Settings.ButtonL2_ButtonR2,BTN_NONE);
							break;
						case SETTING_CONTROLS_BUTTON_L2_BUTTON_R3:
							do_controls_settings_entry(&Settings.ButtonL2_ButtonR3,BTN_QUICKLOAD);
							break;
						case SETTING_CONTROLS_BUTTON_R2_BUTTON_R3:
							do_controls_settings_entry(&Settings.ButtonR2_ButtonR3,BTN_QUICKSAVE);
							break;
						case SETTING_CONTROLS_ANALOG_R_UP:
							do_controls_settings_entry(&Settings.AnalogR_Up,BTN_CHEATENABLE);
							if(CellInput->WasButtonPressed(0, CTRL_SELECT))
							{
								Settings.AnalogR_Up_Type = !Settings.AnalogR_Up_Type;
							}
							break;
						case SETTING_CONTROLS_ANALOG_R_DOWN:
							do_controls_settings_entry(&Settings.AnalogR_Down,BTN_CHEATDISABLE);
							if(CellInput->WasButtonPressed(0, CTRL_SELECT))
							{
								Settings.AnalogR_Down_Type = !Settings.AnalogR_Down_Type;
							}
							break;
						case SETTING_CONTROLS_ANALOG_R_LEFT:
							do_controls_settings_entry(&Settings.AnalogR_Left,BTN_DECREMENTSAVE);
							if(CellInput->WasButtonPressed(0, CTRL_SELECT))
							{
								Settings.AnalogR_Left_Type = !Settings.AnalogR_Left_Type;
							}
							break;
						case SETTING_CONTROLS_ANALOG_R_RIGHT:
							do_controls_settings_entry(&Settings.AnalogR_Right,BTN_INCREMENTSAVE);
							if(CellInput->WasButtonPressed(0, CTRL_SELECT))
							{
								Settings.AnalogR_Right_Type = !Settings.AnalogR_Right_Type;
							}
							break;
						case SETTING_CONTROLS_BUTTON_L2_ANALOG_R_RIGHT:
							do_controls_settings_entry(&Settings.ButtonL2_AnalogR_Right,BTN_INCREMENTCHEAT);
							break;
						case SETTING_CONTROLS_BUTTON_L2_ANALOG_R_LEFT:
							do_controls_settings_entry(&Settings.ButtonL2_AnalogR_Left,BTN_DECREMENTCHEAT);
							break;
						case SETTING_CONTROLS_BUTTON_L2_ANALOG_R_UP:
							do_controls_settings_entry(&Settings.ButtonL2_AnalogR_Up,BTN_NONE);
							break;
						case SETTING_CONTROLS_BUTTON_L2_ANALOG_R_DOWN:
							do_controls_settings_entry(&Settings.ButtonL2_AnalogR_Down,BTN_NONE);
							break;
						case SETTING_CONTROLS_BUTTON_R2_ANALOG_R_RIGHT:
							do_controls_settings_entry(&Settings.ButtonR2_AnalogR_Right,BTN_NONE);
							break;
						case SETTING_CONTROLS_BUTTON_R2_ANALOG_R_LEFT:
							do_controls_settings_entry(&Settings.ButtonR2_AnalogR_Left,BTN_NONE);
							break;
						case SETTING_CONTROLS_BUTTON_R2_ANALOG_R_UP:
							do_controls_settings_entry(&Settings.ButtonR2_AnalogR_Up,BTN_NONE);
							break;
						case SETTING_CONTROLS_BUTTON_R2_ANALOG_R_DOWN:
							do_controls_settings_entry(&Settings.ButtonR2_AnalogR_Down,BTN_NONE);
							break;
						case SETTING_CONTROLS_BUTTON_R3_BUTTON_L3:
							do_controls_settings_entry(&Settings.ButtonR3_ButtonL3,BTN_EXITTOMENU);
							break;
						case SETTING_CONTROLS_SAVE_CUSTOM_CONTROLS:
						if(CellInput->WasButtonPressed(0, CTRL_LEFT) | CellInput->WasAnalogPressedLeft(0, CTRL_LSTICK) | CellInput->WasButtonPressed(0, CTRL_RIGHT) | CellInput->WasAnalogPressedRight(0, CTRL_LSTICK) | CellInput->WasButtonPressed(0, CTRL_CROSS) | CellInput->IsButtonPressed(0, CTRL_START))
						{
							Emulator_Implementation_SaveCustomControls(true);
						}
							break;
						case SETTING_CONTROLS_DEFAULT_ALL:
						if(CellInput->WasButtonPressed(0, CTRL_LEFT) | CellInput->WasAnalogPressedLeft(0, CTRL_LSTICK) | CellInput->WasButtonPressed(0, CTRL_RIGHT) | CellInput->WasAnalogPressedRight(0, CTRL_LSTICK) | CellInput->WasButtonPressed(0, CTRL_CROSS) | CellInput->IsButtonPressed(0, CTRL_START))
						{
							Settings.ControlScheme = CONTROL_SCHEME_DEFAULT;
							Emulator_Implementation_ButtonMappingSettings(MAP_BUTTONS_OPTION_DEFAULT);
						}
							break;
				default:
					break;
			} // end of switch
	}

	float yPos = 0.09;
	float ySpacing = 0.04;

	cellDbgFontPuts		(0.09f,	0.05f,	Emulator_GetFontSize(),	GREEN,	"GENERAL");
	cellDbgFontPuts		(0.27f,	0.05f,	Emulator_GetFontSize(),	GREEN,	"SNES9x");
	cellDbgFontPuts		(0.45f,	0.05f,	Emulator_GetFontSize(),	GREEN,	"PATH");
	cellDbgFontPuts		(0.63f, 0.05f,	Emulator_GetFontSize(), RED,	"CONTROLS"); 
	Graphics->FlushDbgFont();

//PAGE 1
//FIXME: Terrible hardcoding
if((currently_selected_controller_setting-FIRST_CONTROLS_SETTING) < NUM_ENTRY_PER_SETTINGS_PAGE)
{
	cellDbgFontPuts		(0.09f,	yPos,	Emulator_GetFontSize(),	currently_selected_controller_setting == SETTING_CONTROLS_SCHEME ? YELLOW : WHITE,	"Control Scheme");
	switch(Settings.ControlScheme)
	{
		case CONTROL_SCHEME_DEFAULT:
			cellDbgFontPrintf(0.5f, yPos, Emulator_GetFontSize(), Settings.ControlScheme == CONTROL_SCHEME_DEFAULT ? GREEN : ORANGE, "Default");
			break;
		case CONTROL_SCHEME_NEW:
			cellDbgFontPrintf(0.5f, yPos, Emulator_GetFontSize(), Settings.ControlScheme == CONTROL_SCHEME_DEFAULT ? GREEN : ORANGE, "New");
			break;
		case CONTROL_SCHEME_CUSTOM:
			cellDbgFontPrintf(0.5f, yPos, Emulator_GetFontSize(), Settings.ControlScheme == CONTROL_SCHEME_DEFAULT ? GREEN : ORANGE, "Custom");
			break;
	}

	yPos += ySpacing;
	cellDbgFontPuts		(0.09f,	yPos,	Emulator_GetFontSize(),	currently_selected_controller_setting == SETTING_CONTROLS_DPAD_UP ? YELLOW : WHITE,	"D-Pad Up");
	cellDbgFontPuts		(0.5f,	yPos,	Emulator_GetFontSize(),	Settings.DPad_Up == BTN_UP ? GREEN : ORANGE, Input_PrintMappedButton(Settings.DPad_Up));

	yPos += ySpacing;
	cellDbgFontPuts		(0.09f,	yPos,	Emulator_GetFontSize(),	currently_selected_controller_setting == SETTING_CONTROLS_DPAD_DOWN ? YELLOW : WHITE,	"D-Pad Down");
	cellDbgFontPuts		(0.5f,	yPos,	Emulator_GetFontSize(),	Settings.DPad_Down == BTN_DOWN ? GREEN : ORANGE, Input_PrintMappedButton(Settings.DPad_Down));
	Graphics->FlushDbgFont();

	yPos += ySpacing;
	cellDbgFontPuts		(0.09f,	yPos,	Emulator_GetFontSize(),	currently_selected_controller_setting == SETTING_CONTROLS_DPAD_LEFT ? YELLOW : WHITE,	"D-Pad Left");
	cellDbgFontPuts		(0.5f,	yPos,	Emulator_GetFontSize(),	Settings.DPad_Left == BTN_LEFT ? GREEN : ORANGE, Input_PrintMappedButton(Settings.DPad_Left));
	Graphics->FlushDbgFont();

	yPos += ySpacing;
	cellDbgFontPuts		(0.09f,	yPos,	Emulator_GetFontSize(),	currently_selected_controller_setting == SETTING_CONTROLS_DPAD_RIGHT ? YELLOW : WHITE,	"D-Pad Right");
	cellDbgFontPuts		(0.5f,	yPos,	Emulator_GetFontSize(),	Settings.DPad_Right == BTN_RIGHT ? GREEN : ORANGE, Input_PrintMappedButton(Settings.DPad_Right));
	Graphics->FlushDbgFont();

	yPos += ySpacing;
	cellDbgFontPuts		(0.09f,	yPos,	Emulator_GetFontSize(),	currently_selected_controller_setting == SETTING_CONTROLS_BUTTON_CIRCLE ? YELLOW : WHITE,	"Circle button");
	cellDbgFontPuts		(0.5f,	yPos,	Emulator_GetFontSize(),	Settings.ButtonCircle == BTN_A ? GREEN : ORANGE, Input_PrintMappedButton(Settings.ButtonCircle));
	Graphics->FlushDbgFont();

	yPos += ySpacing;
	cellDbgFontPuts		(0.09f,	yPos,	Emulator_GetFontSize(),	currently_selected_controller_setting == SETTING_CONTROLS_BUTTON_CROSS ? YELLOW : WHITE,	"Cross button");
	cellDbgFontPuts		(0.5f,	yPos,	Emulator_GetFontSize(),	Settings.ButtonCross == BTN_B ? GREEN : ORANGE, Input_PrintMappedButton(Settings.ButtonCross));
	Graphics->FlushDbgFont();

	yPos += ySpacing;
	cellDbgFontPuts		(0.09f,	yPos,	Emulator_GetFontSize(),	currently_selected_controller_setting == SETTING_CONTROLS_BUTTON_TRIANGLE ? YELLOW : WHITE,	"Triangle button");
	cellDbgFontPuts		(0.5f,	yPos,	Emulator_GetFontSize(),	Settings.ButtonTriangle == BTN_X ? GREEN : ORANGE, Input_PrintMappedButton(Settings.ButtonTriangle));
	Graphics->FlushDbgFont();

	yPos += ySpacing;
	cellDbgFontPuts		(0.09f,	yPos,	Emulator_GetFontSize(),	currently_selected_controller_setting == SETTING_CONTROLS_BUTTON_SQUARE ? YELLOW : WHITE,	"Square button");
	cellDbgFontPuts		(0.5f,	yPos,	Emulator_GetFontSize(),	Settings.ButtonSquare == BTN_Y ? GREEN : ORANGE, Input_PrintMappedButton(Settings.ButtonSquare));
	Graphics->FlushDbgFont();

	yPos += ySpacing;
	cellDbgFontPuts		(0.09f,	yPos,	Emulator_GetFontSize(),	currently_selected_controller_setting == SETTING_CONTROLS_BUTTON_SELECT ? YELLOW : WHITE,	"Select button");
	cellDbgFontPuts		(0.5f,	yPos,	Emulator_GetFontSize(),	Settings.ButtonSelect == BTN_SELECT ? GREEN : ORANGE, Input_PrintMappedButton(Settings.ButtonSelect));
	Graphics->FlushDbgFont();

	yPos += ySpacing;
	cellDbgFontPuts		(0.09f,	yPos,	Emulator_GetFontSize(),	currently_selected_controller_setting == SETTING_CONTROLS_BUTTON_START ? YELLOW : WHITE,	"Start button");
	cellDbgFontPuts		(0.5f,	yPos,	Emulator_GetFontSize(),	Settings.ButtonStart == BTN_START ? GREEN : ORANGE, Input_PrintMappedButton(Settings.ButtonStart));
	Graphics->FlushDbgFont();

	yPos += ySpacing;
	cellDbgFontPuts		(0.09f,	yPos,	Emulator_GetFontSize(),	currently_selected_controller_setting == SETTING_CONTROLS_BUTTON_L1 ? YELLOW : WHITE,	"L1 button");
	cellDbgFontPuts		(0.5f,	yPos,	Emulator_GetFontSize(),	Settings.ButtonL1 == BTN_L ? GREEN : ORANGE, Input_PrintMappedButton(Settings.ButtonL1));
	Graphics->FlushDbgFont();

	yPos += ySpacing;
	cellDbgFontPuts		(0.09f,	yPos,	Emulator_GetFontSize(),	currently_selected_controller_setting == SETTING_CONTROLS_BUTTON_R1 ? YELLOW : WHITE,	"R1 button");
	cellDbgFontPuts		(0.5f,	yPos,	Emulator_GetFontSize(),	Settings.ButtonR1 == BTN_R ? GREEN : ORANGE, Input_PrintMappedButton(Settings.ButtonR1));
	Graphics->FlushDbgFont();

	yPos += ySpacing;
	cellDbgFontPuts		(0.09f,	yPos,	Emulator_GetFontSize(),	currently_selected_controller_setting == SETTING_CONTROLS_BUTTON_L2 ? YELLOW : WHITE,	"L2 button");
	cellDbgFontPuts		(0.5f,	yPos,	Emulator_GetFontSize(),	Settings.ButtonL2 == BTN_NONE ? GREEN : ORANGE, Input_PrintMappedButton(Settings.ButtonL2));
	Graphics->FlushDbgFont();

	yPos += ySpacing;
	cellDbgFontPuts		(0.09f,	yPos,	Emulator_GetFontSize(),	currently_selected_controller_setting == SETTING_CONTROLS_BUTTON_R2 ? YELLOW : WHITE,	"R2 button");
	cellDbgFontPuts		(0.5f,	yPos,	Emulator_GetFontSize(),	Settings.ButtonR2 == BTN_NONE ? GREEN : ORANGE, Input_PrintMappedButton(Settings.ButtonR2));
	Graphics->FlushDbgFont();

	yPos += ySpacing;
	cellDbgFontPuts		(0.09f,	yPos,	Emulator_GetFontSize(),	currently_selected_controller_setting == SETTING_CONTROLS_BUTTON_L3 ? YELLOW : WHITE,	"L3 button");
	cellDbgFontPuts		(0.5f,	yPos,	Emulator_GetFontSize(),	Settings.ButtonL3 == BTN_NONE ? GREEN : ORANGE, Input_PrintMappedButton(Settings.ButtonL3));
	Graphics->FlushDbgFont();

	yPos += ySpacing;
	cellDbgFontPuts		(0.09f,	yPos,	Emulator_GetFontSize(),	currently_selected_controller_setting == SETTING_CONTROLS_BUTTON_R3 ? YELLOW : WHITE,	"R3 button");
	cellDbgFontPuts		(0.5f,	yPos,	Emulator_GetFontSize(),	Settings.ButtonR3 == BTN_NONE ? GREEN : ORANGE, Input_PrintMappedButton(Settings.ButtonR3));
	Graphics->FlushDbgFont();

	yPos += ySpacing;
	cellDbgFontPuts		(0.09f,	yPos,	Emulator_GetFontSize(),	currently_selected_controller_setting == SETTING_CONTROLS_BUTTON_L2_BUTTON_L3 ? YELLOW : WHITE,	"Button combo: L2 & L3");
	cellDbgFontPuts		(0.5f,	yPos,	Emulator_GetFontSize(),	Settings.ButtonL2_ButtonL3 == BTN_NONE ? GREEN : ORANGE, Input_PrintMappedButton(Settings.ButtonL2_ButtonL3));
	Graphics->FlushDbgFont();

}

//PAGE 2
//FIXME: Terrible hardcoding 
if((currently_selected_controller_setting-FIRST_CONTROLS_SETTING) >= NUM_ENTRY_PER_SETTINGS_PAGE)
{
	yPos = 0.09;

	cellDbgFontPuts		(0.09f,	yPos,	Emulator_GetFontSize(),	currently_selected_controller_setting == SETTING_CONTROLS_BUTTON_L2_BUTTON_R2 ? YELLOW : WHITE,	"Button combo: L2 & R2");
	cellDbgFontPuts		(0.5f,	yPos,	Emulator_GetFontSize(),	Settings.ButtonL2_ButtonR2 == BTN_NONE ? GREEN : ORANGE, Input_PrintMappedButton(Settings.ButtonL2_ButtonR2));
	Graphics->FlushDbgFont();

	yPos += ySpacing;
	cellDbgFontPuts		(0.09f,	yPos,	Emulator_GetFontSize(),	currently_selected_controller_setting == SETTING_CONTROLS_BUTTON_L2_BUTTON_R3 ? YELLOW : WHITE,	"Button combo: L2 & R3");
	cellDbgFontPuts		(0.5f,	yPos,	Emulator_GetFontSize(),	Settings.ButtonL2_ButtonR3 == BTN_QUICKLOAD ? GREEN : ORANGE, Input_PrintMappedButton(Settings.ButtonL2_ButtonR3));
	Graphics->FlushDbgFont();

	yPos += ySpacing;
	cellDbgFontPuts		(0.09f,	yPos,	Emulator_GetFontSize(),	currently_selected_controller_setting == SETTING_CONTROLS_BUTTON_L2_ANALOG_R_RIGHT ? YELLOW : WHITE,	"Button combo: L2 & RStick Right");
	cellDbgFontPuts		(0.5f,	yPos,	Emulator_GetFontSize(),	Settings.ButtonL2_AnalogR_Right == BTN_INCREMENTCHEAT ? GREEN : ORANGE, Input_PrintMappedButton(Settings.ButtonL2_AnalogR_Right));
	Graphics->FlushDbgFont();

	yPos += ySpacing;
	cellDbgFontPuts		(0.09f,	yPos,	Emulator_GetFontSize(),	currently_selected_controller_setting == SETTING_CONTROLS_BUTTON_L2_ANALOG_R_LEFT ? YELLOW : WHITE,	"Button combo: L2 & RStick Left");
	cellDbgFontPuts		(0.5f,	yPos,	Emulator_GetFontSize(),	Settings.ButtonL2_AnalogR_Left == BTN_DECREMENTCHEAT ? GREEN : ORANGE, Input_PrintMappedButton(Settings.ButtonL2_AnalogR_Left));
	Graphics->FlushDbgFont();

	yPos += ySpacing;
	cellDbgFontPuts		(0.09f,	yPos,	Emulator_GetFontSize(),	currently_selected_controller_setting == SETTING_CONTROLS_BUTTON_L2_ANALOG_R_UP ? YELLOW : WHITE,	"Button combo: L2 & RStick Up");
	cellDbgFontPuts		(0.5f,	yPos,	Emulator_GetFontSize(),	Settings.ButtonL2_AnalogR_Up == BTN_NONE ? GREEN : ORANGE, Input_PrintMappedButton(Settings.ButtonL2_AnalogR_Up));
	Graphics->FlushDbgFont();

	yPos += ySpacing;
	cellDbgFontPuts		(0.09f,	yPos,	Emulator_GetFontSize(),	currently_selected_controller_setting == SETTING_CONTROLS_BUTTON_L2_ANALOG_R_DOWN ? YELLOW : WHITE,	"Button combo: L2 & RStick Down");
	cellDbgFontPuts		(0.5f,	yPos,	Emulator_GetFontSize(),	Settings.ButtonL2_AnalogR_Down == BTN_NONE ? GREEN : ORANGE, Input_PrintMappedButton(Settings.ButtonL2_AnalogR_Down));
	Graphics->FlushDbgFont();

	yPos += ySpacing;
	cellDbgFontPuts		(0.09f,	yPos,	Emulator_GetFontSize(),	currently_selected_controller_setting == SETTING_CONTROLS_BUTTON_R2_ANALOG_R_RIGHT ? YELLOW : WHITE,	"Button combo: R2 & RStick Right");
	cellDbgFontPuts		(0.5f,	yPos,	Emulator_GetFontSize(),	Settings.ButtonR2_AnalogR_Right == BTN_NONE ? GREEN : ORANGE, Input_PrintMappedButton(Settings.ButtonR2_AnalogR_Right));
	Graphics->FlushDbgFont();

	yPos += ySpacing;
	cellDbgFontPuts		(0.09f,	yPos,	Emulator_GetFontSize(),	currently_selected_controller_setting == SETTING_CONTROLS_BUTTON_R2_ANALOG_R_LEFT ? YELLOW : WHITE,	"Button combo: R2 & RStick Left");
	cellDbgFontPuts		(0.5f,	yPos,	Emulator_GetFontSize(),	Settings.ButtonR2_AnalogR_Left == BTN_NONE ? GREEN : ORANGE, Input_PrintMappedButton(Settings.ButtonR2_AnalogR_Left));
	Graphics->FlushDbgFont();

	yPos += ySpacing;
	cellDbgFontPuts		(0.09f,	yPos,	Emulator_GetFontSize(),	currently_selected_controller_setting == SETTING_CONTROLS_BUTTON_R2_ANALOG_R_UP ? YELLOW : WHITE,	"Button combo: R2 & RStick Up");
	cellDbgFontPuts		(0.5f,	yPos,	Emulator_GetFontSize(),	Settings.ButtonR2_AnalogR_Up == BTN_NONE ? GREEN : ORANGE, Input_PrintMappedButton(Settings.ButtonR2_AnalogR_Up));
	Graphics->FlushDbgFont();

	yPos += ySpacing;
	cellDbgFontPuts		(0.09f,	yPos,	Emulator_GetFontSize(),	currently_selected_controller_setting == SETTING_CONTROLS_BUTTON_R2_ANALOG_R_DOWN ? YELLOW : WHITE,	"Button combo: R2 & RStick Down");
	cellDbgFontPuts		(0.5f,	yPos,	Emulator_GetFontSize(),	Settings.ButtonR2_AnalogR_Down == BTN_NONE ? GREEN : ORANGE, Input_PrintMappedButton(Settings.ButtonR2_AnalogR_Down));
	Graphics->FlushDbgFont();

	yPos += ySpacing;
	cellDbgFontPuts		(0.09f,	yPos,	Emulator_GetFontSize(),	currently_selected_controller_setting == SETTING_CONTROLS_BUTTON_R2_BUTTON_R3 ? YELLOW : WHITE,	"Button combo: R2 & R3");
	cellDbgFontPuts		(0.5f,	yPos,	Emulator_GetFontSize(),	Settings.ButtonR2_ButtonR3 == BTN_QUICKSAVE ? GREEN : ORANGE, Input_PrintMappedButton(Settings.ButtonR2_ButtonR3));
	Graphics->FlushDbgFont();

	yPos += ySpacing;
	cellDbgFontPuts		(0.09f,	yPos,	Emulator_GetFontSize(),	currently_selected_controller_setting == SETTING_CONTROLS_BUTTON_R3_BUTTON_L3 ? YELLOW : WHITE,	"Button combo: R3 & L3");
	cellDbgFontPuts		(0.5f,	yPos,	Emulator_GetFontSize(),	Settings.ButtonR3_ButtonL3 == BTN_EXITTOMENU ? GREEN : ORANGE, Input_PrintMappedButton(Settings.ButtonR3_ButtonL3));
	Graphics->FlushDbgFont();

	yPos += ySpacing;
	cellDbgFontPrintf		(0.09f,	yPos,	Emulator_GetFontSize(),	currently_selected_controller_setting == SETTING_CONTROLS_ANALOG_R_UP ? YELLOW : WHITE,	"Right Stick - Up %s", Settings.AnalogR_Up_Type ? "(IsPressed)" : "(WasPressed)");
	cellDbgFontPuts		(0.5f,	yPos,	Emulator_GetFontSize(),	Settings.AnalogR_Up == BTN_CHEATENABLE ? GREEN : ORANGE, Input_PrintMappedButton(Settings.AnalogR_Up));
	Graphics->FlushDbgFont();

	yPos += ySpacing;
	cellDbgFontPrintf		(0.09f,	yPos,	Emulator_GetFontSize(),	currently_selected_controller_setting == SETTING_CONTROLS_ANALOG_R_DOWN ? YELLOW : WHITE,	"Right Stick - Down %s", Settings.AnalogR_Down_Type ? "(IsPressed)" : "(WasPressed)");
	cellDbgFontPuts		(0.5f,	yPos,	Emulator_GetFontSize(),	Settings.AnalogR_Down == BTN_CHEATDISABLE ? GREEN : ORANGE, Input_PrintMappedButton(Settings.AnalogR_Down));
	Graphics->FlushDbgFont();

	yPos += ySpacing;
	cellDbgFontPrintf		(0.09f,	yPos,	Emulator_GetFontSize(),	currently_selected_controller_setting == SETTING_CONTROLS_ANALOG_R_LEFT ? YELLOW : WHITE,	"Right Stick - Left %s", Settings.AnalogR_Left_Type ? "(IsPressed)" : "(WasPressed)");
	cellDbgFontPuts		(0.5f,	yPos,	Emulator_GetFontSize(),	Settings.AnalogR_Left == BTN_DECREMENTSAVE ? GREEN : ORANGE, Input_PrintMappedButton(Settings.AnalogR_Left));
	Graphics->FlushDbgFont();

	yPos += ySpacing;
	cellDbgFontPrintf		(0.09f,	yPos,	Emulator_GetFontSize(),	currently_selected_controller_setting == SETTING_CONTROLS_ANALOG_R_RIGHT ? YELLOW : WHITE,	"Right Stick - Right %s", Settings.AnalogR_Right_Type ? "(IsPressed)" : "(WasPressed)");
	cellDbgFontPuts		(0.5f,	yPos,	Emulator_GetFontSize(),	Settings.AnalogR_Right == BTN_INCREMENTSAVE ? GREEN : ORANGE, Input_PrintMappedButton(Settings.AnalogR_Right));
	Graphics->FlushDbgFont();

	yPos += ySpacing;
	cellDbgFontPrintf(0.09f, yPos, Emulator_GetFontSize(), currently_selected_controller_setting == SETTING_CONTROLS_SAVE_CUSTOM_CONTROLS ? YELLOW : GREEN, "SAVE CUSTOM CONTROLS");

	yPos += ySpacing;
	cellDbgFontPrintf(0.09f, yPos, Emulator_GetFontSize(), currently_selected_controller_setting == SETTING_CONTROLS_DEFAULT_ALL ? YELLOW : GREEN, "DEFAULT");
}

	DisplayHelpMessage(currently_selected_controller_setting);

	cellDbgFontPuts(0.09f, 0.89f, Emulator_GetFontSize(), YELLOW,
	"UP/DOWN - select  L2+R2 - resume game   X/LEFT/RIGHT - change");
	cellDbgFontPuts(0.09f, 0.93f, Emulator_GetFontSize(), YELLOW,
	"START - default   L1/CIRCLE - go back");
	Graphics->FlushDbgFont();
}


// void do_path_settings()
// // called from ROM menu by pressing the SELECT button
// // return to ROM menu by pressing the CIRCLE button
void do_path_settings()
{
	if(CellInput->UpdateDevice(0) == CELL_OK)
	{
			// back to ROM menu if CIRCLE is pressed
			if (CellInput->WasButtonPressed(0, CTRL_L1) | CellInput->WasButtonPressed(0, CTRL_CIRCLE))
			{
				menuStack.pop();
				return;
			}

			if (CellInput->WasButtonPressed(0, CTRL_R1))
			{
				menuStack.push(do_controls_settings);
				return;
			}

			if (CellInput->WasButtonHeld(0, CTRL_DOWN) | CellInput->IsAnalogPressedDown(0, CTRL_LSTICK))	// down to next setting
			{
				currently_selected_path_setting++;
				if (currently_selected_path_setting >= MAX_NO_OF_PATH_SETTINGS)
				{
					currently_selected_path_setting = FIRST_PATH_SETTING;
				}

				if(CellInput->IsButtonPressed(0,CTRL_DOWN) | CellInput->IsAnalogPressedDown(0, CTRL_LSTICK))
				{
					sys_timer_usleep(SETTINGS_DELAY);
				}
			}

			if (CellInput->IsButtonPressed(0,CTRL_L2) && CellInput->IsButtonPressed(0,CTRL_R2))
			{
				// if a rom is loaded then resume it
				if (Emulator_IsROMLoaded())
				{
					MenuStop();
					Emulator_StartROMRunning();
				}
				return;
			}

			if (CellInput->WasButtonHeld(0, CTRL_UP) | CellInput->IsAnalogPressedUp(0, CTRL_LSTICK))	// up to previous setting
			{
					currently_selected_path_setting--;
					if (currently_selected_path_setting < FIRST_PATH_SETTING)
					{
						currently_selected_path_setting = MAX_NO_OF_PATH_SETTINGS-1;
					}
					if(CellInput->IsButtonPressed(0,CTRL_UP) | CellInput->IsAnalogPressedUp(0, CTRL_LSTICK))
					{
						sys_timer_usleep(SETTINGS_DELAY);
					}
			}

					switch(currently_selected_path_setting)
					{
						case SETTING_PATH_DEFAULT_ROM_DIRECTORY:
						if(CellInput->WasButtonPressed(0, CTRL_LEFT) | CellInput->WasAnalogPressedLeft(0, CTRL_LSTICK) | CellInput->WasButtonPressed(0, CTRL_RIGHT) | CellInput->WasAnalogPressedRight(0, CTRL_LSTICK) | CellInput->WasButtonPressed(0, CTRL_CROSS))
						{
							menuStack.push(do_pathChoice);
							tmpBrowser = NULL;
						}
						if(CellInput->IsButtonPressed(0, CTRL_START))
						{
							Settings.PS3PathROMDirectory = "/";
						}
							break;
						case SETTING_PATH_SAVESTATES_DIRECTORY:
						if(CellInput->WasButtonPressed(0, CTRL_LEFT) | CellInput->WasAnalogPressedLeft(0, CTRL_LSTICK) | CellInput->WasButtonPressed(0, CTRL_RIGHT) | CellInput->WasAnalogPressedRight(0, CTRL_LSTICK) | CellInput->WasButtonPressed(0, CTRL_CROSS))
						{
							menuStack.push(do_pathChoice);
							tmpBrowser = NULL;
						}
						if(CellInput->IsButtonPressed(0, CTRL_START))
						{
							Settings.PS3PathSaveStates = USRDIR;
						}
							break;
						case SETTING_PATH_SRAM_DIRECTORY:
						if(CellInput->WasButtonPressed(0, CTRL_LEFT) | CellInput->WasAnalogPressedLeft(0, CTRL_LSTICK) | CellInput->WasButtonPressed(0, CTRL_RIGHT) | CellInput->WasAnalogPressedRight(0, CTRL_LSTICK) | CellInput->WasButtonPressed(0, CTRL_CROSS))
						{
							menuStack.push(do_pathChoice);
							tmpBrowser = NULL;
						}
						if(CellInput->IsButtonPressed(0, CTRL_START))
						{
							Settings.PS3PathSRAM = "";
						}
							break;
						case SETTING_PATH_CHEATS:
						if(CellInput->WasButtonPressed(0, CTRL_LEFT) | CellInput->WasAnalogPressedLeft(0, CTRL_LSTICK) | CellInput->WasButtonPressed(0, CTRL_RIGHT) | CellInput->WasAnalogPressedRight(0, CTRL_LSTICK) | CellInput->WasButtonPressed(0, CTRL_CROSS))
						{
							menuStack.push(do_pathChoice);
							tmpBrowser = NULL;
						}
						if(CellInput->IsButtonPressed(0, CTRL_START))
						{
							Settings.PS3PathCheats = USRDIR;
						}
							break;
						case SETTING_PATH_DEFAULT_ALL:
						if(CellInput->WasButtonPressed(0, CTRL_LEFT) | CellInput->WasAnalogPressedLeft(0, CTRL_LSTICK) | CellInput->WasButtonPressed(0, CTRL_RIGHT) | CellInput->WasAnalogPressedRight(0, CTRL_LSTICK) | CellInput->WasButtonPressed(0, CTRL_CROSS) | CellInput->IsButtonPressed(0, CTRL_START))
						{
							Settings.PS3PathROMDirectory = "/";
							Settings.PS3PathSaveStates = USRDIR;
							Settings.PS3PathCheats = USRDIR;
							Settings.PS3PathSRAM = "";
						}
							break;
				default:
					break;
			} // end of switch
	}

	float yPos = 0.09;
	float ySpacing = 0.04;

	cellDbgFontPuts		(0.09f,	0.05f,	Emulator_GetFontSize(),	GREEN,	"GENERAL");
	cellDbgFontPuts		(0.27f,	0.05f,	Emulator_GetFontSize(),	GREEN,	"SNES9x");
	cellDbgFontPuts		(0.45f,	0.05f,	Emulator_GetFontSize(),	RED,	"PATH");
	cellDbgFontPuts		(0.63f, 0.05f,	Emulator_GetFontSize(), GREEN,	"CONTROLS"); 
	Graphics->FlushDbgFont();

	cellDbgFontPuts		(0.09f,	yPos,	Emulator_GetFontSize(),	currently_selected_path_setting == SETTING_PATH_DEFAULT_ROM_DIRECTORY ? YELLOW : WHITE,	"Startup ROM Directory");
	cellDbgFontPuts		(0.5f,	yPos,	Emulator_GetFontSize(),	!(strcmp(Settings.PS3PathROMDirectory.c_str(),"/")) ? GREEN : ORANGE, Settings.PS3PathROMDirectory.c_str());

	yPos += ySpacing;
	cellDbgFontPuts		(0.09f,	yPos,	Emulator_GetFontSize(),	currently_selected_path_setting == SETTING_PATH_SAVESTATES_DIRECTORY ? YELLOW : WHITE,	"Savestate Directory");
	cellDbgFontPuts		(0.5f,	yPos,	Emulator_GetFontSize(),	!(strcmp(Settings.PS3PathSaveStates.c_str(),USRDIR)) ? GREEN : ORANGE, Settings.PS3PathSaveStates.c_str());
	Graphics->FlushDbgFont();

	yPos += ySpacing;
	cellDbgFontPuts		(0.09f,	yPos,	Emulator_GetFontSize(),	currently_selected_path_setting == SETTING_PATH_SRAM_DIRECTORY ? YELLOW : WHITE,	"SRAM Directory");
	cellDbgFontPuts		(0.5f,	yPos,	Emulator_GetFontSize(),	!(strcmp(Settings.PS3PathSRAM.c_str(),"")) ? GREEN : ORANGE, !(strcmp(Settings.PS3PathSRAM.c_str(),"")) ? "Same dir as ROM" : Settings.PS3PathSRAM.c_str());
	Graphics->FlushDbgFont();

	yPos += ySpacing;
	cellDbgFontPuts		(0.09f,	yPos,	Emulator_GetFontSize(),	currently_selected_path_setting == SETTING_PATH_CHEATS ? YELLOW : WHITE,	"Cheatfile directory");
	cellDbgFontPuts		(0.5f,	yPos,	Emulator_GetFontSize(),	!(strcmp(Settings.PS3PathCheats.c_str(),USRDIR)) ? GREEN : ORANGE, Settings.PS3PathCheats.c_str());

	yPos += ySpacing;
	cellDbgFontPrintf(0.09f, yPos, Emulator_GetFontSize(), currently_selected_path_setting == SETTING_PATH_DEFAULT_ALL ? YELLOW : GREEN, "DEFAULT");

	DisplayHelpMessage(currently_selected_path_setting);

	cellDbgFontPuts(0.09f, 0.89f, Emulator_GetFontSize(), YELLOW,
	"UP/DOWN - select  L2+R2 - resume game   X/LEFT/RIGHT - change");
	cellDbgFontPuts(0.09f, 0.93f, Emulator_GetFontSize(), YELLOW,
	"START - default   L1/CIRCLE - go back   R1 - go forward");
	Graphics->FlushDbgFont();
}

void do_snes9x_settings()
{
	if(CellInput->UpdateDevice(0) == CELL_PAD_OK)
	{
			// back to ROM menu if CIRCLE is pressed
			if (CellInput->WasButtonPressed(0, CTRL_L1) | CellInput->WasButtonPressed(0, CTRL_CIRCLE))
			{
				menuStack.pop();
				return;
			}

			if (CellInput->WasButtonPressed(0, CTRL_R1))
			{
				menuStack.push(do_path_settings);
				return;
			}

			if (CellInput->WasButtonHeld(0, CTRL_DOWN) | CellInput->IsAnalogPressedDown(0, CTRL_LSTICK))	// down to next setting
			{
				currently_selected_snes9x_setting++;
				if (currently_selected_snes9x_setting >= MAX_NO_OF_SNES9X_SETTINGS)
				{
					currently_selected_snes9x_setting = FIRST_SNES9X_SETTING;
				}
				if(CellInput->IsButtonPressed(0,CTRL_DOWN) | CellInput->IsAnalogPressedDown(0, CTRL_LSTICK))
				{
					sys_timer_usleep(SETTINGS_DELAY);
				}
			}
			if (CellInput->WasButtonHeld(0, CTRL_UP) | CellInput->IsAnalogPressedUp(0, CTRL_LSTICK))	// up to previous setting
			{
					currently_selected_snes9x_setting--;
					if (currently_selected_snes9x_setting < FIRST_SNES9X_SETTING)
					{
						currently_selected_snes9x_setting = MAX_NO_OF_SNES9X_SETTINGS-1;
					}
					if(CellInput->IsButtonPressed(0,CTRL_UP) | CellInput->IsAnalogPressedUp(0, CTRL_LSTICK))
					{
						sys_timer_usleep(SETTINGS_DELAY);
					}
			}
			
			if (CellInput->IsButtonPressed(0,CTRL_L2) && CellInput->IsButtonPressed(0,CTRL_R2))
			{
				// if a rom is loaded then resume it
				if (Emulator_IsROMLoaded())
				{
					MenuStop();
					Emulator_StartROMRunning();
				}
				return;
			}
			
			switch(currently_selected_snes9x_setting)
			{
					// display framerate on/off
					case SETTING_SNES9X_FPS:
						if(CellInput->WasButtonPressed(0, CTRL_LEFT) | CellInput->WasAnalogPressedLeft(0, CTRL_LSTICK) | CellInput->WasButtonPressed(0, CTRL_RIGHT) | CellInput->WasAnalogPressedRight(0, CTRL_LSTICK) | CellInput->WasButtonPressed(0, CTRL_CROSS))
						{
							Settings.DisplayFrameRate = !Settings.DisplayFrameRate;
						}
						if(CellInput->IsButtonPressed(0, CTRL_START))
						{
							Settings.DisplayFrameRate = false;
						}
						break;
					case SETTING_SNES9X_SOUND_INPUT_RATE:
						if(CellInput->WasButtonHeld(0, CTRL_LEFT) | CellInput->IsAnalogPressedLeft(0,CTRL_LSTICK))
						{
							if(Settings.SoundInputRate > 30000)
							{
								Settings.SoundInputRate--;
							}
							if(CellInput->IsButtonPressed(0,CTRL_LEFT) | CellInput->IsAnalogPressedLeft(0,CTRL_LSTICK))
							{
								sys_timer_usleep(SETTINGS_DELAY);
							}
						}
						if(CellInput->WasButtonHeld(0, CTRL_RIGHT) | CellInput->IsAnalogPressedRight(0,CTRL_LSTICK) | CellInput->WasButtonPressed(0,CTRL_CROSS))
						{
							if(Settings.SoundInputRate < 32000) // SNES APU has an input rate of 32000, no higher
							{
								Settings.SoundInputRate++;
							}
							if(CellInput->IsButtonPressed(0,CTRL_RIGHT) | CellInput->IsAnalogPressedRight(0,CTRL_LSTICK))
							{
								sys_timer_usleep(SETTINGS_DELAY);
							}
						}
						if(CellInput->IsButtonPressed(0, CTRL_START))
						{
							Settings.SoundInputRate = 31942;
						}
						break;
					case SETTING_SNES9X_TRANSPARENCY:
						if(CellInput->WasButtonPressed(0, CTRL_LEFT) | CellInput->WasAnalogPressedLeft(0,CTRL_LSTICK) | CellInput->WasButtonPressed(0, CTRL_RIGHT) | CellInput->WasAnalogPressedRight(0,CTRL_LSTICK) | CellInput->WasButtonPressed(0, CTRL_CROSS))
						{
							Settings.Transparency = !Settings.Transparency;
						}
						if(CellInput->IsButtonPressed(0, CTRL_START))
						{
							Settings.Transparency = true;
						}
						break;
					case SETTING_SNES9X_SKIP_FRAMES:
						   if(CellInput->WasButtonPressed(0, CTRL_LEFT) | CellInput->WasAnalogPressedLeft(0,CTRL_LSTICK))
						   {
							   if (Settings.SkipFrames <= 1)
							   {
								   Settings.SkipFrames = AUTO_FRAMERATE;
							   }
							   else if(Settings.SkipFrames != AUTO_FRAMERATE)
							   {
								   Settings.SkipFrames--;
							   }
						   }
						   if(CellInput->WasButtonPressed(0, CTRL_RIGHT) | CellInput->WasAnalogPressedRight(0,CTRL_LSTICK) | CellInput->WasButtonPressed(0, CTRL_CROSS))
						   {
							   if (Settings.SkipFrames == AUTO_FRAMERATE)
							   {
								   Settings.SkipFrames = 1;
							   }
							   else if (Settings.SkipFrames < 10)
							   {
								   Settings.SkipFrames++;
							   }
						   }
						   if(CellInput->IsButtonPressed(0, CTRL_START))
						   {
							   Settings.SkipFrames = AUTO_FRAMERATE;
						   }
						   break;
					case SETTING_SNES9X_DISABLE_GRAPHIC_WINDOWS:
						   if(CellInput->WasButtonPressed(0, CTRL_RIGHT) | CellInput->WasAnalogPressedRight(0,CTRL_LSTICK) | CellInput->WasAnalogPressedLeft(0,CTRL_LSTICK) | CellInput->WasButtonPressed(0, CTRL_LEFT) | CellInput->WasButtonPressed(0, CTRL_CROSS))
						   {
							   Settings.DisableGraphicWindows = !Settings.DisableGraphicWindows;
						   }
						   if(CellInput->IsButtonPressed(0, CTRL_START))
						   {
							   Settings.DisableGraphicWindows = false;
						   }
						   break;
					case SETTING_SNES9X_DISPLAY_PRESSED_KEYS:
						   if(CellInput->WasButtonPressed(0, CTRL_RIGHT) | CellInput->WasAnalogPressedRight(0, CTRL_LSTICK) | CellInput->WasAnalogPressedLeft(0, CTRL_LSTICK) | CellInput->WasButtonPressed(0, CTRL_LEFT) | CellInput->WasButtonPressed(0, CTRL_CROSS))
						   {
							   Settings.DisplayPressedKeys = !Settings.DisplayPressedKeys;
						   }
						   if(CellInput->IsButtonPressed(0, CTRL_START))
						   {
							   Settings.DisplayPressedKeys = false;
						   }
						   break;
					case SETTING_SNES9X_FORCE_PAL:
						   if(CellInput->WasButtonPressed(0, CTRL_RIGHT) | CellInput->WasAnalogPressedRight(0, CTRL_LSTICK) | CellInput->WasAnalogPressedLeft(0, CTRL_LSTICK) | CellInput->WasButtonPressed(0, CTRL_LEFT) | CellInput->WasButtonPressed(0, CTRL_CROSS))
						   {
							   Settings.ForcePAL = !Settings.ForcePAL;
							   if(Settings.ForceNTSC)
							   {
								   Settings.ForceNTSC = false;
							   }
						   }
						   if(CellInput->IsButtonPressed(0, CTRL_START))
						   {
							   Settings.ForcePAL = false;
						   }
						   break;
					case SETTING_SNES9X_FORCE_NTSC:
						   if(CellInput->WasButtonPressed(0, CTRL_RIGHT) | CellInput->WasAnalogPressedRight(0, CTRL_LSTICK) | CellInput->WasAnalogPressedLeft(0, CTRL_LSTICK) | CellInput->WasButtonPressed(0, CTRL_LEFT) | CellInput->WasButtonPressed(0, CTRL_CROSS))
						   {
							   Settings.ForceNTSC = !Settings.ForceNTSC;
							   if(Settings.ForcePAL)
							   {
								   Settings.ForcePAL = false;
							   }
						   }
						   if(CellInput->IsButtonPressed(0, CTRL_START))
						   {
							   Settings.ForceNTSC = false;
						   }
						   break;
					case SETTING_SNES9X_PAL_TIMING:
						   if(CellInput->WasButtonPressed(0, CTRL_LEFT) | CellInput->WasAnalogPressedLeft(0,CTRL_LSTICK))
						   {
							   Settings.FrameTimePAL = 16667;
						   }
						   if(CellInput->WasButtonPressed(0, CTRL_RIGHT) | CellInput->WasAnalogPressedRight(0,CTRL_LSTICK))
						   {
							   Settings.FrameTimePAL = 20000;
						   }
						   if(CellInput->IsButtonPressed(0, CTRL_START))
						   {
							   Settings.FrameTimePAL = 20000;
						   }
						   break;
					case SETTING_SNES9X_AUTO_APPLY_CHEATS:
						   if(CellInput->WasButtonPressed(0, CTRL_RIGHT) | CellInput->WasAnalogPressedRight(0, CTRL_LSTICK) | CellInput->WasAnalogPressedLeft(0, CTRL_LSTICK) | CellInput->WasButtonPressed(0, CTRL_LEFT) | CellInput->WasButtonPressed(0, CTRL_CROSS))
						   {
							   Settings.ApplyCheats = !Settings.ApplyCheats;
						   }
						   if(CellInput->IsButtonPressed(0, CTRL_START))
						   {
							   Settings.ApplyCheats = true;
						   }
						   break;
					case SETTING_SNES9X_AUTO_APPLY_PATCH:
						   if(CellInput->WasButtonPressed(0, CTRL_RIGHT) | CellInput->WasAnalogPressedRight(0, CTRL_LSTICK) | CellInput->WasAnalogPressedLeft(0, CTRL_LSTICK) | CellInput->WasButtonPressed(0, CTRL_LEFT) | CellInput->WasButtonPressed(0, CTRL_CROSS))
						   {
							   Settings.NoPatch = !Settings.NoPatch;
						   }
						   if(CellInput->IsButtonPressed(0, CTRL_START))
						   {
							   Settings.NoPatch = true;
						   }
						   break;
					case SETTING_SNES9X_RESET_BEFORE_RECORDING_MOVIE:
						   if(CellInput->WasButtonPressed(0, CTRL_RIGHT) | CellInput->WasAnalogPressedRight(0, CTRL_LSTICK) | CellInput->WasAnalogPressedLeft(0, CTRL_LSTICK) | CellInput->WasButtonPressed(0, CTRL_LEFT) | CellInput->WasButtonPressed(0, CTRL_CROSS))
						   {
							   Settings.ResetBeforeRecordingMovie = !Settings.ResetBeforeRecordingMovie;
						   }
						   if(CellInput->IsButtonPressed(0, CTRL_START))
						   {
							   Settings.ResetBeforeRecordingMovie = false;
						   }
						   break;
					case SETTING_SNES9X_SRAM_WRITEPROTECT:
						   if(CellInput->WasButtonPressed(0, CTRL_RIGHT) | CellInput->WasAnalogPressedRight(0, CTRL_LSTICK) | CellInput->WasAnalogPressedLeft(0, CTRL_LSTICK) | CellInput->WasButtonPressed(0, CTRL_LEFT) | CellInput->WasButtonPressed(0, CTRL_CROSS))
						   {
							   Settings.SRAMWriteProtect = !Settings.SRAMWriteProtect;
						   }
						   if(CellInput->IsButtonPressed(0, CTRL_START))
						   {
							   Settings.SRAMWriteProtect = false;
						   }
						   break;
					case SETTING_SNES9X_TURBO_SKIP_FRAMES:
						if(CellInput->WasButtonHeld(0, CTRL_LEFT) | CellInput->IsAnalogPressedLeft(0,CTRL_LSTICK))
						{
							if(Settings.TurboSkipFrames > 0)
							{
								Settings.TurboSkipFrames--;
							}
							if(CellInput->IsButtonPressed(0,CTRL_LEFT) | CellInput->IsAnalogPressedLeft(0,CTRL_LSTICK))
							{
								sys_timer_usleep(SETTINGS_DELAY);
							}
						}
						if(CellInput->WasButtonHeld(0, CTRL_RIGHT) | CellInput->IsAnalogPressedRight(0,CTRL_LSTICK) | CellInput->WasButtonPressed(0,CTRL_CROSS))
						{
							if(Settings.TurboSkipFrames < 100)
							{
								Settings.TurboSkipFrames++;
							}
							if(CellInput->IsButtonPressed(0,CTRL_RIGHT) | CellInput->IsAnalogPressedRight(0,CTRL_LSTICK))
							{
								sys_timer_usleep(SETTINGS_DELAY);
							}
						}
						if(CellInput->IsButtonPressed(0, CTRL_START))
						{
							Settings.TurboSkipFrames = 15;
						}
						break;
					case SETTING_SNES9X_ACCESSORY_AUTODETECTION:
						if(CellInput->WasButtonHeld(0, CTRL_LEFT) | CellInput->IsAnalogPressedLeft(0,CTRL_LSTICK))
						{
							if(Settings.AccessoryAutoDetection != ACCESSORY_AUTODETECTION_CONFIRM)
							{
								Settings.AccessoryAutoDetection--;
							}
							if(CellInput->IsButtonPressed(0,CTRL_LEFT) | CellInput->IsAnalogPressedLeft(0,CTRL_LSTICK))
							{
								sys_timer_usleep(SETTINGS_DELAY);
							}
						}
						if(CellInput->WasButtonHeld(0, CTRL_RIGHT) | CellInput->IsAnalogPressedRight(0,CTRL_LSTICK) | CellInput->WasButtonPressed(0,CTRL_CROSS))
						{
							if(Settings.AccessoryAutoDetection != ACCESSORY_AUTODETECTION_NONE)
							{
								Settings.AccessoryAutoDetection++;
							}
							if(CellInput->IsButtonPressed(0,CTRL_RIGHT) | CellInput->IsAnalogPressedRight(0,CTRL_LSTICK))
							{
								sys_timer_usleep(SETTINGS_DELAY);
							}
						}
						if(CellInput->IsButtonPressed(0, CTRL_START))
						{
							Settings.AccessoryAutoDetection = ACCESSORY_AUTODETECTION_CONFIRM;
						}
					break;
					case SETTING_SNES9X_DEFAULT_ALL:
						if(CellInput->WasButtonPressed(0, CTRL_LEFT) | CellInput->WasAnalogPressedLeft(0,CTRL_LSTICK) | CellInput->WasButtonPressed(0, CTRL_RIGHT) | CellInput->WasAnalogPressedRight(0,CTRL_LSTICK) | CellInput->IsButtonPressed(0, CTRL_START) | CellInput->WasButtonPressed(0, CTRL_CROSS))
					{
						Settings.DisplayFrameRate = false;
						Settings.SoundInputRate = 31942;
						Settings.Transparency = true;
						Settings.SkipFrames = AUTO_FRAMERATE;
						Settings.DisableGraphicWindows = false;
						Settings.DisplayPressedKeys = false;
						Settings.ForceNTSC = false;
						Settings.ForcePAL = false;
						Settings.FrameTimePAL = 20000;
						Settings.ShutdownMaster = false;
						Settings.NoPatch = false;
						Settings.ApplyCheats = true;
						Settings.ResetBeforeRecordingMovie = false;
						Settings.TurboSkipFrames = 15;
						Settings.AccessoryAutoDetection = ACCESSORY_AUTODETECTION_CONFIRM;
						Settings.SRAMWriteProtect = false;
					}
					break;
				default:
					break;
			} // end of switch
	}

	cellDbgFontPuts		(0.09f,	0.05f,	Emulator_GetFontSize(),	GREEN,	"GENERAL");
	cellDbgFontPuts		(0.27f,	0.05f,	Emulator_GetFontSize(),	RED,	"SNES9x");
	cellDbgFontPuts		(0.45f,	0.05f,	Emulator_GetFontSize(),	GREEN,	"PATH");
	cellDbgFontPuts		(0.63f, 0.05f,	Emulator_GetFontSize(), GREEN,	"CONTROLS"); 
	Graphics->FlushDbgFont();

	float yPos = 0.09;
	float ySpacing = 0.04;

	cellDbgFontPuts		(0.09f,	yPos,	Emulator_GetFontSize(),	currently_selected_snes9x_setting == SETTING_SNES9X_FPS ? YELLOW : WHITE,	"Display Framerate");
	cellDbgFontPuts		(0.5f,	yPos,	Emulator_GetFontSize(),	Settings.DisplayFrameRate == false ? GREEN : ORANGE, Settings.DisplayFrameRate == true ? "ON" : "OFF");

	yPos += ySpacing;
	cellDbgFontPuts		(0.09f,	yPos,	Emulator_GetFontSize(),	currently_selected_snes9x_setting == SETTING_SNES9X_SOUND_INPUT_RATE ? YELLOW : WHITE,	"Sound inputrate");
	cellDbgFontPrintf	(0.5f,	yPos,	Emulator_GetFontSize(),	Settings.SoundInputRate == 31942 ? GREEN : ORANGE, "%d", Settings.SoundInputRate);
	Graphics->FlushDbgFont();

	yPos += ySpacing;
	cellDbgFontPuts		(0.09f,	yPos,	Emulator_GetFontSize(),	currently_selected_snes9x_setting == SETTING_SNES9X_TRANSPARENCY ? YELLOW : WHITE, "Transparency effects");
	cellDbgFontPuts		(0.5f,	yPos,	Emulator_GetFontSize(),	Settings.Transparency == true ? GREEN : ORANGE, Settings.Transparency == true ? "ON" : "OFF");

	yPos += ySpacing;
	cellDbgFontPuts		(0.09f,	yPos,	Emulator_GetFontSize(),	currently_selected_snes9x_setting == SETTING_SNES9X_SKIP_FRAMES ? YELLOW : WHITE,	"Frame skip");

	if(Settings.SkipFrames == AUTO_FRAMERATE)
	{
		cellDbgFontPuts(0.5f, yPos, Emulator_GetFontSize(), GREEN, "AUTO");
	}
	else {
		cellDbgFontPrintf(0.5f, yPos, Emulator_GetFontSize(), ORANGE, "%d", Settings.SkipFrames);
	}

	yPos += ySpacing;
	cellDbgFontPuts		(0.09f,	yPos,	Emulator_GetFontSize(),	currently_selected_snes9x_setting == SETTING_SNES9X_DISABLE_GRAPHIC_WINDOWS ? YELLOW : WHITE,	"Disable graphic windows");
	cellDbgFontPuts(0.5f,  yPos, Emulator_GetFontSize(), Settings.DisableGraphicWindows == false ? GREEN : ORANGE, Settings.DisableGraphicWindows == true ? "ON" : "OFF");
	Graphics->FlushDbgFont();

	yPos += ySpacing;
	cellDbgFontPuts(0.09f, yPos, Emulator_GetFontSize(), currently_selected_snes9x_setting == SETTING_SNES9X_DISPLAY_PRESSED_KEYS ? YELLOW : WHITE, "Display button input");
	cellDbgFontPuts(0.5f,  yPos, Emulator_GetFontSize(), Settings.DisplayPressedKeys == false ? GREEN : ORANGE, Settings.DisplayPressedKeys == true ? "ON" : "OFF");

	yPos += ySpacing;
	cellDbgFontPuts(0.09f, yPos, Emulator_GetFontSize(), currently_selected_snes9x_setting == SETTING_SNES9X_FORCE_PAL ? YELLOW : WHITE, "Force NTSC ROM to PAL");
	cellDbgFontPuts(0.5f,  yPos, Emulator_GetFontSize(), Settings.ForcePAL == false ? GREEN : ORANGE, Settings.ForcePAL == true ? "ON" : "OFF");
	Graphics->FlushDbgFont();

	yPos += ySpacing;
	cellDbgFontPuts(0.09f, yPos, Emulator_GetFontSize(), currently_selected_snes9x_setting == SETTING_SNES9X_FORCE_NTSC ? YELLOW : WHITE, "Force PAL ROM to NTSC");
	cellDbgFontPuts(0.5f,  yPos, Emulator_GetFontSize(), Settings.ForceNTSC == false ? GREEN : ORANGE, Settings.ForceNTSC == true ? "ON" : "OFF");

	yPos += ySpacing;
	cellDbgFontPuts(0.09f, yPos, Emulator_GetFontSize(), currently_selected_snes9x_setting == SETTING_SNES9X_PAL_TIMING ? YELLOW : WHITE, "PAL Timing");
	cellDbgFontPrintf(0.5f, yPos, Emulator_GetFontSize(), Settings.FrameTimePAL == 20000 ? GREEN : ORANGE, "%d", Settings.FrameTimePAL);

	yPos += ySpacing;
	cellDbgFontPuts(0.09f, yPos, Emulator_GetFontSize(), currently_selected_snes9x_setting == SETTING_SNES9X_AUTO_APPLY_CHEATS ? YELLOW : WHITE, "Auto-apply cheats");
	cellDbgFontPuts(0.5f,  yPos, Emulator_GetFontSize(), Settings.ApplyCheats ? GREEN : ORANGE, Settings.ApplyCheats == true ? "ON" : "OFF");
	Graphics->FlushDbgFont();

	yPos += ySpacing;
	cellDbgFontPuts(0.09f, yPos, Emulator_GetFontSize(), currently_selected_snes9x_setting == SETTING_SNES9X_AUTO_APPLY_PATCH ? YELLOW : WHITE, "Auto-apply IPS/UPS patch");
	cellDbgFontPuts(0.5f,  yPos, Emulator_GetFontSize(), Settings.NoPatch ? ORANGE : GREEN, Settings.NoPatch ? "OFF" : "ON");

	yPos += ySpacing;
	cellDbgFontPuts(0.09f, yPos, Emulator_GetFontSize(), currently_selected_snes9x_setting == SETTING_SNES9X_RESET_BEFORE_RECORDING_MOVIE ? YELLOW : WHITE, "Reset Before Recording Movie");
	cellDbgFontPuts(0.5f,  yPos, Emulator_GetFontSize(), Settings.ResetBeforeRecordingMovie ? ORANGE : GREEN, Settings.ResetBeforeRecordingMovie ? "ON" : "OFF");

	yPos += ySpacing;
	cellDbgFontPuts		(0.09f,	yPos,	Emulator_GetFontSize(),	currently_selected_snes9x_setting == SETTING_SNES9X_SRAM_WRITEPROTECT ? YELLOW : WHITE,	"Savestate SRAM Overwrite Protect");
	cellDbgFontPrintf	(0.5f,	yPos,	Emulator_GetFontSize(),	Settings.SRAMWriteProtect == false ? GREEN : ORANGE, Settings.SRAMWriteProtect ? "ON" : "OFF");
	Graphics->FlushDbgFont();

	yPos += ySpacing;
	cellDbgFontPuts		(0.09f,	yPos,	Emulator_GetFontSize(),	currently_selected_snes9x_setting == SETTING_SNES9X_TURBO_SKIP_FRAMES ? YELLOW : WHITE,	"Fast forwarding - frameskip");
	cellDbgFontPrintf	(0.5f,	yPos,	Emulator_GetFontSize(),	Settings.TurboSkipFrames == 15 ? GREEN : ORANGE, "%d", Settings.TurboSkipFrames);
	Graphics->FlushDbgFont();

	yPos += ySpacing;
	cellDbgFontPuts		(0.09f,	yPos,	Emulator_GetFontSize(),	currently_selected_snes9x_setting == SETTING_SNES9X_ACCESSORY_AUTODETECTION ? YELLOW : WHITE,	"Accessory Autodetection");
	cellDbgFontPrintf	(0.5f,	yPos,	Emulator_GetFontSize(),	Settings.AccessoryAutoDetection == ACCESSORY_AUTODETECTION_CONFIRM ? GREEN : ORANGE, "%s", Settings.AccessoryAutoDetection == ACCESSORY_AUTODETECTION_ENABLED ? "ON" : Settings.AccessoryAutoDetection == ACCESSORY_AUTODETECTION_CONFIRM ? "Confirm" : "OFF");
	Graphics->FlushDbgFont();

	yPos += ySpacing;
	cellDbgFontPrintf(0.09f, yPos, Emulator_GetFontSize(), currently_selected_snes9x_setting == SETTING_SNES9X_DEFAULT_ALL ? YELLOW : GREEN, "DEFAULT");
	Graphics->FlushDbgFont();

	DisplayHelpMessage(currently_selected_snes9x_setting);

	cellDbgFontPuts(0.09f, 0.89f, Emulator_GetFontSize(), YELLOW,
	"UP/DOWN - select  L2+R2 - resume game   X/LEFT/RIGHT - change");
	cellDbgFontPuts(0.09f, 0.93f, Emulator_GetFontSize(), YELLOW,
	"START - default   L1/CIRCLE - go back   R1 - go forward");
	Graphics->FlushDbgFont();
}

void do_general_settings()
{
	if(CellInput->UpdateDevice(0) == CELL_PAD_OK)
	{
			// back to ROM menu if CIRCLE is pressed
			if (CellInput->WasButtonPressed(0, CTRL_CIRCLE) || CellInput->WasButtonPressed(0, CTRL_L1))
			{
				menuStack.pop();
				return;
			}

			if (CellInput->WasButtonPressed(0, CTRL_R1))
			{
				menuStack.push(do_snes9x_settings);
				return;
			}

			if (CellInput->WasButtonHeld(0, CTRL_DOWN) | CellInput->IsAnalogPressedDown(0, CTRL_LSTICK))	// down to next setting
			{
				currently_selected_setting++;
				if (currently_selected_setting >= MAX_NO_OF_SETTINGS)
				{
					currently_selected_setting = FIRST_GENERAL_SETTING;
				}
				if(CellInput->IsButtonPressed(0,CTRL_DOWN) | CellInput->IsAnalogPressedDown(0, CTRL_LSTICK))
				{
					sys_timer_usleep(SETTINGS_DELAY);
				}
			}

			if (CellInput->WasButtonHeld(0, CTRL_UP) | CellInput->IsAnalogPressedUp(0, CTRL_LSTICK))	// up to previous setting
			{
					currently_selected_setting--;
					if (currently_selected_setting < FIRST_GENERAL_SETTING)
					{
						currently_selected_setting = MAX_NO_OF_SETTINGS-1;
					}
					if(CellInput->IsButtonPressed(0,CTRL_UP) | CellInput->IsAnalogPressedUp(0, CTRL_LSTICK))
					{
						sys_timer_usleep(SETTINGS_DELAY);
					}
			}

			if (CellInput->IsButtonPressed(0,CTRL_L2) && CellInput->IsButtonPressed(0,CTRL_R2))
			{
				// if a rom is loaded then resume it
				if (Emulator_IsROMLoaded())
				{
					MenuStop();
					Emulator_StartROMRunning();
				}
				return;
			}

					switch(currently_selected_setting)
					{
					case SETTING_CURRENT_SAVE_STATE_SLOT:
						   if(CellInput->WasButtonHeld(0, CTRL_LEFT) | CellInput->IsAnalogPressedLeft(0,CTRL_LSTICK) | CellInput->WasButtonPressed(0, CTRL_CROSS))
						   {
							Emulator_Implementation_DecrementCurrentSaveStateSlot();
							if(CellInput->IsButtonPressed(0,CTRL_LEFT) | CellInput->IsAnalogPressedLeft(0,CTRL_LSTICK))
							{
								sys_timer_usleep(SETTINGS_DELAY);
							}
						   }
						   if(CellInput->WasButtonHeld(0, CTRL_RIGHT) | CellInput->IsAnalogPressedRight(0,CTRL_LSTICK) | CellInput->WasButtonPressed(0, CTRL_CROSS))
						   {
							Emulator_Implementation_IncrementCurrentSaveStateSlot();
							if(CellInput->IsButtonPressed(0,CTRL_RIGHT) | CellInput->IsAnalogPressedRight(0,CTRL_LSTICK))
							{
								sys_timer_usleep(SETTINGS_DELAY);
							}
						   }
						   if(CellInput->IsButtonPressed(0, CTRL_START))
						   {
							   Emulator_Implementation_SetCurrentSaveStateSlot(0);
						   }
						   break;
					case SETTING_CHANGE_RESOLUTION:
						   if(CellInput->WasButtonHeld(0, CTRL_RIGHT) | CellInput->IsAnalogPressedRight(0,CTRL_LSTICK))
						   {
							   Graphics->NextResolution();
							if(CellInput->IsButtonPressed(0,CTRL_RIGHT) | CellInput->IsAnalogPressedRight(0,CTRL_LSTICK))
							{
								sys_timer_usleep(SETTINGS_DELAY);
							}
						   }
						   if(CellInput->WasButtonHeld(0, CTRL_LEFT) | CellInput->IsAnalogPressedLeft(0,CTRL_LSTICK))
						   {
							   Graphics->PreviousResolution();
							if(CellInput->IsButtonPressed(0,CTRL_LEFT) | CellInput->IsAnalogPressedLeft(0,CTRL_LSTICK))
							{
								sys_timer_usleep(SETTINGS_DELAY);
							}
						   }
						   if(CellInput->WasButtonPressed(0, CTRL_CROSS))
						   {
							   if (Graphics->GetCurrentResolution() == CELL_VIDEO_OUT_RESOLUTION_576)
							   {
								   if(Graphics->CheckResolution(CELL_VIDEO_OUT_RESOLUTION_576))
								   {
									   Graphics->SetPAL60Hz(Settings.PS3PALTemporalMode60Hz);
									   Graphics->SwitchResolution(Graphics->GetCurrentResolution(), Settings.PS3PALTemporalMode60Hz);
								   }
							   }
							   else
							   {
								   Graphics->SetPAL60Hz(false);
								   Graphics->SwitchResolution(Graphics->GetCurrentResolution(), false);
							   }
						   }
						   if(CellInput->IsButtonPressed(0, CTRL_START))
						   {
							   Graphics->SwitchResolution(Graphics->GetInitialResolution(), Settings.PS3PALTemporalMode60Hz);
						   }
						   break;
/*
				case SETTING_PAL60_MODE:
						   if(CellInput->WasButtonPressed(0, CTRL_RIGHT) | CellInput->WasAnalogPressedLeft(0,CTRL_LSTICK) | CellInput->WasButtonPressed(0,CTRL_CROSS) | CellInput->WasButtonPressed(0, CTRL_LEFT) | CellInput->WasAnalogPressedLeft(0,CTRL_LSTICK))
						   {
							   if (Graphics->GetCurrentResolution() == CELL_VIDEO_OUT_RESOLUTION_576)
							   {
								   if(Graphics->CheckResolution(CELL_VIDEO_OUT_RESOLUTION_576))
								   {
									   Settings.PS3PALTemporalMode60Hz = !Settings.PS3PALTemporalMode60Hz;
									   Graphics->SetPAL60Hz(Settings.PS3PALTemporalMode60Hz);
									   Graphics->SwitchResolution(Graphics->GetCurrentResolution(), Settings.PS3PALTemporalMode60Hz);
								   }
							   }

						   }
						   break;
*/
				case SETTING_SHADER:
						if(CellInput->WasButtonPressed(0, CTRL_LEFT) | CellInput->WasAnalogPressedLeft(0, CTRL_LSTICK) | CellInput->WasButtonPressed(0, CTRL_RIGHT) | CellInput->WasAnalogPressedRight(0, CTRL_LSTICK) | CellInput->WasButtonPressed(0, CTRL_CROSS))
						{
							menuStack.push(do_shaderChoice);
							tmpBrowser = NULL;
						}
						if(CellInput->IsButtonPressed(0, CTRL_START))
						{
							Graphics->LoadFragmentShader(DEFAULT_SHADER_FILE);
						}
						break;
				case SETTING_FONT_SIZE:
					if(CellInput->WasButtonHeld(0, CTRL_LEFT) | CellInput->IsAnalogPressedLeft(0,CTRL_LSTICK) | CellInput->WasButtonPressed(0,CTRL_CROSS))
					{
						if(Settings.PS3FontSize > -100)
						{
							Settings.PS3FontSize--;
							if(CellInput->IsButtonPressed(0,CTRL_LEFT) | CellInput->IsAnalogPressedLeft(0,CTRL_LSTICK))
							{
								sys_timer_usleep(SETTINGS_DELAY);
							}
						}
					}
					if(CellInput->WasButtonHeld(0, CTRL_RIGHT) | CellInput->IsAnalogPressedRight(0,CTRL_LSTICK) | CellInput->WasButtonPressed(0,CTRL_CROSS))
					{
						if((Settings.PS3FontSize < 200))
						{
							Settings.PS3FontSize++;
							if(CellInput->IsButtonPressed(0,CTRL_RIGHT) | CellInput->IsAnalogPressedRight(0,CTRL_LSTICK))
							{
								sys_timer_usleep(SETTINGS_DELAY);
							}
						}
					}
					if(CellInput->IsButtonPressed(0, CTRL_START))
					{
						Settings.PS3FontSize = 100;
					}
					break;
				case SETTING_KEEP_ASPECT_RATIO:
					if(CellInput->WasButtonPressed(0, CTRL_LEFT) | CellInput->WasAnalogPressedLeft(0,CTRL_LSTICK) | CellInput->WasButtonPressed(0, CTRL_RIGHT) | CellInput->WasAnalogPressedRight(0,CTRL_LSTICK) | CellInput->WasButtonPressed(0,CTRL_CROSS))
					{
						Settings.PS3KeepAspect = !Settings.PS3KeepAspect;
						Graphics->SetAspectRatio(Settings.PS3KeepAspect);
					}
					if(CellInput->IsButtonPressed(0, CTRL_START))
					{
						Settings.PS3KeepAspect = true;
						Graphics->SetAspectRatio(Settings.PS3KeepAspect);
					}
					break;
				case SETTING_HW_TEXTURE_FILTER:
					if(CellInput->WasButtonPressed(0, CTRL_LEFT) | CellInput->WasAnalogPressedLeft(0,CTRL_LSTICK) | CellInput->WasButtonPressed(0, CTRL_RIGHT) | CellInput->WasAnalogPressedRight(0,CTRL_LSTICK) | CellInput->WasButtonPressed(0,CTRL_CROSS))
					{
						Settings.PS3Smooth = !Settings.PS3Smooth;
						Graphics->SetSmooth(Settings.PS3Smooth);
					}
					if(CellInput->IsButtonPressed(0, CTRL_START))
					{
						Settings.PS3Smooth = true;
						Graphics->SetSmooth(Settings.PS3Smooth);
					}
					break;
				case SETTING_HW_OVERSCAN_AMOUNT:
					if(CellInput->WasButtonHeld(0, CTRL_LEFT) | CellInput->IsAnalogPressedLeft(0,CTRL_LSTICK) | CellInput->WasButtonPressed(0,CTRL_CROSS))
					{
						if(Settings.PS3OverscanAmount > -40)
						{
							Settings.PS3OverscanAmount--;
							Settings.PS3OverscanEnabled = true;
							Graphics->SetOverscan(Settings.PS3OverscanEnabled, (float)Settings.PS3OverscanAmount/100);
							if(CellInput->IsButtonPressed(0,CTRL_LEFT) | CellInput->IsAnalogPressedLeft(0,CTRL_LSTICK))
							{
								sys_timer_usleep(SETTINGS_DELAY);
							}
						}
						if(Settings.PS3OverscanAmount == 0)
						{
							Settings.PS3OverscanEnabled = false;
							Graphics->SetOverscan(Settings.PS3OverscanEnabled, (float)Settings.PS3OverscanAmount/100);
							if(CellInput->IsButtonPressed(0,CTRL_LEFT) | CellInput->IsAnalogPressedLeft(0,CTRL_LSTICK))
							{
								sys_timer_usleep(SETTINGS_DELAY);
							}
						}
					}
					if(CellInput->WasButtonHeld(0, CTRL_RIGHT) | CellInput->IsAnalogPressedRight(0,CTRL_LSTICK) | CellInput->WasButtonPressed(0,CTRL_CROSS))
					{
						if((Settings.PS3OverscanAmount < 40))
						{
							Settings.PS3OverscanAmount++;
							Settings.PS3OverscanEnabled = true;
							Graphics->SetOverscan(Settings.PS3OverscanEnabled, (float)Settings.PS3OverscanAmount/100);
							if(CellInput->IsButtonPressed(0,CTRL_RIGHT) | CellInput->IsAnalogPressedRight(0,CTRL_LSTICK))
							{
								sys_timer_usleep(SETTINGS_DELAY);
							}
						}
						if(Settings.PS3OverscanAmount == 0)
						{
							Settings.PS3OverscanEnabled = false;
							Graphics->SetOverscan(Settings.PS3OverscanEnabled, (float)Settings.PS3OverscanAmount/100);
							if(CellInput->IsButtonPressed(0,CTRL_RIGHT) | CellInput->IsAnalogPressedRight(0,CTRL_LSTICK))
							{
								sys_timer_usleep(SETTINGS_DELAY);
							}
						}
					}
					if(CellInput->IsButtonPressed(0, CTRL_START))
					{
						Settings.PS3OverscanAmount = 0;
						Settings.PS3OverscanEnabled = false;
						Graphics->SetOverscan(Settings.PS3OverscanEnabled, (float)Settings.PS3OverscanAmount/100);
					}
					break;
				case SETTING_RSOUND_ENABLED:
					if(CellInput->WasButtonPressed(0, CTRL_LEFT) | CellInput->WasAnalogPressedLeft(0,CTRL_LSTICK) | CellInput->WasButtonPressed(0, CTRL_RIGHT) | CellInput->WasAnalogPressedRight(0,CTRL_LSTICK) | CellInput->WasButtonPressed(0,CTRL_CROSS))
					{
						Settings.RSoundEnabled = !Settings.RSoundEnabled;
						Emulator_Implementation_ToggleSound();
					}
					if(CellInput->IsButtonPressed(0, CTRL_START))
					{
						Settings.RSoundEnabled = false;
						Emulator_Implementation_ToggleSound();
					}
					break;
				case SETTING_RSOUND_SERVER_IP_ADDRESS:
					if(CellInput->WasButtonPressed(0, CTRL_LEFT) | CellInput->WasAnalogPressedLeft(0,CTRL_LSTICK) | CellInput->WasButtonPressed(0, CTRL_RIGHT) | CellInput->WasButtonPressed(0, CTRL_CROSS) | CellInput->WasAnalogPressedRight(0,CTRL_LSTICK))
					{
						Emulator_OSKStart(L"Enter IP address for the RSound Server. Example:", L"192.168.1.1");
						Settings.RSoundServerIPAddress = Emulator_OSKOutputString();
					}
					if(CellInput->IsButtonPressed(0, CTRL_START))
					{
						Settings.RSoundServerIPAddress = "0.0.0.0";
					}
					break;
				case SETTING_THROTTLE_MODE:
					if(CellInput->WasButtonPressed(0, CTRL_LEFT) | CellInput->WasAnalogPressedLeft(0,CTRL_LSTICK) | CellInput->WasButtonPressed(0, CTRL_RIGHT) | CellInput->WasButtonPressed(0, CTRL_CROSS) | CellInput->WasAnalogPressedRight(0,CTRL_LSTICK))
					{
						Settings.Throttled = !Settings.Throttled;
						Graphics->ToggleVSync(Settings.Throttled ? true : false);
					}
					if(CellInput->IsButtonPressed(0, CTRL_START))
					{
						Settings.Throttled = true;
						Graphics->ToggleVSync(Settings.Throttled);
					}
					break;
				case SETTING_DEFAULT_ALL:
					if(CellInput->WasButtonPressed(0, CTRL_LEFT) | CellInput->WasAnalogPressedLeft(0,CTRL_LSTICK) | CellInput->WasButtonPressed(0, CTRL_RIGHT) | CellInput->WasAnalogPressedRight(0,CTRL_LSTICK) | CellInput->IsButtonPressed(0, CTRL_START) | CellInput->WasButtonPressed(0, CTRL_CROSS))
					{
						Settings.PS3KeepAspect = true;
						Settings.PS3Smooth = true;
						Settings.RSoundEnabled = false;
						Settings.PS3OverscanEnabled = false;
						Settings.PS3OverscanAmount = 0;
						Settings.RSoundServerIPAddress = "0.0.0.0";
						Settings.PS3FontSize = 100;
						Graphics->SetAspectRatio(Settings.PS3KeepAspect);
						Graphics->SetSmooth(Settings.PS3Smooth);
						Graphics->SetOverscan(Settings.PS3OverscanEnabled, (float)Settings.PS3OverscanAmount/100);
						Emulator_Implementation_SetCurrentSaveStateSlot(0);
						Emulator_Implementation_ToggleSound();
						Settings.PS3PALTemporalMode60Hz = false;
						Graphics->SetPAL60Hz(Settings.PS3PALTemporalMode60Hz);
						Settings.Throttled = true;
						Graphics->ToggleVSync(Settings.Throttled);
					}
					break;
				default:
					break;
			} // end of switch
	}

	float yPos = 0.09;
	float ySpacing = 0.04;

	cellDbgFontPuts		(0.09f,	0.05f,	Emulator_GetFontSize(),	RED,	"GENERAL");
	cellDbgFontPuts		(0.27f,	0.05f,	Emulator_GetFontSize(),	GREEN,	"SNES9x");
	cellDbgFontPuts		(0.45f,	0.05f,	Emulator_GetFontSize(),	GREEN,	"PATH");
	cellDbgFontPuts		(0.63f, 0.05f,	Emulator_GetFontSize(), GREEN,	"CONTROLS"); 

	cellDbgFontPuts(0.09f, yPos, Emulator_GetFontSize(), currently_selected_setting == SETTING_CURRENT_SAVE_STATE_SLOT ? YELLOW : WHITE, "Current save state slot");
	cellDbgFontPrintf(0.5f, yPos, Emulator_GetFontSize(), Emulator_Implementation_CurrentSaveStateSlot() == 0 ? GREEN : ORANGE, "%d", Emulator_Implementation_CurrentSaveStateSlot());

	yPos += ySpacing;

	cellDbgFontPuts(0.09f, yPos, Emulator_GetFontSize(), currently_selected_setting == SETTING_CHANGE_RESOLUTION ? YELLOW : WHITE, "Resolution");

	switch(Graphics->GetCurrentResolution())
	{
		case CELL_VIDEO_OUT_RESOLUTION_480:
			cellDbgFontPrintf(0.5f, yPos, Emulator_GetFontSize(), Graphics->GetInitialResolution() == CELL_VIDEO_OUT_RESOLUTION_480 ? GREEN : ORANGE, "720x480 (480p)");
			break;
		case CELL_VIDEO_OUT_RESOLUTION_720:
			cellDbgFontPrintf(0.5f, yPos, Emulator_GetFontSize(), Graphics->GetInitialResolution() == CELL_VIDEO_OUT_RESOLUTION_720 ? GREEN : ORANGE, "1280x720 (720p)");
			break;
		case CELL_VIDEO_OUT_RESOLUTION_1080:
			cellDbgFontPrintf(0.5f, yPos, Emulator_GetFontSize(), Graphics->GetInitialResolution() == CELL_VIDEO_OUT_RESOLUTION_1080 ? GREEN : ORANGE, "1920x1080 (1080p)");
			break;
		case CELL_VIDEO_OUT_RESOLUTION_576:
			cellDbgFontPrintf(0.5f, yPos, Emulator_GetFontSize(), Graphics->GetInitialResolution() == CELL_VIDEO_OUT_RESOLUTION_576 ? GREEN : ORANGE, "720x576 (576p)");
			break;
		case CELL_VIDEO_OUT_RESOLUTION_1600x1080:
			cellDbgFontPrintf(0.5f, yPos, Emulator_GetFontSize(), Graphics->GetInitialResolution() == CELL_VIDEO_OUT_RESOLUTION_1600x1080 ? GREEN : ORANGE, "1600x1080");
			break;
		case CELL_VIDEO_OUT_RESOLUTION_1440x1080:
			cellDbgFontPrintf(0.5f, yPos, Emulator_GetFontSize(), Graphics->GetInitialResolution() == CELL_VIDEO_OUT_RESOLUTION_1440x1080 ? GREEN : ORANGE, "1440x1080");
			break;
		case CELL_VIDEO_OUT_RESOLUTION_1280x1080:
			cellDbgFontPrintf(0.5f, yPos, Emulator_GetFontSize(), Graphics->GetInitialResolution() == CELL_VIDEO_OUT_RESOLUTION_1280x1080 ? GREEN : ORANGE, "1280x1080");
			break;
		case CELL_VIDEO_OUT_RESOLUTION_960x1080:
			cellDbgFontPrintf(0.5f, yPos, Emulator_GetFontSize(), Graphics->GetInitialResolution() == CELL_VIDEO_OUT_RESOLUTION_960x1080 ? GREEN : ORANGE, "960x1080");
			break;
	}
	Graphics->FlushDbgFont();

/*
	yPos += ySpacing;
	cellDbgFontPuts		(0.09f,	yPos,	Emulator_GetFontSize(),	currently_selected_setting == SETTING_PAL60_MODE ? YELLOW : WHITE,	"PAL60 Mode (576p only)");
	cellDbgFontPrintf	(0.5f,	yPos,	Emulator_GetFontSize(),	Settings.PS3PALTemporalMode60Hz == true ? ORANGE : GREEN, Settings.PS3PALTemporalMode60Hz == true ? "ON" : "OFF");

*/
	yPos += ySpacing;
	cellDbgFontPuts(0.09f, yPos, Emulator_GetFontSize(), currently_selected_setting == SETTING_SHADER ? YELLOW : WHITE, "Selected shader");
	cellDbgFontPrintf(0.5f, yPos, Emulator_GetFontSize(), 
			GREEN, 
			"%s", Graphics->GetFragmentShaderPath().substr(Graphics->GetFragmentShaderPath().find_last_of('/')).c_str());

	yPos += ySpacing;
	cellDbgFontPuts(0.09f, yPos, Emulator_GetFontSize(), currently_selected_setting == SETTING_FONT_SIZE ? YELLOW : WHITE, "Font size");
	cellDbgFontPrintf(0.5f,	yPos,	Emulator_GetFontSize(),	Settings.PS3FontSize == 100 ? GREEN : ORANGE, "%f", Emulator_GetFontSize());

	yPos += ySpacing;
	cellDbgFontPuts(0.09f, yPos, Emulator_GetFontSize(), currently_selected_setting == SETTING_KEEP_ASPECT_RATIO ? YELLOW : WHITE, "Aspect Ratio");
	cellDbgFontPrintf(0.5f, yPos, Emulator_GetFontSize(), Settings.PS3KeepAspect == true ? GREEN : ORANGE, "%s", Settings.PS3KeepAspect == true ? "Scaled" : "Stretched");
	Graphics->FlushDbgFont();

	yPos += ySpacing;
	cellDbgFontPuts(0.09f, yPos, Emulator_GetFontSize(), currently_selected_setting == SETTING_HW_TEXTURE_FILTER ? YELLOW : WHITE, "Hardware Filtering");
	cellDbgFontPrintf(0.5f, yPos, Emulator_GetFontSize(), Settings.PS3Smooth == true ? GREEN : ORANGE,
			"%s", Settings.PS3Smooth == true ? "Linear interpolation" : "Point filtering");

	yPos += ySpacing;
	cellDbgFontPuts		(0.09f,	yPos,	Emulator_GetFontSize(),	currently_selected_setting == SETTING_HW_OVERSCAN_AMOUNT ? YELLOW : WHITE,	"Overscan");
	cellDbgFontPrintf	(0.5f,	yPos,	Emulator_GetFontSize(),	Settings.PS3OverscanAmount == 0 ? GREEN : ORANGE, "%f", (float)Settings.PS3OverscanAmount/100);


	yPos += ySpacing;
	cellDbgFontPuts(0.09f, yPos, Emulator_GetFontSize(), currently_selected_setting == SETTING_RSOUND_ENABLED ? YELLOW : WHITE, "Sound");
	cellDbgFontPuts(0.5f, yPos, Emulator_GetFontSize(), Settings.RSoundEnabled == false ? GREEN : ORANGE, Settings.RSoundEnabled == true ? "RSound" : "Normal");

	yPos += ySpacing;
	cellDbgFontPuts(0.09f, yPos, Emulator_GetFontSize(), currently_selected_setting == SETTING_RSOUND_SERVER_IP_ADDRESS ? YELLOW : WHITE, "RSound Server IP Address");
	cellDbgFontPuts(0.5f, yPos, Emulator_GetFontSize(), strcmp(Settings.RSoundServerIPAddress,"0.0.0.0") ? ORANGE : GREEN, Settings.RSoundServerIPAddress);

	yPos += ySpacing;
	cellDbgFontPuts(0.09f, yPos, Emulator_GetFontSize(), currently_selected_setting == SETTING_THROTTLE_MODE ? YELLOW : WHITE, "Throttle Mode");
	cellDbgFontPuts(0.5f, yPos, Emulator_GetFontSize(), Settings.Throttled == true ? GREEN : ORANGE, Settings.Throttled == true ? "ON" : "OFF");

	yPos += ySpacing;
	cellDbgFontPrintf(0.09f, yPos, Emulator_GetFontSize(), currently_selected_setting == SETTING_DEFAULT_ALL ? YELLOW : GREEN, "DEFAULT");
	Graphics->FlushDbgFont();

	DisplayHelpMessage(currently_selected_setting);

	cellDbgFontPuts(0.09f, 0.89f, Emulator_GetFontSize(), YELLOW,
	"UP/DOWN - select  L2+R2 - resume game   X/LEFT/RIGHT - change");
	cellDbgFontPuts(0.09f, 0.93f, Emulator_GetFontSize(), YELLOW,
	"START - default   L1/CIRCLE - go back   R1 - go forward");
	Graphics->FlushDbgFont();
}

void do_ROMMenu()
{
	string rom_path;

	if (CellInput->UpdateDevice(0) == CELL_PAD_OK)
	{
		UpdateBrowser(browser);

		if (CellInput->WasButtonPressed(0,CTRL_SELECT))
		{
			menuStack.push(do_general_settings);
		}

		if (CellInput->WasButtonPressed(0,CTRL_CROSS))
		{
			if(browser->IsCurrentADirectory())
			{
				browser->PushDirectory( browser->GetCurrentDirectoryInfo().dir + "/" + browser->GetCurrentEntry()->d_name,
						CELL_FS_TYPE_REGULAR | CELL_FS_TYPE_DIRECTORY,
						"smc|fig|sfc|gd3|swc|zip|jma|SMC|FIG|SFC|GD3|SWC|ZIP|JMA");
			}
			else if (browser->IsCurrentAFile())
			{
				// load game (standard controls), go back to main loop
				rom_path = browser->GetCurrentDirectoryInfo().dir + "/" + browser->GetCurrentEntry()->d_name;

				MenuStop();

				// switch emulator to emulate mode
				Emulator_StartROMRunning();

				Emulator_SetControllerMode(TWO_JOYSTICKS);

				//FIXME: 1x dirty const char* to char* casts... menu sucks.
				Emulator_RequestLoadROM((char*)rom_path.c_str(), true);

				return;
			}
		}
		if (CellInput->WasButtonPressed(0,CTRL_TRIANGLE))
		{
			if (browser->IsCurrentAFile())
			{
				// load game (standard controls), go back to main loop
				rom_path = browser->GetCurrentDirectoryInfo().dir + "/" + browser->GetCurrentEntry()->d_name;

				MenuStop();

				// switch emulator to emulate mode
				Emulator_StartROMRunning();

				Emulator_SetControllerMode(MULTITAP);

				//FIXME: 1x dirty const char* to char* casts... menu sucks.
				Emulator_RequestLoadROM((char*)rom_path.c_str(), true);

				return;
			}
		}
		if (CellInput->IsButtonPressed(0,CTRL_L2) && CellInput->IsButtonPressed(0,CTRL_R2))
		{
			// if a rom is loaded then resume it
			if (Emulator_IsROMLoaded())
			{
				MenuStop();

				Emulator_StartROMRunning();
			}
			
			return;
		}
	}

	if(browser->IsCurrentADirectory())
	{
		if(!strcmp(browser->GetCurrentEntry()->d_name,"app_home") || !strcmp(browser->GetCurrentEntry()->d_name,"host_root"))
		{
			cellDbgFontPrintf(0.09f, 0.90f, 0.91f, RED, "%s","WARNING - Do not open this directory, or you might have to restart!");
		}
		else if(!strcmp(browser->GetCurrentEntry()->d_name,".."))
		{
			cellDbgFontPrintf(0.09f, 0.90f, 0.91f, LIGHTBLUE, "%s","INFO - Press X to go back to the previous directory.");
		}
		else
		{
			cellDbgFontPrintf(0.09f, 0.90f, 0.91f, LIGHTBLUE, "%s","INFO - Press X to enter the directory.");
		}
	}
	if(browser->IsCurrentAFile())
	{
		cellDbgFontPrintf(0.09f, 0.90f, 0.91f, LIGHTBLUE, "%s", "INFO - Press X to load the game. ");
	}
	cellDbgFontPuts	(0.09f,	0.05f,	Emulator_GetFontSize(),	RED,	"FILE BROWSER");
	cellDbgFontPrintf(0.7f, 0.05f, 0.82f, WHITE, "SNES9x PS3 v%s", EMULATOR_VERSION);
	cellDbgFontPrintf(0.09f, 0.09f, Emulator_GetFontSize(), YELLOW, "PATH: %s", browser->GetCurrentDirectoryInfo().dir.c_str());
	cellDbgFontPuts(0.09f, 0.93f, Emulator_GetFontSize(), YELLOW,
	"L2 + R2 - resume game           SELECT - Settings screen");
	Graphics->FlushDbgFont();

	RenderBrowser(browser);
}

void MenuMainLoop()
{
	// create file browser->if null
	if (browser == NULL)
	{
		browser = new FileBrowser(Settings.PS3PathROMDirectory);
		browser->SetEntryWrap(false);
	}


	// FIXME: could always just return to last menu item... don't pop on resume kinda thing
	if (menuStack.empty())
	{
		menuStack.push(do_ROMMenu);
	}

	// menu loop
	menuRunning = true;
	while (!menuStack.empty() && menuRunning)
	{
		Graphics->Clear();

		menuStack.top()();

		Graphics->Swap();

		cellSysutilCheckCallback();
/* COMMENTED OUT FOR NOW
#ifdef EMUDEBUG
		if (CellConsole_IsInitialized())
		{
			cellConsolePoll();
		}
		*/
	}
}
